#ifndef BUSSIM_ARC_H_INCLUDED
#define BUSSIM_ARC_H_INCLUDED

// the following class template is used to store a list of blocks in which an arc is involved
class BUSSIM_ARC_IN_BLOCK {
public:
    class BUSSIM_ARC_IN_BLOCK *prev;
    class BUSSIM_ARC_IN_BLOCK *next;
    class BUSSIM_BLOCK *ref;

    BUSSIM_ARC_IN_BLOCK();
    ~BUSSIM_ARC_IN_BLOCK();
};

// the following class template is used to save the sequence of currently waiting vehicles at a node
// added by version 1.05
class BUSSIM_ARC_WAITING_VEHICLE {
public:
    int veh_id;
    class BUSSIM_ARC_WAITING_VEHICLE *next;
};


class BUSSIM_ARC_IN_LINE {
public:
    int LINE_ID;
    double x_offset;
    double y_offset;
    class BUSSIM_ARC_IN_LINE *next;
};

class BUSSIM_ARC {
public:
    int ID;                 // unique identifier
    int orig_node;          // refers to id of source BUSSIM_POINT of this arc
    int dest_node;          // refers to id of target BUSSIM_POINT of this arc

    int pos;                // only needed if arc is saved in line
    double length_screen;   // length on screen (scaled into [-1;1]x[-1;1] determined by Pythagoras

    double length;          // V3.00 to store the travel distance associated with this arc

    int type;               // defines in which direction this arc shows

    int vehicletype;        // V3.00: determines which kind of vehicle can use this arc (default = BUSSIM_ARCTYPE_ALL)
    int in_blocks;          // V3.00: used to store the number of blocks to which this arc belongs

    int ROUTING_TYPE;       // V3.00: BUSSIM_RT_VEHICLE (traversing by vehicle)
                            //      : BUSSIM_RT_WALK (traversing by walking)

    int SERVED_BY_REGULAR_LINE; // V3.00: =1 if at least one regular lines travels along this arc (otherwise and by default =0)
    class BUSSIM_ARC_IN_LINE *first_line;
    class BUSSIM_ARC_IN_LINE *last_line;

    class BUSSIM_ARC_IN_BLOCK *first_block; // added from version 3.00
    class BUSSIM_ARC_IN_BLOCK *last_block;  // added from version 3.00

    int lines_on_arc;

    double orig_screen_x;   // coordinates on screen
    double orig_screen_y;
    double dest_screen_x;
    double dest_screen_y;

    class BUSSIM_ARC_WAITING_VEHICLE *first_waiting_vehicle;    // added by version 1.05
    class BUSSIM_ARC_WAITING_VEHICLE *last_waiting_vehicle;    // added by version 1.05

    BUSSIM_ARC();
    BUSSIM_ARC(int ID, int _ORIGIN, int _DESTINATION, double _ORIG_SCREEN_X, double _ORIG_SCREEN_Y, double _DEST_SCREEN_X, double _DEST_SCREEN_Y);
    ~BUSSIM_ARC();

    void print_on_screen(class BUSSIM_NETWORK *_NET);
    void print_on_screen_no_heads(class BUSSIM_NETWORK *_NET);     // V3.00: print a non-transfer arc
    void print_transfer_arc(class BUSSIM_NETWORK *_NET);           // V3.00: print a transfer arc
    void print_queue_on_screen(class BUSSIM_NETWORK *_NET);
    void print_label(class BUSSIM_NETWORK *_NET);                  // V3.00: print the arc ID to the screen
    void update_length(class BUSSIM_NETWORK *_NET);
    void append_line(class BUSSIM_ARC_IN_LINE *_Ptr, class BUSSIM_NETWORK *_NET);
    void set_direction_type(void);
    // V1.05: appends the vehicle _VEH_ID to the local waiting queue
    void append_arrived_vehicle(int _VEH_ID);
    // V1.05 remove the first vehicle from the local waiting queue
    int remove_vehicle_from_queue(int _VEH_ID, class BUSSIM_NETWORK *_NET);
    // V3.00 connect arc to a block
    void add_block(class BUSSIM_BLOCK *RefToBlock);
    // V3.00 to decide if a vehicle can enter an arc
    int get_allowance(class BUSSIM_ARC *_Ptr2NextArc);
    // V3.00 used to allocate block capacity
    void check_in_vehicle(void);
    // V3.00 used to release block capacity
    void check_out_vehicle(void);
    // V3.00 checks if both involved nodes exists
    // ???
    // set the core arc attributes
    void specify(int _ID, int _ORIG_NODE, int _DEST_NODE);
    // sets the arc routing type
    void set_rt_type(int _RT_TYPE);
    // gets the arc routing type
    int get_rt_type(void);
};

#endif // BUSSIM_ARC_H_INCLUDED
