#ifndef BUSSIM_BLOCK_H_INCLUDED
#define BUSSIM_BLOCK_H_INCLUDED

#include "bussim_global.h"
#include <string>

// we use this object to group a collection of arcs that require
// a coordinated usage by vehicles

// added in version 3.0 - 03.11.2021

class BUSSIM_BLOCK_MEMBER {
    public:
    class BUSSIM_BLOCK_MEMBER *prev;
    class BUSSIM_BLOCK_MEMBER *next;
    class BUSSIM_ARC *arc;

    BUSSIM_BLOCK_MEMBER();
    ~BUSSIM_BLOCK_MEMBER();
};

class BUSSIM_BLOCK {
    public:
    int ID;         // a unique identifier
    int check_A;    // V3.00: used to check the entry allowance in the context of a block
    int check_B;    // V3.00: used to check the entry allowance in the context of a block
    int CAPACITY;   // maximal number of vehicles in this block at a time point
    int USED_CAP;   // current numbers of vehicles in this block
    std::string DESC;    // for a verbal description
    class BUSSIM_BLOCK_MEMBER *FIRST_MEMBER;   // reference to the first member of the list of contained BUSSIM_ARC-objects
    class BUSSIM_BLOCK_MEMBER *LAST_MEMBER;   // reference to the last member of the list of contained BUSSIM_ARC-objects
    double RED;
    double GREEN;
    double BLUE;

    BUSSIM_BLOCK();
    BUSSIM_BLOCK(int _ID, int _CAPACITY, std::string _DESC, double _RED, double _GREEN, double _BLUE);
    ~BUSSIM_BLOCK();

    void add_arc(class BUSSIM_ARC *ArcRef);
    void check_in_vehicle(void);
    void check_out_vehicle(void);
    void specify(int _ID, int _CAPACITY, std::string _DESC, double _RED, double _GREEN, double _BLUE);

};

#endif // BUSSIM_BLOCK_H_INCLUDED
