#ifndef BUSSIM_NETWORK_H_INCLUDED
#define BUSSIM_NETWORK_H_INCLUDED

#include "bussim_global.h"
//#include "bussim_point.h"
//#include "bussim_arc.h"
//#include "bussim_line.h"
//#include "bussim_vehicle.h"
//#include "bussim_event.h"

struct BUSSIM_VP_MODIFICATION {             // used to store viewport modification steps
    int type;
    struct BUSSIM_VP_MODIFICATION *next;

};

class BUSSIM_NETWORK {
public:
    int POINTS;             // number of network points
    int BUNCHPOINTS;        // number of saved bunching points
    int ARCS;               // number of network arcs
    int TOIS;               // number of things of interest (added in version 1.05)
    int LINES;              // number of lines in the network
    int EXT_LINES;          // number of user-specified lines (added in version 3.00)

    int VEHICLES;           // number of vehicles in the network
    int ACTIVE_VEHICLES;    // number of active vehicles
    int WAITING_VEHICLES;   // number of waiting vehicles
    int BLOCKS;             // number of blocks in the infrastructure // added in version 3.0
    int SIMULATION_COMPLETE;    // added in version 3.0 used to display the last message to the user
    long EVENTS;         // number of stored events
    long DUTIES;         // number of duties
    long BLOCK_WAIT;     // number of events when a vehicle is blocked by an occupied block
    int ROTATIONS;       // number of rotations
    int STOPS;              // number of stops

    long SEGMENT_CNTR;  // unique segment identifier;

    int PAXS;           // added in 3.00: number of paxes in this simulation

    double xmin;        // needed to scale everything into [-1;-1] x [-1;1]
    double xmax;
    double ymin;
    double ymax;

    double x_factor;    // needed to maintain an undistored picture of the network
    double y_factor;

    double SPEEDUP_FACTOR;  // speeding up the simulation in automatic mode
    double CURRENT_TIME;    // the current time stamp
    double TIME_STEP;

    int BETRIEBS_ENDE;      // 0 or 1: if 1 all vehicles will be deactived as soon as they reach the terminus of their current line

    int DETECTED_BUNCHES;   // number of requested takeover manoevres

    int LAST_HOURS;         // added in version 3.00 to save the last simulation time
    int LAST_MINUTES;

    class BUSSIM_POINT *PT;     // array of network points
    class BUSSIM_ARC *ARC;      // array of network arcs
    class BUSSIM_TOI *TOI;      // array of network tois    added in version 1.05
    class BUSSIM_LINE *LINE;  // array of lines in the network
    class BUSSIM_VEHICLE *VEHICLE;  // array of vehicles in the network

    class BUSSIM_POINT *BP;     // array of detected bunching points

    class BUSSIM_BLOCK *BLOCK;  // array to host infrastructure blocks // added in version 3.0

    class BUSSIM_EVENT *first_event;    // used to store all events detected during the simulation
    class BUSSIM_EVENT *last_event;

    class BUSSIM_BUNCHPOINT *first_bp;       // used to store all events detected during the simulation
    class BUSSIM_BUNCHPOINT *last_bp;

    class BUSSIM_STOP *STOP;        // used to store the stops in the simulation

    int **PRED;                 // sequencing of pairs of vehicles

    GLuint BGTextureId;         // TextureID of the Background

    double center_longitude;    // center longitude position
    double center_latitude;     // center latitude position

    double max_long;            // extreme coodinates
    double min_long;
    double max_lat;
    double min_lat;

    int Current_Zoom_Level;
    int Current_MENUE_OFFSET_X;
    int Current_MENUE_OFFSET_Y;

    struct BUSSIM_VP_MODIFICATION *FIRST_VP_LIST;  // 3:00 - to store viewport modifications
    struct BUSSIM_VP_MODIFICATION *LAST_VP_LIST;
    int Stored_VP_Move_Items;

    class BUSSIM_PAX_LIST *PAX_UNKNOWN;             // 3.00 - to manage the passengers
    class BUSSIM_PAX_LIST *PAX_IN;
    class BUSSIM_PAX_LIST *PAX_ARRIVED;

    class BUSSIM_PAX_LIST *PAX_WALKING;    // 3.0 - list of all pax currently walking

    int ***SERVICE;

    class BUSSIM_NODELIST *PLATFORM_LIST;           // 3.00 - to store refs to the nodes
                                                    // where passengers can (de)-board a vehicle

    long NUM_PAX_IN_WAITING;                            // 3.00 pax num currently waiting at platforms

    BUSSIM_NETWORK();   // this is the default constructor
    BUSSIM_NETWORK(int _STOPS, int _POINTS, int _TOIS, int _ARCS, int _BLOCKS, int _LINES, int _VEHICLES); // this is an overloaded constructor
    BUSSIM_NETWORK(int _STOPS, int _POINTS, int _TOIS, int _ARCS, int _BLOCKS, int _LINES, int _VEHICLES, int _PAXS); // this is an overloaded constructor

    ~BUSSIM_NETWORK();  // this is the default constructor

    void specify(void); // use it to define and prepare the network
    void specify_nodes(void);
    void specify_nodes_longlat(void); // added in version 1.05
    void specify_arcs(void);
    void specify_tois(void);            // added in version 1.05
    void specify_lines(void);

    void specify_lines_intern(void);    // added in version 3.00

    void specify_vehicles(void);
    void specify_vehicle_activation_times(void);    // added in version 3.00

    void specify_rotations(void);
    void specify_rotations_deployment_lines(void); // added in version 3.00
    void specify_rotations_parking_lines(void); // added in version 3.00

    void specify_blocks(void);          // added in version 3.0
    void display_active_vehicles(void);
    void display_active_vehicles_orig(void);
    void start_rotations(void);
    void start_schedules(void);         // added in version 3.0 for the schedules
    void append_bunch_point(class BUSSIM_BUNCHPOINT *_POINT);
    void print_bunch_points(void);
    void append_event(class BUSSIM_EVENT *_EVENT);
    void print_event_list(void);
    void print_stop_list(void);
    int number_of_active_vehicles(void);
    void center_network(void);

    void specify_schedules(void);       // by version 3.00

    void merge_lines_arcs(void);

    void set_visibility(void);
    void draw_depots(class BUSSIM_NETWORK *_NET);
    void draw_service_arcs(void);
    //void draw_poi(void);

    void append_duty_to_vehicle_rotation(int _VEH_ID, int _TRIP_ID);
    void longlat_to_worldxy(void);      // added in version 1.05
    void draw_tois(void);               // added in version 1.05

    void save_protocol(void);           // added in version 3.00
    void specify_stops(void);           // added in version 3.00

    void set_stop_colors(void);         // added in version 3.00

    class BUSSIM_ARC get_arc_by_od(int _ORIGIN_NODE_ID, int _DESTINATION_NODE_ID);      // added in version 3.00

    void append_vp_move(int _TYPE);     // add a vp move item
    void clear_vp_move_list(void);
    void save_vp_move_list(void);
    void load_vp_move_list(void);

    // needed to connect arcs and stops
    void attach_internal_arcs_to_stops(void);       // creates a list of internal arcs to each stop

    // needed to make the initial vehicle positions feasible (if possible)
    void inspect_initial_vehicle_positions(void);
    void simple_dijkstra_at_stop(int _START_NODE, int _TARGET_NODE, class BUSSIM_LINE *_PTR_LINE);

    // needed to organizing the handover to a RSC
    void inspect_final_vehicle_positions(void);

    // consistency check procedures
    void check_stop_consistency(void);   // checks if the N stops are numbered by 0,1,...,N-1
    void check_node_consistency(void);  // checks if the N nodes are numbered by 0,1,...,N-1
    void check_stop_and_node_consistency(void); // checks the correct interrelations between stops and nodes
    void check_arc_consistency(void);

    // specify the initial pax lists
    void specify_paxlists(void);    // added in version 3.00

    // update appearance state of passengers
    void update_paxlists(void);

    // update position of all walking pax
    void update_walking_paxes(void);

    // construct the transfer connections
    void specify_transfer_arcs(void);

    // compile the service matrix
    void compile_service_matrix(void);

    // this procedure is used to fill the passenger window with data
    void fill_passenger_window(void);

    // set the platform positions within line
    void specify_platform_positions_in_lines(void);

    // set the automatic terminus labels of the lines
    void specify_automatic_terminus_labels_of_lines(void);
};

// this is the node list definition
// we use the BUSSIM_NODELIST-class to manage lists of references to BUSSIM_POINT-OBJECTs

class BUSSIM_NODELIST {
    std::string DESC;
    long NODES;
    class BUSSIM_NODELIST_ENTRY *REF2FIRST;
    class BUSSIM_NODELIST_ENTRY *REF2LAST;
public:
    BUSSIM_NODELIST();      // the standard-constructor
    BUSSIM_NODELIST(std::string _DESC);  // an overloaded constructor
    void append_node(class BUSSIM_NODELIST_ENTRY *_NEW_NODELIST_ENTRY);
    void print_to_shell(void);
    long get_random_node_id(void);
    // returns an array of different nodes
    void get_random_nodes_id(long _NUM, long *_NUMARRAY);
    ~BUSSIM_NODELIST();      // the standard-destructor
};

class BUSSIM_NODELIST_ENTRY {

    friend class BUSSIM_NODELIST;
    class BUSSIM_POINT *REF2POINT;
    class BUSSIM_NODELIST_ENTRY *previous;
    class BUSSIM_NODELIST_ENTRY *next;
    double SortValue;
public:
    BUSSIM_NODELIST_ENTRY();
    BUSSIM_NODELIST_ENTRY(class BUSSIM_POINT *_REF2POINT);
    BUSSIM_NODELIST_ENTRY(class BUSSIM_POINT *_REF2POINT, double _SortValue);
    ~BUSSIM_NODELIST_ENTRY();
};

#endif // BUSSIM_NETWORK_H_INCLUDED
