//#include "bussim_point.h"
//#include "bussim_network.h"
#include "bussim_global.h"

#include <math.h>           // needed to draw the circle
#include <iostream>
#include <stdio.h>
#include <string.h>

//#include <GL/gl.h>
//#include <freeglut/freeglut.h>

BUSSIM_POINT::BUSSIM_POINT()
{
    // this is the source code of the standard constructor
    // it sets both coordinates to -1

    this->ID = -1;
    this->SHORTNAME="n/a";
    this->label_pos = BUSSIM_DEFAULT_LABEL_POS;
    this->world_x = -1.0;
    this->world_y = -1.0;
    this->screen_x = -1.0;
    this->screen_y = -1.0;

    this->world_longitude = 0;  // added in version 1.05
    this->world_latitude = 0;  // added in version 1.05

    this->type = BUSSIM_NODE_LINE;

    this->INFRASTRUCTURE = BUSSIM_DEFAULT_INFRASTRUCTURE_POINT;

    this->INFRATYPE = BUSSIM_INFRA_UNDEF;

    this->STOPNAME = "STOPNAME";
    this->SHORTNAME = "SHORTNAME";

    this->default_waiting_time = BUSSIM_MEAN_WAITING_TIME;

    this->IS_POSITIONED = 0;

    this->ref2stop = nullptr;

    this->first_outbound_arc = nullptr;
    this->last_outbound_arc = nullptr;

    this->WAITINGLIST = nullptr;

    this->PlatformPosInLine = nullptr;
}

BUSSIM_POINT::BUSSIM_POINT(double _X, double _Y)
{
    // this is the source code of a first overloaded constructor
    // it sets the coordinates to the specified values

    this->ID = -1;
    this->SHORTNAME = "n/a";
    this->label_pos = BUSSIM_DEFAULT_LABEL_POS;
    this->world_x = _X;
    this->world_y = _Y;
    this->screen_x = -1.0;
    this->screen_y = -1.0;
    this->type = BUSSIM_NODE_LINE;

    this->INFRASTRUCTURE = BUSSIM_DEFAULT_INFRASTRUCTURE_POINT;
    this->INFRATYPE = BUSSIM_INFRA_UNDEF;

    this->STOPNAME = "STOPNAME";
    this->SHORTNAME = "SHORTNAME";

    this->default_waiting_time = BUSSIM_MEAN_WAITING_TIME;

    this->IS_POSITIONED = 0;

    this->ref2stop = nullptr;

    this->first_outbound_arc = nullptr;
    this->last_outbound_arc = nullptr;

    this->WAITINGLIST = nullptr;

    this->PlatformPosInLine = nullptr;
}

BUSSIM_POINT::~BUSSIM_POINT()
{
    // this is the source code of the standard destructor
    // it does nothing (but should be there)
}

void BUSSIM_POINT::calculate_screen_coordinates(class BUSSIM_NETWORK *_NET, double _xmin, double _xmax, double _ymin, double _ymax)
{
    if( (_ymax - _ymin) > (_xmax - _xmin) )
    {
        _NET->y_factor = 1;
        _NET->x_factor = (_xmax - _xmin) / (_ymax - _ymin);
    }
    else
    {
        _NET->x_factor = 1;
        _NET->y_factor = (_ymax - _ymin) / (_xmax - _xmin);
    }

    this->screen_x = -1 + (this->world_x-_xmin)/(_xmax-_xmin)*2*_NET->x_factor;
    this->screen_y = -1 + (this->world_y-_ymin)/(_ymax-_ymin)*2*_NET->y_factor;

}

void BUSSIM_POINT::print_on_screen(void)
{
  double r = 0.030;
    int num_segments = 1000;

    glBegin(GL_LINE_LOOP);
    //glColor4f(0.0f, 1.0f, 0.0f, 0.0f);
    for(int ii = 0; ii < num_segments; ii++)
    {
        float theta = 2.0f * 3.1415926f * float(ii) / float(num_segments);//get the current angle

        float x = r * cosf(theta);//calculate the x component
        float y = r * sinf(theta);//calculate the y component

        glVertex2f(this->screen_x, this->screen_y);
        glVertex2f(x + this->screen_x, y + this->screen_y);//output vertex

    }
    glEnd();

}

void BUSSIM_POINT::print_on_screen_small(class BUSSIM_NETWORK *_NET)
{
    double r = 0.01;
    double f;
    f = 1.0 / (double)(pow(1.05,_NET->Current_Zoom_Level));
    double PI = 3.14159265358979323846;
    if( this->INFRASTRUCTURE < 1 )
    {
        if( this->type == BUSSIM_REGULARLINE )
        {
            if( this->ref2stop != nullptr )
                glColor3d((this->ref2stop->RED), (this->ref2stop->GREEN), (this->ref2stop->BLUE));

            glBegin(GL_POLYGON);
                glVertex2f(this->screen_x - r*f, this->screen_y + r*f);
                glVertex2f(this->screen_x + r*f, this->screen_y + r*f);
                glVertex2f(this->screen_x + r*f, this->screen_y - r*f);
                glVertex2f(this->screen_x - r*f, this->screen_y - r*f);
                glVertex2f(this->screen_x - r*f, this->screen_y + r*f);
            glEnd();

        }
        else
        {
            if( this->ref2stop != nullptr )
                glColor3d((this->ref2stop->RED), (this->ref2stop->GREEN), (this->ref2stop->BLUE));

            glLineWidth(3.0);
            glBegin(GL_LINE_LOOP);
                glVertex2f(this->screen_x - r*f, this->screen_y + r*f);
                glVertex2f(this->screen_x + r*f, this->screen_y + r*f);
                glVertex2f(this->screen_x + r*f, this->screen_y - r*f);
                glVertex2f(this->screen_x - r*f, this->screen_y - r*f);
                glVertex2f(this->screen_x - r*f, this->screen_y + r*f);
            glEnd();

            glColor3d(BUSSIM_BGCOLOR_MAIN_RED, BUSSIM_BGCOLOR_MAIN_GREEN, BUSSIM_BGCOLOR_MAIN_BLUE);
            glBegin(GL_POLYGON);
                glVertex2f(this->screen_x - r*f*0.5, this->screen_y + r*f*0.5);
                glVertex2f(this->screen_x + r*f*0.5, this->screen_y + r*f*0.5);
                glVertex2f(this->screen_x + r*f*0.5, this->screen_y - r*f*0.5);
                glVertex2f(this->screen_x - r*f*0.5, this->screen_y - r*f*0.5);
                glVertex2f(this->screen_x - r*f*0.5, this->screen_y + r*f*0.5);
            glEnd();

        }
    }
    else
    {
        if( this->INFRATYPE == BUSSIM_INFRA_SWITCH )
        {
            glColor3f(BUSSIM_NODE_DEFAULT_COLOR_RED,
                      BUSSIM_NODE_DEFAULT_COLOR_GREEN,
                      BUSSIM_NODE_DEFAULT_COLOR_BLUE);
            glBegin(GL_POLYGON);
                glVertex2f(this->screen_x + r*f, this->screen_y);
                glVertex2f(this->screen_x + cos(5.0/4.0*PI)*r*f, this->screen_y - cos(5.0/4.0*PI)*r*f);
                glVertex2f(this->screen_x + cos(5.0/4.0*PI)*r*f, this->screen_y + cos(5.0/4.0*PI)*r*f);
                glVertex2f(this->screen_x + r*f, this->screen_y);
            glEnd();
        }
        if( this->INFRATYPE == BUSSIM_INFRA_TRACKPOS )
        {
            glColor3f(BUSSIM_NODE_DEFAULT_COLOR_RED,
                      BUSSIM_NODE_DEFAULT_COLOR_GREEN,
                      BUSSIM_NODE_DEFAULT_COLOR_BLUE);

            glBegin(GL_POLYGON);
                glVertex2f(this->screen_x + sin(1.0/8.0*PI)*r*f, this->screen_y + cos(1.0/8.0*PI)*r*f);
                glVertex2f(this->screen_x + sin(3.0/8.0*PI)*r*f, this->screen_y + cos(3.0/8.0*PI)*r*f);
                glVertex2f(this->screen_x + sin(5.0/8.0*PI)*r*f, this->screen_y + cos(5.0/8.0*PI)*r*f);
                glVertex2f(this->screen_x + sin(7.0/8.0*PI)*r*f, this->screen_y + cos(7.0/8.0*PI)*r*f);
                glVertex2f(this->screen_x + sin(9.0/8.0*PI)*r*f, this->screen_y + cos(9.0/8.0*PI)*r*f);
                glVertex2f(this->screen_x + sin(11.0/8.0*PI)*r*f, this->screen_y + cos(11.0/8.0*PI)*r*f);
                glVertex2f(this->screen_x + sin(13.0/8.0*PI)*r*f, this->screen_y + cos(13.0/8.0*PI)*r*f);
                glVertex2f(this->screen_x + sin(15.0/8.0*PI)*r*f, this->screen_y + cos(15.0/8.0*PI)*r*f);
            glEnd();
        }
    }
}

void BUSSIM_POINT::print_label_on_screen(class BUSSIM_NETWORK *_NET)
{
    if( 1 < 2)
    {
    double r;
    //r = pow(2.0*BUSSIM_STOP_LABEL_OFFSET,2.0);      // specify the distance
    r = 1.0 / (double)(pow(1.05,_NET->Current_Zoom_Level));
    double f = 1.0;
    f = 1.0 / sqrt( 2.0 );

    if( this->INFRASTRUCTURE < 1 )
    {
        if( this->ref2stop != nullptr )
            glColor3d((this->ref2stop->RED), (this->ref2stop->GREEN), (this->ref2stop->BLUE));
    }
    else
    {
        //glColor3d(0.0f, 1.0f, 0.0f);
        glColor3d((BUSSIM_BGCOLOR_MAIN_RED), (BUSSIM_BGCOLOR_MAIN_GREEN), (BUSSIM_BGCOLOR_MAIN_BLUE));
    }
    if( this->label_pos == 0 )
        glRasterPos2d(this->screen_x + 0*r * (BUSSIM_STOP_LABEL_OFFSET-0.01)-r*0.025, this->screen_y + 1.0*r * BUSSIM_STOP_LABEL_OFFSET);
    if( this->label_pos == 1 )
        glRasterPos2d(this->screen_x + f*r * BUSSIM_STOP_LABEL_OFFSET, this->screen_y + f*r * BUSSIM_STOP_LABEL_OFFSET);
    if( this->label_pos == 2 )
        glRasterPos2d(this->screen_x + 1.0*r * BUSSIM_STOP_LABEL_OFFSET, this->screen_y-0.0*r * BUSSIM_STOP_LABEL_OFFSET+r*0.01);
    if( this->label_pos == 3 )
        glRasterPos2d(this->screen_x+f*r * BUSSIM_STOP_LABEL_OFFSET, this->screen_y-f*r * (BUSSIM_STOP_LABEL_OFFSET+0.025));
    if( this->label_pos == 4 )
        glRasterPos2d(this->screen_x-0.0*r * (BUSSIM_STOP_LABEL_OFFSET) -r*0.05, this->screen_y-r * (BUSSIM_STOP_LABEL_OFFSET+0.025));
    if( this->label_pos == 5 )
        glRasterPos2d(this->screen_x-f*r * (BUSSIM_STOP_LABEL_OFFSET+0.08), this->screen_y-f*r * (BUSSIM_STOP_LABEL_OFFSET+0.025));
    if( this->label_pos == 6 )
        glRasterPos2d(this->screen_x-r * (BUSSIM_STOP_LABEL_OFFSET+0.05)-r*0.05, this->screen_y+0.0*r * (BUSSIM_STOP_LABEL_OFFSET+0.05));
    if( this->label_pos == 7 )
        glRasterPos2d(this->screen_x-f*r * (BUSSIM_STOP_LABEL_OFFSET+0.05), this->screen_y+f*r * BUSSIM_STOP_LABEL_OFFSET);

    //glRasterPos2d(this->screen_x+r * BUSSIM_STOP_LABEL_OFFSET, this->screen_y+r * BUSSIM_STOP_LABEL_OFFSET);

    char S[256];
    sprintf(S,"%s(%d)",this->SHORTNAME.c_str(),this->ID);

    unsigned char* ubuf = reinterpret_cast<unsigned char*>(S);

    glutBitmapString(GLUT_BITMAP_HELVETICA_12, ubuf);
    //glColor4f(1.0f, 0.0f, 0.0f, 0.0f);
    }
}

void BUSSIM_POINT::print_label_on_screen_short(class BUSSIM_NETWORK *_NET)
{
    if( 1 < 2)
    {
    double r;
    //r = pow(2.0*BUSSIM_STOP_LABEL_OFFSET,2.0);      // specify the distance
    r = 1.0 / (double)(pow(1.05,_NET->Current_Zoom_Level));
    double f = 1.0;
    f = 1.0 / sqrt( 2.0 );

    if( this->INFRASTRUCTURE < 1 )
    {
        if( this->ref2stop != nullptr )
            glColor3d((this->ref2stop->RED), (this->ref2stop->GREEN), (this->ref2stop->BLUE));
    }
    else
    {
        //glColor3d(0.0f, 1.0f, 0.0f);
        if( this->ref2stop != nullptr )
            glColor3d((BUSSIM_BGCOLOR_MAIN_RED), (BUSSIM_BGCOLOR_MAIN_GREEN), (BUSSIM_BGCOLOR_MAIN_BLUE));
    }
    if( this->label_pos == 0 )
        glRasterPos2d(this->screen_x + 0*r * (BUSSIM_STOP_LABEL_OFFSET-0.01)-r*0.025, this->screen_y + 1.0*r * BUSSIM_STOP_LABEL_OFFSET);
    if( this->label_pos == 1 )
        glRasterPos2d(this->screen_x + f*r * BUSSIM_STOP_LABEL_OFFSET, this->screen_y + f*r * BUSSIM_STOP_LABEL_OFFSET);
    if( this->label_pos == 2 )
        glRasterPos2d(this->screen_x + 1.0*r * BUSSIM_STOP_LABEL_OFFSET, this->screen_y-0.0*r * BUSSIM_STOP_LABEL_OFFSET+r*0.01);
    if( this->label_pos == 3 )
        glRasterPos2d(this->screen_x+f*r * BUSSIM_STOP_LABEL_OFFSET, this->screen_y-f*r * (BUSSIM_STOP_LABEL_OFFSET+0.025));
    if( this->label_pos == 4 )
        glRasterPos2d(this->screen_x-0.0*r * (BUSSIM_STOP_LABEL_OFFSET) -r*0.05, this->screen_y-r * (BUSSIM_STOP_LABEL_OFFSET+0.025));
    if( this->label_pos == 5 )
        glRasterPos2d(this->screen_x-f*r * (BUSSIM_STOP_LABEL_OFFSET+0.08), this->screen_y-f*r * (BUSSIM_STOP_LABEL_OFFSET+0.025));
    if( this->label_pos == 6 )
        glRasterPos2d(this->screen_x-r * (BUSSIM_STOP_LABEL_OFFSET+0.05)-r*0.05, this->screen_y+0.0*r * (BUSSIM_STOP_LABEL_OFFSET+0.05));
    if( this->label_pos == 7 )
        glRasterPos2d(this->screen_x-f*r * (BUSSIM_STOP_LABEL_OFFSET+0.05), this->screen_y+f*r * BUSSIM_STOP_LABEL_OFFSET);

    //glRasterPos2d(this->screen_x+r * BUSSIM_STOP_LABEL_OFFSET, this->screen_y+r * BUSSIM_STOP_LABEL_OFFSET);

    char S[256];
    sprintf(S,"%d",this->ID);

    unsigned char* ubuf = reinterpret_cast<unsigned char*>(S);

    glutBitmapString(GLUT_BITMAP_HELVETICA_12, ubuf);
    //glColor4f(1.0f, 0.0f, 0.0f, 0.0f);
    }
}

void BUSSIM_POINT::set_as_infrastructure_point(void)
{
    this->INFRASTRUCTURE = 1;
}

void BUSSIM_POINT::unset_as_infrastructure_point(void)
{
    this->INFRASTRUCTURE = 0;
}

void BUSSIM_POINT::specify_regular(double _LONG, double _LAT, int _STOP_ID, class BUSSIM_NETWORK *_NET)
{

    this->world_longitude=_LONG;
    this->world_latitude=_LAT;
    this->INFRASTRUCTURE = 0;

    // connect to the stop
    if( _STOP_ID >= 0 )
    {
        _NET->STOP[_STOP_ID].append_point(this->ID);
        this->STOPNAME = _NET->STOP[_STOP_ID].STOP_NAME;
        this->SHORTNAME = _NET->STOP[_STOP_ID].STOP_SHORT_NAME;
        this->ref2stop = &_NET->STOP[_STOP_ID];
    }
    this->type = BUSSIM_NODE_LINE;

    // add to the platform list
    class BUSSIM_NODELIST_ENTRY *TmpRef;
    TmpRef = new class BUSSIM_NODELIST_ENTRY(this);
    _NET->PLATFORM_LIST->append_node(TmpRef);
    TmpRef = nullptr;

    // install the (empty) passenger waiting lists
    this->WAITINGLIST = new class BUSSIM_PAX_LIST* [_NET->EXT_LINES + 1];
    for( long i=0 ; i < (_NET->EXT_LINES + 1) ; i ++ )
        if( i < (_NET->EXT_LINES) )
            this->WAITINGLIST[i] = new class BUSSIM_PAX_LIST("WL platform" + std::to_string(this->ID) + " / line " + std::to_string(i));
        else
            this->WAITINGLIST[i] = new class BUSSIM_PAX_LIST("WL platform" + std::to_string(this->ID) + " / walking");
}
/*
void BUSSIM_POINT::specify_infra(double _LONG, double _LAT, std::string _STOPNAME, std::string _SHORTNAME)
{
    this->world_longitude=_LONG;
    this->world_latitude=_LAT;
    this->STOPNAME=_STOPNAME;
    this->SHORTNAME=_SHORTNAME;
    this->INFRASTRUCTURE = 1;
}
*/
void BUSSIM_POINT::specify_switch(double _LONG, double _LAT, int _STOP_ID, std::string _STOPNAME, std::string _SHORTNAME, class BUSSIM_NETWORK *_NET)
{
    this->world_longitude=_LONG;
    this->world_latitude=_LAT;
    this->STOPNAME=_STOPNAME;
    this->SHORTNAME=_SHORTNAME;
    this->INFRASTRUCTURE = 1;
    this->INFRATYPE = BUSSIM_INFRA_SWITCH;

    // connect to stop
    if( _STOP_ID >= 0 )
    {
        _NET->STOP[_STOP_ID].append_point(this->ID);
        this->ref2stop = &_NET->STOP[_STOP_ID];
    }

    this->type = BUSSIM_NODE_SERVICE;
}

void BUSSIM_POINT::specify_trackpos(double _LONG, double _LAT, int _STOP_ID, std::string _STOPNAME, std::string _SHORTNAME, class BUSSIM_NETWORK *_NET)
{
    this->world_longitude=_LONG;
    this->world_latitude=_LAT;
    this->STOPNAME=_STOPNAME;
    this->SHORTNAME=_SHORTNAME;
    this->INFRASTRUCTURE = 1;
    this->INFRATYPE = BUSSIM_INFRA_TRACKPOS;

    // connect to stop
    if( _STOP_ID >= 0 )
    {
        _NET->STOP[_STOP_ID].append_point(this->ID);
        this->ref2stop = &_NET->STOP[_STOP_ID];
    }

    this->type = BUSSIM_NODE_SERVICE;
}

void BUSSIM_POINT::specify_depot(double _LONG, double _LAT, int _STOP_ID, class BUSSIM_NETWORK *_NET)
{

    this->world_longitude=_LONG;
    this->world_latitude=_LAT;
    this->INFRASTRUCTURE = 0;

    // connect to the stop
    _NET->STOP[_STOP_ID].append_point(this->ID);
    this->STOPNAME = _NET->STOP[_STOP_ID].STOP_NAME;
    this->SHORTNAME = _NET->STOP[_STOP_ID].STOP_SHORT_NAME;
    this->ref2stop = &_NET->STOP[_STOP_ID];

    this->type = BUSSIM_NODE_SERVICE;

    // add to the platform list
    class BUSSIM_NODELIST_ENTRY *TmpRef;
    TmpRef = new class BUSSIM_NODELIST_ENTRY(this);
    _NET->PLATFORM_LIST->append_node(TmpRef);
    TmpRef = nullptr;

    // install the (empty) passenger waiting lists
    this->WAITINGLIST = new class BUSSIM_PAX_LIST* [_NET->EXT_LINES + 1];
    for( long i=0 ; i < (_NET->EXT_LINES + 1) ; i ++ )
        if( i < (_NET->EXT_LINES) )
            this->WAITINGLIST[i] = new class BUSSIM_PAX_LIST("WL platform" + std::to_string(this->ID) + " / line " + std::to_string(i));
        else
            this->WAITINGLIST[i] = new class BUSSIM_PAX_LIST("WL platform" + std::to_string(this->ID) + " / walking");
}

void BUSSIM_POINT::specify_siding(double _LONG, double _LAT, int _STOP_ID, class BUSSIM_NETWORK *_NET)
{
    // the same as depot but not added to platform list
    this->world_longitude=_LONG;
    this->world_latitude=_LAT;
    this->INFRASTRUCTURE = 0;

    // connect to the stop
    _NET->STOP[_STOP_ID].append_point(this->ID);
    this->STOPNAME = _NET->STOP[_STOP_ID].STOP_NAME;
    this->SHORTNAME = _NET->STOP[_STOP_ID].STOP_SHORT_NAME;
    this->ref2stop = &_NET->STOP[_STOP_ID];

    this->type = BUSSIM_NODE_SERVICE;

    // add to the platform list
    // class BUSSIM_NODELIST_ENTRY *TmpRef;
    // TmpRef = new class BUSSIM_NODELIST_ENTRY(this);
    //_NET->PLATFORM_LIST->append_node(TmpRef);
    //TmpRef = nullptr;

        // install the (empty) passenger waiting lists
    this->WAITINGLIST = new class BUSSIM_PAX_LIST* [_NET->EXT_LINES + 1];
    for( long i=0 ; i < (_NET->EXT_LINES + 1) ; i ++ )
        if( i < (_NET->EXT_LINES) )
            this->WAITINGLIST[i] = new class BUSSIM_PAX_LIST("WL platform" + std::to_string(this->ID) + " / line " + std::to_string(i));
        else
            this->WAITINGLIST[i] = new class BUSSIM_PAX_LIST("WL platform" + std::to_string(this->ID) + " / walking");
}

void BUSSIM_POINT::set_label_pos_auto(class BUSSIM_NETWORK *_NET)
{
    double PI = 3.14159265358979323846;

    int SEC_USED[8];
    for( int i=0 ; i < 8 ; i++ )
        SEC_USED[i] = 0;

    // std::cout << "looking for opt. label pos of point ID=" << this->ID << "... ";

    double x_diff = 0.0;
    double y_diff = 0.0;
    double winkel = 0.0;

    for(int a=0 ; a < _NET->ARCS ; a++ )
    {
        winkel = 0.0;
        if( _NET->ARC[a].orig_node == this->ID )
        {
            //std::cout << "outbound arc to " << _NET->ARC[a].dest_node << std::endl;
            x_diff = _NET->PT[_NET->ARC[a].dest_node].screen_x - _NET->PT[_NET->ARC[a].orig_node].screen_x;
            y_diff = _NET->PT[_NET->ARC[a].dest_node].screen_y - _NET->PT[_NET->ARC[a].orig_node].screen_y;

            if( ( pow(x_diff,2.0) + pow(y_diff,2.0) ) > 0 )
            {
                // not the same coordinates
                if( x_diff > 0 )
                    winkel = acos(y_diff / sqrt( pow(x_diff,2.0) + pow(y_diff,2.0) ) );
                else
                    winkel = 3.141 + acos(-y_diff / sqrt( pow(x_diff,2.0) + pow(y_diff,2.0) ) );

                for( int i=0 ; i < 8 ; i ++ )
                    if( (winkel >= (double)(i)/4.0 * PI ) && (winkel < (double)(i+1)/4.0 * PI ) )
                        SEC_USED[i]++;
            }
            //std:: cout << "winkel=" << winkel << std::endl;
        }
        if( _NET->ARC[a].dest_node == this->ID )
        {
            //std::cout << "inbound arc from " << _NET->ARC[a].orig_node << std::endl;

            x_diff =  -(_NET->PT[_NET->ARC[a].dest_node].screen_x - _NET->PT[_NET->ARC[a].orig_node].screen_x);
            y_diff =  -(_NET->PT[_NET->ARC[a].dest_node].screen_y - _NET->PT[_NET->ARC[a].orig_node].screen_y);

            if( ( pow(x_diff,2.0) + pow(y_diff,2.0) ) > 0 )
            {
                // not the same coordinates
                if( x_diff > 0 )
                    winkel = acos(y_diff / sqrt( pow(x_diff,2.0) + pow(y_diff,2.0) ) );
                else
                    winkel = 3.141 + acos(-y_diff / sqrt( pow(x_diff,2.0) + pow(y_diff,2.0) ) );

                for( int i=0 ; i < 8 ; i ++ )
                    if( (winkel >= (double)(i)/4.0 * PI ) && (winkel < (double)(i+1)/4.0 * PI ) )
                        SEC_USED[i]++;
            }

            // jetzt noch prfen, ob es schon gelabelte Knoten im Umkreis gibt und ggf.
            // weitere Sektoren Sperren
            //std:: cout << "winkel=" << winkel << std::endl;
        }
    }

    // now check the vincinty for close nodes
    for(int i=0 ; i < _NET->POINTS ; i++ )
    {
        // check only for regular stops
        if( _NET->PT[i].INFRASTRUCTURE < 1 )
        {
            // different from this
            if( this->ID != _NET->PT[i].ID )
            {
                x_diff = _NET->PT[i].screen_x - this->screen_x;
                y_diff = _NET->PT[i].screen_y - this->screen_y;

                if( ( pow(x_diff,2.0) + pow(y_diff,2.0) ) > 0 )
                    if( sqrt( ( pow(x_diff,2.0) + pow(y_diff,2.0) ) ) < 0.1 )
                {
                    // not the same coordinates
                    if( x_diff > 0 )
                        winkel = acos(y_diff / sqrt( pow(x_diff,2.0) + pow(y_diff,2.0) ) );
                    else
                        winkel = 3.141 + acos(-y_diff / sqrt( pow(x_diff,2.0) + pow(y_diff,2.0) ) );

                    for( int j=0 ; j < 8 ; j ++ )
                        if( (winkel >= (double)(j)/4.0 * PI ) && (winkel < (double)(j+1)/4.0 * PI ) )
                            SEC_USED[j]++;
                }
            }
        }
    }

        // now check the label positions for close nodes
    for(int i=0 ; i < _NET->POINTS ; i++ )
    {
        // check only for regular stops
        if( _NET->PT[i].IS_POSITIONED == 1 )
        {
            // different from this
            if( this->ID != _NET->PT[i].ID )
            {
                //std::cout << "compared with node " << _NET->PT[i].ID << std::endl;
                x_diff = _NET->PT[i].screen_x - this->screen_x;
                y_diff = _NET->PT[i].screen_y - this->screen_y;

                if( ( pow(x_diff,2.0) + pow(y_diff,2.0) ) > 0 )
                    if( sqrt( ( pow(x_diff,2.0) + pow(y_diff,2.0) ) ) < 0.001 )
                {
                    //std::cout << "in radius" << std::endl;
                    // not the same coordinates
                    if( x_diff > 0 )
                        winkel = acos(y_diff / sqrt( pow(x_diff,2.0) + pow(y_diff,2.0) ) );
                    else
                        winkel = 3.141 + acos(-y_diff / sqrt( pow(x_diff,2.0) + pow(y_diff,2.0) ) );

                    //std::cout << "winkel=" << winkel << std::endl;
                    //std::cout << "labelpos=" << _NET->PT[i].label_pos << std::endl;
                    for( int k=0 ; k < 8 ; k++ )
                    {
                        if( (winkel >= (double)(k)/4.0 * PI ) && (winkel < (double)(k+1)/4.0 * PI ) )
                        {
                            //std::cout << "in direction " << k << std::endl;
                            if( _NET->PT[i].label_pos == ((k+0) % 8) )
                            {
                                //std::cout << "gaga" << std::endl;
                                //SEC_USED[(k-2) % 8]++;
                                //SEC_USED[(k-1) % 8]++;
                                SEC_USED[(k) % 8]++;
                                SEC_USED[(k+1) % 8]++;
                                SEC_USED[(k+2) % 8]++;
                                SEC_USED[(k+3) % 8]++;
                            }
                            if( _NET->PT[i].label_pos == ((k+1) % 8) )
                            {
                                //std::cout << "gaga" << std::endl;
                                //SEC_USED[(k-2) % 8]++;
                                //SEC_USED[(k-1) % 8]++;
                                SEC_USED[(k) % 8]++;
                                SEC_USED[(k+1) % 8]++;
                                SEC_USED[(k+2) % 8]++;
                                SEC_USED[(k+3) % 8]++;
                            }
                            if( _NET->PT[i].label_pos == ((k+2) % 8) )
                            {
                                //std::cout << "gaga" << std::endl;
                                //SEC_USED[(k-2) % 8]++;
                                //SEC_USED[(k-1) % 8]++;
                                SEC_USED[(k) % 8]++;
                                SEC_USED[(k+1) % 8]++;
                                SEC_USED[(k+2) % 8]++;
                                SEC_USED[(k+3) % 8]++;
                            }
                            if( _NET->PT[i].label_pos == ((k+3) % 8) )
                            {
                                //std::cout << "gaga" << std::endl;
                                //SEC_USED[(k-2) % 8]++;
                                //SEC_USED[(k-1) % 8]++;
                                SEC_USED[(k) % 8]++;
                                SEC_USED[(k+1) % 8]++;
                                SEC_USED[(k+2) % 8]++;
                                SEC_USED[(k+3) % 8]++;
                            }
                            if( _NET->PT[i].label_pos == ((k+4) % 8) )
                            {
                                //std::cout << "gaga" << std::endl;
                                //SEC_USED[(k-2) % 8]++;
                                //SEC_USED[(k-1) % 8]++;
                                SEC_USED[(k) % 8]++;
                                SEC_USED[(k+1) % 8]++;
                                SEC_USED[(k+2) % 8]++;
                                SEC_USED[(k+3) % 8]++;
                            }
                            if( _NET->PT[i].label_pos == ((k+5) % 8) )
                            {
                                //std::cout << "gaga" << std::endl;
                                //SEC_USED[(k-2) % 8]++;
                                //SEC_USED[(k-1) % 8]++;
                                SEC_USED[(k) % 8]++;
                                SEC_USED[(k+1) % 8]++;
                                SEC_USED[(k+2) % 8]++;
                                SEC_USED[(k+3) % 8]++;
                            }
                            if( _NET->PT[i].label_pos == ((k+6) % 8) )
                            {
                                //std::cout << "gaga" << std::endl;
                                //SEC_USED[(k-2) % 8]++;
                                //SEC_USED[(k-1) % 8]++;
                                SEC_USED[(k) % 8]++;
                                SEC_USED[(k+1) % 8]++;
                                SEC_USED[(k+2) % 8]++;
                                SEC_USED[(k+3) % 8]++;
                           }
                            if( _NET->PT[i].label_pos == ((k+7) % 8) )
                            {
                                //std::cout << "gaga" << std::endl;
                                //SEC_USED[(k-2) % 8]++;
                                //SEC_USED[(k-1) % 8]++;
                                SEC_USED[(k) % 8]++;
                                SEC_USED[(k+1) % 8]++;
                                SEC_USED[(k+2) % 8]++;
                                SEC_USED[(k+3) % 8]++;
                           }
                        }
                    }
                }

            }
        }
    }

    // select the label position based on allocated cones
    this->label_pos = BUSSIM_DEFAULT_LABEL_POS;

    /*
    for( int i=0 ; i < 8 ; i ++ )
        std::cout << "USED[" << i << "] = " << SEC_USED[i] << std::endl;
    */

    // now, evaluate the different positions
    struct _EVAL {
        int pos;
        int value;
    };

    struct _EVAL POS[8];
    for( int i=0 ; i < 8 ; i ++ )
    {
        POS[i].pos = i;
        POS[i].value = -1;
    }

    for( int i=0 ; i < 8 ; i ++ )
        for( int k=0 ; k < 4 ; k++ )
        {
            if( ( SEC_USED[ (i+k) % 8 ] == 0 ) && ( SEC_USED[ (i-k) % 8 ] == 0 ) )
            {
                POS[i].value = k;
            }
            else
                k = 5;
        }

    // sort the candidate positions
    struct _EVAL TMP;
    for( int i=0 ; i < 7 ; i++ )
        for( int j=0 ; j < 7 ; j++ )
            if( POS[j].value < POS[j+1].value )
            {
                TMP = POS[j];
                POS[j] = POS[j+1];
                POS[j+1] = TMP;
            }

    /*
    for( int i=0 ; i < 8 ; i ++ )
        std::cout << "POS[" << i << "].pos = " << POS[i].pos << " / " << "value=" << POS[i].value << std::endl;
    */
    this->label_pos = POS[0].pos;

    this->IS_POSITIONED = 1;

    //std::cout << " determined pos = " << this->label_pos << std::endl;
}

void BUSSIM_POINT::print_waiting_lists(double _XPOS, double _YPOS, class BUSSIM_NETWORK *_NET)
{
    class BUSSIM_PAX_LISTITEM *Ptr;
    if( this->WAITINGLIST != nullptr )
    {
        for(long i=0 ; i < _NET->EXT_LINES + 1 ; i++ )
        {
            std::cout << "This is waiting list " << this->WAITINGLIST[i]->get_listname() << std::endl;
            Ptr = this->WAITINGLIST[i]->get_first_pax_item();
            while( Ptr != nullptr )
            {
                std::cout << "pax " << Ptr->get_ref2pax()->ID << std::endl;
                Ptr = Ptr->get_next();
            }
        }
    }
    /*
    int GAGA;
    std::cin >> GAGA;
    */
}

void BUSSIM_POINT::init_add_to_waiting_list(class BUSSIM_PAX *_REF2PAX, class BUSSIM_NETWORK *_NET)
{
    // this procedure adds the pax stored at the end of _REF2PAX into the
    // waiting list of the line found at _REF2PAX->Ref2CurrentTPSeg at this point
    //
    // note: Ref2CurrentTPSeg must be updated prior to calling this function

    long WaitingForLine;

    if( _REF2PAX->TravelPath != nullptr )
    {
        // there is a path store with the passenger
        WaitingForLine = _REF2PAX->TravelPath->get_first_line();
        if( WaitingForLine == -1 )
        {
            // passenger walks the first part if its travelpath
            // we add this pax to the list of walking passengers
            _NET->PAX_WALKING->append_pax(_REF2PAX,1);

            //_REF2PAX->proceed_on_travelpath(_NET);
            //this->WAITINGLIST[_NET->EXT_LINES]->append_pax(_REF2PAX,0);

            // Memorize the platform from where the current walk started
            //_REF2PAX->WalkingStartPlatform = this->ID;

            // update the walking position
            _REF2PAX->update_pax_pos_walking(_NET);
        }
        else
        {
            // passenger waits for a regular line
            this->WAITINGLIST[WaitingForLine]->append_pax(_REF2PAX,0);
        }
    }
}
