#ifndef BUSSIM_SCHEDULE_H_INCLUDED
#define BUSSIM_SCHEDULE_H_INCLUDED

#include "bussim_global.h"

//#include "bussim_arc.h"
//#include "bussim_duty.h"
//#include <string>
//#include "bussim_schedule.h"
//#include "bussim_vehicle.h"
//#include "bussim_network.h"

class BUSSIM_SEGMENT {
public:

    // this class is used to store all information for a scheduled
    // origin-to-destination move of a vehicle along an arc
    int ID;                         // unique identifier
    class BUSSIM_ARC *ref2arc;      // along this arc

    double time_in;                 // planned time when vehicle enters this arc
    double time_out;                // planned time when vehicle reaches of of this arc
    class BUSSIM_DUTY *ref2duty;    // as part of this duty
    int on_line;                    // on which service?
    double park_pos_on_arc;         // position on arc where vehicle terminates ([0..1])
    int pos_in_schedule;            // segment position in schedule
    std::string dest_desc;          // current destination name

    class BUSSIM_SCHEDULE *ref2schedule;    // part of schedule

    class BUSSIM_PAX_LIST *ref2DeBoardPaxList; // refers to the paxlist in a vehicle that is associated
                                                // with the dest node of this segment's arc
    class BUSSIM_SEGMENT *prev;     // ref to previous segment
    class BUSSIM_SEGMENT *next;     // ref to next segment

    int DeboardAtEnd;                       // 3.00: =1 if dest of this arc is a depot or regular node, =0 else

    int PosDeboardPaxList;                  // position of this platform in the current duty

    // the constructors
    BUSSIM_SEGMENT();   // the standard constructor
    BUSSIM_SEGMENT(int ID, class BUSSIM_ARC *ref2arc,
                   double time_in,
                   double time_out,
                   class BUSSIM_DUTY *ref2duty,
                   int on_line,
                   std::string dest_desc,
                   class BUSSIM_SCHEDULE *ref2schedule,
                   class BUSSIM_SEGMENT *prev,
                   class BUSSIM_SEGMENT *next,
                   class BUSSIM_NETWORK *_NET,
                   int _INC);   // the overloaded constructor
    ~BUSSIM_SEGMENT();   // the standard destructor
};

class BUSSIM_SCHEDULE {
public:

    // this class is used to host the schedule of exactly one vehicle

    int ID;                             // unique identifier

    int segments;                       // number of segments

    class BUSSIM_VEHICLE *ref2veh;      // reference to the serving vehicle

    class BUSSIM_SEGMENT *first_segment;
    class BUSSIM_SEGMENT *last_segment;
    class BUSSIM_SEGMENT *current_segment;

    BUSSIM_SCHEDULE();                   // the standard constructor
    BUSSIM_SCHEDULE(int iD, class BUSSIM_VEHICLE *ref2veh);  // an overloaded constructor
    ~BUSSIM_SCHEDULE();                  // the standard destructor

    void append_segment(class BUSSIM_SEGMENT *Ptr, class BUSSIM_NETWORK *_NET);
                                        // append the next trip segment to this schedule
    void clean(class BUSSIM_NETWORK *_NET);
                                        // removes all segment from the schedule
    void print_on_shell(class BUSSIM_NETWORK *_NET);
                                        // to print the schedule on the shell
    void print_on_shell_remaining_duty(class BUSSIM_NETWORK *_NET);
                                        // to print the schedule on the shell from currentsegment to the end of current duty
};

#endif // BUSSIM_SCHEDULE_H_INCLUDED
