#ifndef BUSSIM_VEHICLE_H_INCLUDED
#define BUSSIM_VEHICLE_H_INCLUDED

#include "bussim_global.h"
//#include "bussim_network.h"

struct BUSSIM_TRANSFER_SITUATION {

    // we use this struct to provide information for the calculation of the
    // transfer time at a stop of a vehicle
    long PAX_ARRIVING;
    long PAX_DEPARTING;

    // this is the result
    double CALCULATED_TRANSFER_TIME;
};

class BUSSIM_VEHICLE {
public:
    int ID;             // unique identifier for this vehicle

    int on_line;        // refers to the currently served line

    double velocity;    // given in km/h

    double activation_time;     // V3.00 activation time of vehicle

    int ACTIVE;         // if true (=1) then this vehicle is updated otherwise (=0) updates are not made
    int WAITING;          // 1 = TRAVELLING, 2 = WAITING

    int WAIT_FOR_BLOCK; // indicates if a vehicle waits to enter an arc grouped in a block

    double last_update_time;        // time point with last update
    double remaing_waiting_time;    // remaining waiting time at a stop

    class BUSSIM_ARC ON_ARC;         // gives the identifier of an arc
    double POS;         // position on ON_ARC, varies between 0 (at arc origin) and 1 (at arc destination)

    double km_travelled_on_line;
    double start_time_on_line;

    double km_travelled_on_arc;     // needed to find out if overtaking is needed on an arc
    double start_time_on_arc;

    int Thwarted;        // number of events where a vehicle is blocked by another

    int VISIBLE;

    int IS_BLOCKED;         // V3.00 indicates if a vehicle is currently thwarted

    class BUSSIM_ROTATION *rotation;   // reference to the vehicle's rotation

    class BUSSIM_ARC next_arc;     // what is the next traversed arc (initialized by -1)
    int last_visited_node;          // what was the last visited node? (initialized by -1);
    double last_node_arrival_time;  // when did it arrived at last_visited_node?;

    class BUSSIM_SCHEDULE *SCHEDULE;    // by V3.00 - reference to the vehicle's schedule

    class BUSSIM_ARC park_arc;  // specifies the final position where a vehicle is parked after
    double park_pos;            // it has fulfilled its schedule

    // the following array is used to store riding passengers in a vehicle
    class BUSSIM_PAX_LIST *DeBoardPaxList;  // by 3.00
    int PAX_LIST_NUM; // by 3.00
    int PAX_ON_BOARD;   // by 3.00

    int capacity;       // declares the number of maximal passengers

    int vehicle_category;   // used to distinguish between trams, bus, train, ...

    BUSSIM_VEHICLE();
    BUSSIM_VEHICLE(int _ID, int _ACTIVE, int _WAITING, double _VELOCITY, double _LAST_UPDATE_TIME, double _POS, double _REMAINING_WAITING_TIME, class BUSSIM_ARC _ON_ARC);
    ~BUSSIM_VEHICLE();

    void configure(int _ID, int _VEHCAT, double _VELOCITY, int _CAPACITY, class BUSSIM_ARC _ON_ARC, class BUSSIM_ARC _PARK_ARC, class BUSSIM_NETWORK *_NET);
    void update(class BUSSIM_NETWORK *_NETWORK);
    void update_new(class BUSSIM_NETWORK *_NETWORK);
    void print_on_screen(class BUSSIM_NETWORK *_NET);
    void print_on_screen_tram(class BUSSIM_NETWORK *_NET);   // by V3.00
    void print_on_screen_bus(class BUSSIM_NETWORK *_NET);   // by V3.00
    void assign_to_line(class BUSSIM_NETWORK *_NET, class BUSSIM_LINE *_LINE);
    int check_for_takeover(class BUSSIM_NETWORK *_NET, double *ADD_DIST);
    int check_for_takeover_crossline(class BUSSIM_NETWORK *_NET, double *ADD_DIST);
    int check_for_takeover_new(class BUSSIM_NETWORK *_NET, double *ADD_DIST);
    int check_for_takeover_4(class BUSSIM_NETWORK *_NET, double *ADD_DIST); // by version 1.05
    void activate(class BUSSIM_NETWORK *_NET);
    void activate_new(class BUSSIM_NETWORK *_NET); // by version 3.00 to use the detailed schedule

    void rotation2schedule(class BUSSIM_NETWORK *_NET); // by V3.00
    void set_activation_time(double _activation_time);  // by V3.00

    int compare_init_pos_with_schedule(class BUSSIM_NETWORK *_NET); // by 3.00
    int compare_park_pos_with_schedule(class BUSSIM_NETWORK *_NET); // by 3.00

    void remove_deboard_pax_lists(class BUSSIM_NETWORK *_NET);  // by 3.00
    void init_deboard_pax_lists(class BUSSIM_NETWORK *_NET, class BUSSIM_SEGMENT *_SEG);    // by 3.00
    void print_paxlists_on_console(class BUSSIM_NETWORK *_NET); // by 3.00

    void pickup_waiting_pax(class BUSSIM_POINT *_REF2POINT, class BUSSIM_NETWORK *_NET); // by 3.00
    void deliver_pax(class BUSSIM_POINT *_REF2POINT, class BUSSIM_NETWORK *_NET); // by 3.00

    void proceed_pax_on_board(class BUSSIM_NETWORK *_NET);  // by 3.00

    void print_paxlist_on_console(class BUSSIM_NETWORK *_NET);

    void determine_transfer_time(struct BUSSIM_TRANSFER_SITUATION *_SIT, class BUSSIM_NETWORK *_NET);
};

#endif // BUSSIM_VEHICLE_H_INCLUDED
