#include "bussim_global.h"
//#include "bussim_network.h"
//#include "bussim_bunchpoint.h"
//#include "bussim_rotation.h"
//#include "bussim_duty.h"
//#include "bussim_vehicle.h"
//#include "bussim_schedule.h"

#include <math.h>
#include <iostream>
#include <iomanip>
#include <fstream>
#include <stdio.h>
#include <string>
#include <time.h>
#include <stdlib.h>

//#include <GL/glut.h>
//#include <freeglut/freeglut.h>

BUSSIM_NETWORK::BUSSIM_NETWORK()
{
    // this is the default constructor

    this->POINTS = 0;
    this->ARCS = 0;
    this->TOIS = 0;
    this->LINES = 0;
    this->EXT_LINES = 0;
    this->VEHICLES = 0;
    this->ACTIVE_VEHICLES = 0;
    this->WAITING_VEHICLES = 0;
    this->BUNCHPOINTS = 0;
    this->EVENTS = 0;
    this->DUTIES = 0;
    this->BLOCK_WAIT = 0;
    this->SIMULATION_COMPLETE = 0;
    this->LAST_HOURS = -1;
    this->LAST_MINUTES = -1;
    this->STOPS = 0;

    this->PT = nullptr;
    this->ARC = nullptr;
    this->LINE = nullptr;
    this->VEHICLE = nullptr;
    this->BP = nullptr;
    this->STOP = nullptr;
    this->TOI = nullptr;

    this->SEGMENT_CNTR = 0;

    this->xmin =  1000000.0;
    this->xmax = -1000000.0;
    this->ymin =  1000000.0;
    this->ymax = -1000000.0;

    this->x_factor = 1.0;
    this->y_factor = 1.0;

    this->BGTextureId = 0;

    this->center_longitude = 0.0;
    this->center_latitude = 0.0;

    this->max_long = 0.0;
    this->min_long = 0.0;
    this->max_lat = 0.0;
    this->min_lat = 0.0;

    this->Current_Zoom_Level = 0;
    this->Current_MENUE_OFFSET_X = 0;
    this->Current_MENUE_OFFSET_Y = 0;

    this->CURRENT_TIME = 0;
    this->SPEEDUP_FACTOR = BUSSIM_INITIAL_SPEEDUP_FACTOR;
    this->TIME_STEP = BUSSIM_TIME_STEP;

    this->BETRIEBS_ENDE = 0;

    this->DETECTED_BUNCHES = 0;

    this->first_event = nullptr;
    this->last_event = nullptr;

    this->first_bp = nullptr;
    this->last_bp = nullptr;

    this->PRED = nullptr;

    this->FIRST_VP_LIST = nullptr;
    this->LAST_VP_LIST = nullptr;
    Stored_VP_Move_Items = 0;

    // set the pointers to the passenger lists
    this->PAX_UNKNOWN = nullptr;
    this->PAX_IN = nullptr;
    this->PAX_ARRIVED = nullptr;

    this->PAX_WALKING = nullptr;

    this->SERVICE = nullptr;

    this->PLATFORM_LIST = nullptr;

    this->NUM_PAX_IN_WAITING = 0;
}

BUSSIM_NETWORK::BUSSIM_NETWORK(int _STOPS, int _POINTS, int _ARCS, int _BLOCKS, int _LINES, int _VEHICLES, int _TOIS, int _PAXS)
{
  // this is an overloaded constructor

    this->POINTS = _POINTS;
    this->ARCS = _ARCS;
    this->TOIS = _TOIS;
    this->EXT_LINES = _LINES;
    this->LINES = _LINES + _VEHICLES + _VEHICLES; // user-defined lines + deployment + return
    this->VEHICLES = _VEHICLES;
    this->ACTIVE_VEHICLES = 0;
    this->WAITING_VEHICLES = 0;
    this->BUNCHPOINTS = 0;
    this->EVENTS = 0;
    this->DUTIES = 0;
    this->BLOCKS = _BLOCKS;
    this->BLOCK_WAIT = 0;
    this->SIMULATION_COMPLETE = 0;
    this->STOPS = _STOPS;

    this->BGTextureId = 0;

    this->center_longitude = 0.0;
    this->center_latitude = 0.0;

    this->max_long = 0.0;
    this->min_long = 0.0;
    this->max_lat = 0.0;
    this->min_lat = 0.0;

    this->Current_Zoom_Level = 0;
    this->Current_MENUE_OFFSET_X = 0;
    this->Current_MENUE_OFFSET_Y = 0;

    this->PAXS = _PAXS;

    if( this->POINTS > 0)
    {
        this->PT = new class BUSSIM_POINT[this->POINTS];
        for( int p=0 ; p < this->POINTS ; p++ )
        {
            this->PT[p].ID = p;
            this->PT[p].label_pos = BUSSIM_DEFAULT_LABEL_POS;
        }
    }
    if( this->ARCS > 0)
        this->ARC = new class BUSSIM_ARC[this->ARCS];
    if( this->BLOCKS > 0)
        this->BLOCK = new class BUSSIM_BLOCK[this->BLOCKS];
    if( this->TOIS > 0)
        this->TOI = new class BUSSIM_TOI[this->TOIS];
    if( this->LINES > 0)
        this->LINE = new class BUSSIM_LINE[this->LINES];
    if( this->VEHICLES > 0)
        this->VEHICLE = new class BUSSIM_VEHICLE[this->VEHICLES];
    if( this->STOPS > 0 )
        this->STOP = new class BUSSIM_STOP[this->STOPS];

    this->BP = nullptr;

    this->SEGMENT_CNTR = 0;

    this->xmin =  1000000.0;
    this->xmax = -1000000.0;
    this->ymin =  1000000.0;
    this->ymax = -1000000.0;

    this->CURRENT_TIME = 0;
    this->SPEEDUP_FACTOR = BUSSIM_INITIAL_SPEEDUP_FACTOR;
    this->TIME_STEP = BUSSIM_TIME_STEP;

    this->BETRIEBS_ENDE = 0;

    this->DETECTED_BUNCHES = 0;

    this->first_event = nullptr;
    this->last_event = nullptr;

    this->first_bp = nullptr;
    this->last_bp = nullptr;

    this->PRED = new int* [this->VEHICLES];
    for( int i=0 ; i < this->VEHICLES ; i++ )
    {
        this->PRED[i] = new int [this->VEHICLES];
        for( int j=0 ; j < this->VEHICLES ; j++ )
            this->PRED[i][j] = 0;
    }

    this->FIRST_VP_LIST = nullptr;
    this->LAST_VP_LIST = nullptr;
    Stored_VP_Move_Items = 0;

        // set the pointers to the passenger lists
    this->PAX_UNKNOWN = new class BUSSIM_PAX_LIST("passengers to come");
    this->PAX_IN = new class BUSSIM_PAX_LIST("passengers in the system");
    this->PAX_ARRIVED = new class BUSSIM_PAX_LIST("passengers processed");

    this->PAX_WALKING = new class BUSSIM_PAX_LIST("walking passengers");

    // init the SERVICE-matrix
    this->SERVICE = nullptr;

    if( this->LINES > 0 )
        if( this->POINTS > 0 )
        {
            this->SERVICE = new int** [this->LINES];
            for(int l=0 ; l < this->LINES ; l++ )
            {
                this->SERVICE[l] = new int* [this->POINTS];
                for(int i=0 ; i < this->POINTS ; i++ )
                {
                    this->SERVICE[l][i] = new int [this->POINTS];
                    for(int j=0 ; j < this->POINTS ; j++ )
                    {
                        this->SERVICE[l][i][j] = 0;
                    }
                }
            }
        }

    // initialize the empty platform list
    this->PLATFORM_LIST = new class BUSSIM_NODELIST("platforms");

    // initialize the counter of currently waiting pax in the system
    this->NUM_PAX_IN_WAITING = 0;
}

BUSSIM_NETWORK::~BUSSIM_NETWORK()
{
    // save current config
    this->save_vp_move_list();

    // this is the default constructor
    if ( this->POINTS > 0)
        delete[] this->PT;
    if ( this->ARCS > 0)
        delete[] this->ARC;
    if ( this->LINES > 0)
        delete[] this->LINE;
    if ( this->VEHICLES > 0)
        delete[] this->VEHICLE;
    if ( this->BUNCHPOINTS > 0)
        delete[] this->BP;
}

void BUSSIM_NETWORK::specify_nodes(void)
{
    // here you type in the node information of your network
    // points specification ends here

    // DO NOT USE ANYMORE use specify_nodes_longlat(double,double) instead

    // set the ID
    for(int i=0 ; i < this->POINTS ; i++)
        this->PT[i].ID = i;
}

void BUSSIM_NETWORK::longlat_to_worldxy(void)
{
   // added in version 1.05
   // this procedure calculates Cartesian coordinates from longlat-coordinates
   // is uses regional distances to determine km-distances between long/lat.
   // the points are centered.

   // start and calculate the geographic center
   double min_long = 360;
   double max_long = -360;
   double min_lat = 360;
   double max_lat = -360;

   for(int i=0 ; i < this->POINTS ; i++ )
   {
        if ( this->PT[i].world_longitude < min_long )
            min_long = this->PT[i].world_longitude;
        if ( this->PT[i].world_longitude > max_long )
            max_long = this->PT[i].world_longitude;
        if ( this->PT[i].world_latitude < min_lat )
            min_lat = this->PT[i].world_latitude;
        if ( this->PT[i].world_latitude > max_lat )
            max_lat = this->PT[i].world_latitude;
   }

   // check of the TOI-coordinates
   for(int i=0 ; i < this->TOIS ; i++ )
   {
       struct BUSSIM_TOI_POINT *Tmp;
       Tmp = this->TOI[i].PT;
       while( Tmp != nullptr )
       {
            if ( Tmp->longitude < min_long )
                min_long = Tmp->longitude;
            if ( Tmp->longitude > max_long )
                max_long = Tmp->longitude;
            if ( Tmp->latitude < min_lat )
                min_lat = Tmp->latitude;
            if ( Tmp->latitude > max_lat )
                max_lat = Tmp->latitude;
            Tmp = Tmp->next;
       }
   }

    // store the network information
    this->center_longitude = ( max_long + min_long ) / 2.0;
    this->center_latitude = ( max_lat + min_lat ) / 2.0;

    this->min_lat = min_lat;
    this->max_lat = max_lat;
    this->min_long = min_long;
    this->max_long = max_long;

   // set the world-coordinates of the stops
   for(int i=0 ; i < this->POINTS ; i++ )
   {
       this->PT[i].world_x = ( this->PT[i].world_longitude - this->center_longitude ) * BUSSIM_LONG_DIST_KM;
       this->PT[i].world_y = ( this->PT[i].world_latitude - this->center_latitude ) * BUSSIM_LAT_DIST_KM;
   }

   // set the world-coordinates of the tois
    for(int i=0 ; i < this->TOIS ; i++ )
   {
       struct BUSSIM_TOI_POINT *TmpPtr;
       TmpPtr = this->TOI[i].PT;
       while( TmpPtr != nullptr )
       {
            TmpPtr->world_x = ( TmpPtr->longitude - this->center_longitude ) * BUSSIM_LONG_DIST_KM;
            TmpPtr->world_y = ( TmpPtr->latitude - this->center_latitude ) * BUSSIM_LAT_DIST_KM;
            TmpPtr = TmpPtr->next;
       }
   }

}

void BUSSIM_NETWORK::specify_nodes_longlat(void)
{
    // added in version 1.05
    // specifies the points using geo-coordinates
    // after coordinate specification these are transformed into (world_x;world_y)-pairs
    // here you type in the node information of your network
	//this->PT[0].specify_regular(8.7897806,53.050562,0,this);
	//this->PT[1].specify_trackpos(8.78956,53.05077,-1,"","NLFFLU0",this);
	//this->PT[2].specify_trackpos(8.78664,53.05271,-1,"","NLFFLU1",this);
	//this->PT[3].specify_regular(8.7861692,53.0529497,1,this);
	//this->PT[16].specify_switch(8.78607,53.06159,3,"Weiche 1","BSZSLE0",this);

	this->PT[0].specify_depot(13.79881, 50.99919, 0,  this);
	this->PT[1].specify_trackpos(13.79917, 50.99904, -1, "Trackpos 1",  "TP1", this);
	this->PT[2].specify_trackpos(13.7995, 50.99903, -1, "Trackpos 2",  "TP2", this);
	this->PT[3].specify_trackpos(13.79974, 50.99912, -1, "Trackpos 3",  "TP3", this);
	this->PT[4].specify_switch(13.79984, 50.99923, -1, "Weiche 1 - Ausfahrt prohlis",  "W1", this);
	this->PT[5].specify_trackpos(13.79993, 50.99936, -1, "Trackpos 4",  "TP4", this);
	this->PT[6].specify_trackpos(13.79991, 50.99959, -1, "Trackpos 5",  "TP5", this);
	this->PT[7].specify_trackpos(13.79962, 50.99978, -1, "Trackpos 6",  "TP6", this);
	this->PT[8].specify_trackpos(13.79934, 50.99981, -1, "Trackpos 7",  "TP7", this);
	this->PT[9].specify_trackpos(13.79835, 50.99968, -1, "Trackpos 8",  "TP8", this);
	this->PT[10].specify_trackpos(13.79801, 50.99978, -1, "Trackpos 9",  "TP9", this);
	this->PT[11].specify_trackpos(13.79711, 51.00025, -1, "Trackpos 10",  "TP10", this);
	this->PT[12].specify_trackpos(13.79695, 51.00043, -1, "Trackpos 11",  "TP11", this);
	this->PT[13].specify_trackpos(13.797, 51.00077, -1, "Trackpos 12",  "TP12", this);
	this->PT[14].specify_regular(13.79783, 51.00191, 1,  this);
	this->PT[15].specify_trackpos(13.79801, 51.00219, -1, "Trackpos 13",  "TP13", this);
	this->PT[16].specify_trackpos(13.79812, 51.00253, -1, "Trackpos 14",  "TP14", this);
	this->PT[17].specify_trackpos(13.79799, 51.00394, -1, "Trackpos 15",  "TP15", this);
	this->PT[18].specify_trackpos(13.79811, 51.00437, -1, "Trackpos 16",  "TP16", this);
	this->PT[19].specify_trackpos(13.79855, 51.0048, -1, "Trackpos 17",  "TP17", this);
	this->PT[20].specify_regular(13.7996, 51.00556, 2,  this);
	this->PT[21].specify_regular(13.8034, 51.00833, 3,  this);
	this->PT[22].specify_switch(13.8035, 51.0084, -1, "Weiche 2 ",  "W2", this);
	this->PT[23].specify_trackpos(13.80356, 51.00852, -1, "Trackpos 18",  "TP18", this);
	this->PT[24].specify_trackpos(13.80354, 51.00862, 0.1, "Trackpos 19",  "TP19", this);
	this->PT[25].specify_switch(13.80342, 51.00872, -1, "Weiche 3",  "W3", this);
	this->PT[26].specify_switch(13.80252, 51.00915, -1, "Weiche 4 - Einfahrt BH0",  "W4", this);
	this->PT[27].specify_switch(13.80171, 51.00956, -1, "Weiche 5 - Ausfahrt BH0",  "W5", this);
	this->PT[28].specify_regular(13.80093, 51.00998, 4,  this);
	this->PT[29].specify_trackpos(13.80054, 51.01022, -1, "Trackpos 20",  "TP20", this);
	this->PT[30].specify_trackpos(13.80013, 51.01054, -1, "Trackpos 21",  "TP21", this);
	this->PT[31].specify_trackpos(13.79887, 51.01185, -1, "Trackpos 22",  "TP22", this);
	this->PT[32].specify_trackpos(13.79879, 51.01205, -1, "Trackpos 23",  "TP23", this);
	this->PT[33].specify_trackpos(13.7988, 51.01227, -1, "Trackpos 24",  "TP24", this);
	this->PT[34].specify_trackpos(13.79866, 51.01243, -1, "Trackpos 25",  "TP25", this);
	this->PT[35].specify_trackpos(13.79838, 51.01251, -1, "Trackpos 26",  "TP26", this);
	this->PT[36].specify_trackpos(13.79648, 51.01272, -1, "Trackpos 27",  "TP27", this);
	this->PT[37].specify_trackpos(13.79611, 51.0128, -1, "Trackpos 28",  "TP28", this);
	this->PT[38].specify_regular(13.79565, 51.01294, 5,  this);
	this->PT[39].specify_trackpos(13.79541, 51.013, -1, "Trackpos 29",  "TP29", this);
	this->PT[40].specify_trackpos(13.79496, 51.01307, -1, "Trackpos 30",  "TP30", this);
	this->PT[41].specify_trackpos(13.79423, 51.0131, -1, "Trackpos 31",  "TP31", this);
	this->PT[42].specify_trackpos(13.79334, 51.01306, -1, "Trackpos 32",  "TP32", this);
	this->PT[43].specify_trackpos(13.79258, 51.01313, -1, "Trackpos 33",  "TP33", this);
	this->PT[44].specify_trackpos(13.79141, 51.01334, -1, "Trackpos 34",  "TP34", this);
	this->PT[45].specify_regular(13.79038, 51.01367, 6,  this);
	this->PT[46].specify_trackpos(13.78953, 51.01398, -1, "Trackpos 35",  "TP35", this);
	this->PT[47].specify_trackpos(13.7886, 51.01444, -1, "Trackpos 36",  "TP36", this);
	this->PT[48].specify_trackpos(13.78807, 51.01483, -1, "Trackpos 37",  "TP37", this);
	this->PT[49].specify_regular(13.7872, 51.01559, 7,  this);
	this->PT[50].specify_trackpos(13.78671, 51.01602, -1, "Trackpos 38",  "TP38", this);
	this->PT[51].specify_trackpos(13.78612, 51.0164, -1, "Trackpos 39",  "TP39", this);
	this->PT[52].specify_regular(13.78403, 51.01757, 8,  this);
	this->PT[53].specify_regular(13.78017, 51.01966, 9,  this);
	this->PT[54].specify_regular(13.77463, 51.02254, 10,  this);
	this->PT[55].specify_regular(13.77051, 51.02469, 11,  this);
	this->PT[56].specify_trackpos(13.77014, 51.02487, -1, "Trackpos 40",  "TP40", this);
	this->PT[57].specify_trackpos(13.77, 51.0249, -1, "Trackpos 41",  "TP41", this);
	this->PT[58].specify_trackpos(13.76983, 51.0249, -1, "Trackpos 42",  "TP42", this);
	this->PT[59].specify_trackpos(13.76961, 51.0248, -1, "Trackpos 43",  "TP43", this);
	this->PT[60].specify_trackpos(13.76704, 51.02288, -1, "Trackpos 44",  "TP44", this);

	this->PT[61].specify_trackpos(13.76602, 51.02236, -1, "Trackpos 45",  "TP45", this);
	this->PT[62].specify_trackpos(13.76602, 51.02236, -1, "Trackpos 46",  "TP46", this);
	this->PT[63].specify_trackpos(13.76578, 51.02231, -1, "Trackpos 47",  "TP47", this);
	this->PT[64].specify_trackpos(13.76564, 51.02233, -1, "Trackpos 48",  "TP48", this);
	this->PT[65].specify_switch(13.76553, 51.02236, -1, "Weiche 6 - Lockwitzer Strae",  "W6", this);
	this->PT[66].specify_regular(13.76508, 51.02266, 12,  this);
	this->PT[67].specify_regular(13.76247, 51.02436, 13,  this);
	this->PT[68].specify_regular(13.75979, 51.02752, 14,  this);
	this->PT[69].specify_trackpos(13.75955, 51.0278, -1, "Trackpos 49",  "TP49", this);
	this->PT[70].specify_trackpos(13.75922, 51.02834, -1, "Trackpos 50",  "TP50", this);
	this->PT[71].specify_trackpos(13.75927, 51.0285, -1, "Trackpos 51",  "TP51", this);
	this->PT[72].specify_regular(13.76147, 51.03141, 15,  this);
	this->PT[73].specify_trackpos(13.7625, 51.03262, -1, "Trackpos 52",  "TP52", this);
	this->PT[74].specify_trackpos(13.76255, 51.0327, -1, "Trackpos 53",  "TP53", this);
	this->PT[75].specify_trackpos(13.76252, 51.0328, -1, "Trackpos 54",  "TP54", this);
	this->PT[76].specify_trackpos(13.76238, 51.03288, -1, "Trackpos 55",  "TP55", this);
	this->PT[77].specify_regular(13.75786, 51.03469, 16,  this);
	this->PT[78].specify_regular(13.75253, 51.03681, 17,  this);
	this->PT[79].specify_trackpos(13.7522, 51.03694, -1, "Trackpos 56",  "TP56", this);
	this->PT[80].specify_trackpos(13.7516, 51.03719, -1, "Trackpos 57",  "TP57", this);
	this->PT[81].specify_trackpos(13.75085, 51.03734, -1, "Trackpos 58",  "TP58", this);
	this->PT[82].specify_trackpos(13.75034, 51.0374, -1, "Trackpos 59",  "TP59", this);
	this->PT[83].specify_trackpos(13.7489, 51.03747, -1, "Trackpos 60",  "TP60", this);
	this->PT[84].specify_trackpos(13.7489, 51.03747, -1, "Trackpos 61",  "TP61", this);
	this->PT[85].specify_switch(13.74852, 51.03755, -1, "Weiche 7 - Einfahrt Lenneplatz",  "W7", this);
	this->PT[86].specify_regular(13.74736, 51.03806, 18,  this);
	this->PT[87].specify_switch(13.74715, 51.03816, -1, "Weiche 8 - Ausfahrt Lenneplatz",  "W8", this);
	this->PT[88].specify_trackpos(13.74705, 51.03822, -1, "Trackpos 62",  "TP62", this);
	this->PT[89].specify_trackpos(13.74696, 51.03833, -1, "Trackpos 63",  "TP63", this);
	this->PT[90].specify_trackpos(13.74698, 51.03848, -1, "Trackpos 64",  "TP64", this);
	this->PT[91].specify_trackpos(13.74705, 51.03856, -1, "Trackpos 65",  "TP65", this);
	this->PT[92].specify_switch(13.74713, 51.03862, -1, "Weiche 9 - Lennestrasse",  "W9", this);
	this->PT[93].specify_trackpos(13.74776, 51.039, -1, "Trackpos 66",  "TP66", this);
	this->PT[94].specify_trackpos(13.74794, 51.03916, -1, "Trackpos 67",  "TP67", this);
	this->PT[95].specify_regular(13.75186, 51.04307, 19,  this);
	this->PT[96].specify_regular(13.75461, 51.04578, 20,  this);
	this->PT[97].specify_switch(13.75469, 51.04585, -1, "Weiche 10",  "W10", this);
	this->PT[98].specify_trackpos(13.75504, 51.04623, -1, "Trackpos 68",  "TP68", this);
	this->PT[99].specify_trackpos(13.75522, 51.04647, -1, "Trackpos 69",  "TP69", this);
	this->PT[100].specify_trackpos(13.7568, 51.049, -1, "Trackpos 70",  "TP70", this);
	this->PT[101].specify_trackpos(13.75719, 51.04974, -1, "Trackpos 71",  "TP71", this);
	this->PT[102].specify_regular(13.75733, 51.05015, 21,  this);
	this->PT[103].specify_regular(13.75792, 51.05184, 22,  this);
	this->PT[104].specify_trackpos(13.75807, 51.05226, -1, "Trackpos 72",  "TP72", this);
	this->PT[105].specify_trackpos(13.75829, 51.05284, -1, "Trackpos 73",  "TP73", this);
	this->PT[106].specify_trackpos(13.75836, 51.05308, -1, "Trackpos 74",  "TP74", this);
	this->PT[107].specify_switch(13.75839, 51.05329, -1, "Weiche 11 - Gnzstrae",  "W11", this);
	this->PT[108].specify_trackpos(13.75834, 51.05352, -1, "Trackpos 75",  "TP75", this);
	this->PT[109].specify_trackpos(13.75827, 51.05372, -1, "Trackpos 76",  "TP76", this);
	this->PT[110].specify_trackpos(13.75815, 51.0539, -1, "Trackpos 77",  "TP77", this);
	this->PT[111].specify_switch(13.75801, 51.05405, -1, "Weiche 12 - Gnzplatz",  "W12", this);
	this->PT[112].specify_regular(13.75708, 51.05489, 23,  this);
	this->PT[113].specify_trackpos(13.75324, 51.05842, -1, "Trackpos 78",  "TP78", this);
	this->PT[114].specify_trackpos(13.75315, 51.05853, -1, "Trackpos 79",  "TP79", this);
	this->PT[115].specify_trackpos(13.75305, 51.05869, -1, "Trackpos 80",  "TP80", this);
	this->PT[116].specify_regular(13.75281, 51.05959, 24,  this);
	this->PT[117].specify_switch(13.75276, 51.05977, -1, "Weiche 13",  "W13", this);
	this->PT[118].specify_switch(13.75271, 51.05994, -1, "Weiche 14",  "W14", this);
	this->PT[119].specify_regular(13.75193, 51.06267, 25,  this);
	this->PT[120].specify_switch(13.75192, 51.06272, -1, "Weiche 15",  "W15", this);
	this->PT[121].specify_switch(13.7519, 51.06275, -1, "Weiche 16",  "W16", this);
	this->PT[122].specify_trackpos(13.75184, 51.06303, -1, "Trackpos 81",  "TP81", this);
	this->PT[123].specify_trackpos(13.75189, 51.06322, -1, "Trackpos 82",  "TP82", this);
	this->PT[124].specify_regular(13.75364, 51.0659, 26,  this);
	this->PT[125].specify_trackpos(13.75376, 51.06609, -1, "Trackpos 83",  "TP83", this);
	this->PT[126].specify_switch(13.75378, 51.06621, -1, "Weiche 17",  "W17", this);
	this->PT[127].specify_trackpos(13.75385, 51.06646, -1, "Trackpos 84",  "TP84", this);
	this->PT[128].specify_switch(13.75599, 51.06961, -1, "Weiche 18",  "W18", this);
	this->PT[129].specify_trackpos(13.75603, 51.06967, -1, "Trackpos 85",  "TP85", this);
	this->PT[130].specify_trackpos(13.75601, 51.06977, -1, "Trackpos 86",  "TP86", this);
	this->PT[131].specify_trackpos(13.75586, 51.06984, -1, "Trackpos 87",  "TP87", this);
	this->PT[132].specify_regular(13.75514, 51.07006, 27,  this);
	this->PT[133].specify_regular(13.75111, 51.07128, 28,  this);
	this->PT[134].specify_trackpos(13.75083, 51.07137, -1, "Trackpos 88",  "TP88", this);
	this->PT[135].specify_trackpos(13.75052, 51.07144, -1, "Trackpos 89",  "TP89", this);
	this->PT[136].specify_switch(13.75035, 51.07147, -1, "Weiche 19",  "W19", this);
	this->PT[137].specify_regular(13.74686, 51.07203, 29,  this);
	this->PT[138].specify_trackpos(13.74529, 51.07227, -1, "Trackpos 90",  "TP90", this);
	this->PT[139].specify_trackpos(13.74496, 51.07234, -1, "Trackpos 91",  "TP91", this);
	this->PT[140].specify_regular(13.7406, 51.07356, 30,  this);
	this->PT[141].specify_switch(13.73459, 51.07524, -1, "Weiche 20 - Abzweig Fritz-Reuter-Str",  "W20", this);
	this->PT[142].specify_trackpos(13.7339, 51.07543, -1, "Trackpos 92",  "TP92", this);
	this->PT[143].specify_trackpos(13.73378, 51.07548, -1, "Trackpos 93",  "TP93", this);
	this->PT[144].specify_trackpos(13.73363, 51.07557, -1, "Trackpos 94",  "TP94", this);
	this->PT[145].specify_switch(13.73355, 51.07565, -1, "Weiche 21",  "W21", this);
	this->PT[146].specify_regular(13.73289, 51.07689, 31,  this);
	this->PT[147].specify_switch(13.73284, 51.07701, -1, "Weiche 22 - Liststr. Ausfahrt",  "W22", this);
	this->PT[148].specify_trackpos(13.7327, 51.07713, -1, "Trackpos 95",  "TP95", this);
	this->PT[149].specify_trackpos(13.73245, 51.07725, -1, "Trackpos 96",  "TP96", this);
	this->PT[150].specify_trackpos(13.73218, 51.07734, -1, "Trackpos 97",  "TP97", this);
	this->PT[151].specify_switch(13.7319, 51.07734, -1, "Weiche 23 - Einfahrt Harkortstr.",  "W23", this);
	this->PT[152].specify_trackpos(13.73175, 51.0773, -1, "Trackpos 98",  "TP98", this);
	this->PT[153].specify_trackpos(13.73155, 51.07722, -1, "Trackpos 99",  "TP99", this);
	this->PT[154].specify_trackpos(13.72961, 51.07639, -1, "Trackpos 100",  "TP100", this);
	this->PT[155].specify_trackpos(13.72952, 51.07636, -1, "Trackpos 101",  "TP101", this);
	this->PT[156].specify_trackpos(13.72931, 51.07634, -1, "Trackpos 102",  "TP102", this);
	this->PT[157].specify_trackpos(13.72912, 51.07635, -1, "Trackpos 103",  "TP103", this);
	this->PT[158].specify_regular(13.72572, 51.07692, 32,  this);
	this->PT[159].specify_trackpos(13.72504, 51.07701, -1, "Trackpos 104",  "TP104", this);
	this->PT[160].specify_trackpos(13.72346, 51.07733, -1, "Trackpos 105",  "TP105", this);
	this->PT[161].specify_trackpos(13.72316, 51.07736, -1, "Trackpos 106",  "TP106", this);
	this->PT[162].specify_regular(13.72171, 51.07741, 33,  this);
	this->PT[163].specify_trackpos(13.72143, 51.07742, -1, "Trackpos 107",  "TP107", this);
	this->PT[164].specify_trackpos(13.71976, 51.07738, -1, "Trackpos 108",  "TP108", this);
	this->PT[165].specify_trackpos(13.71958, 51.07739, -1, "Trackpos 109",  "TP109", this);
	this->PT[166].specify_trackpos(13.71884, 51.07745, -1, "Trackpos 110",  "TP110", this);
	this->PT[167].specify_trackpos(13.7183, 51.07752, -1, "Trackpos 111",  "TP111", this);
	this->PT[168].specify_trackpos(13.71805, 51.07754, -1, "Trackpos 112",  "TP112", this);
	this->PT[169].specify_regular(13.71765, 51.07752, 34,  this);
	this->PT[170].specify_trackpos(13.71707, 51.0775, -1, "Trackpos 113",  "TP113", this);
	this->PT[171].specify_trackpos(13.71692, 51.07751, -1, "Trackpos 114",  "TP114", this);
	this->PT[172].specify_trackpos(13.7168, 51.07753, -1, "Trackpos 115",  "TP115", this);
	this->PT[173].specify_switch(13.71666, 51.07757, -1, "Weiche 24",  "W24", this);
	this->PT[174].specify_trackpos(13.71619, 51.07779, -1, "Trackpos 116",  "TP116", this);
	this->PT[175].specify_trackpos(13.71564, 51.07813, -1, "Trackpos 117",  "TP117", this);
	this->PT[176].specify_trackpos(13.71531, 51.07845, -1, "Trackpos 118",  "TP118", this);
	this->PT[177].specify_switch(13.7147, 51.07942, -1, "Weiche 25 / Abzweig Sternstr.",  "W25", this);
	this->PT[178].specify_trackpos(13.71455, 51.07952, -1, "Trackpos 119",  "TP119", this);
	this->PT[179].specify_trackpos(13.71449, 51.07953, -1, "Trackpos 120",  "TP120", this);
	this->PT[180].specify_trackpos(13.71435, 51.07954, -1, "Trackpos 121",  "TP121", this);
	this->PT[181].specify_regular(13.7135, 51.07945, 35,  this);
	this->PT[182].specify_regular(13.70789, 51.07889, 36,  this);
	this->PT[183].specify_trackpos(13.70686, 51.0788, -1, "Trackpos 122",  "TP122", this);
	this->PT[184].specify_trackpos(13.7065, 51.07874, -1, "Trackpos 123",  "TP123", this);
	this->PT[185].specify_regular(13.70246, 51.07791, 37,  this);
	this->PT[186].specify_trackpos(13.70005, 51.07741, -1, "Trackpos 124",  "TP124", this);
	this->PT[187].specify_trackpos(13.69982, 51.07737, -1, "Trackpos 125",  "TP125", this);
	this->PT[188].specify_regular(13.69869, 51.07726, 38,  this);
	this->PT[189].specify_trackpos(13.69805, 51.07722, -1, "Trackpos 126",  "TP126", this);
	this->PT[190].specify_trackpos(13.69792, 51.07723, -1, "Trackpos 127",  "TP127", this);
	this->PT[191].specify_trackpos(13.69775, 51.07728, -1, "Trackpos 128",  "TP128", this);
	this->PT[192].specify_trackpos(13.69762, 51.07737, -1, "Trackpos 129",  "TP129", this);
	this->PT[193].specify_trackpos(13.69752, 51.07754, -1, "Trackpos 130",  "TP130", this);
	this->PT[194].specify_regular(13.69719, 51.07988, 39,  this);
	this->PT[195].specify_trackpos(13.69689, 51.08172, -1, "Trackpos 131",  "TP131", this);
	this->PT[196].specify_trackpos(13.69685, 51.08181, -1, "Trackpos 132",  "TP132", this);
	this->PT[197].specify_trackpos(13.69676, 51.08191, -1, "Trackpos 133",  "TP133", this);
	this->PT[198].specify_trackpos(13.69662, 51.08197, -1, "Trackpos 134",  "TP134", this);
	this->PT[199].specify_trackpos(13.69646, 51.08201, -1, "Trackpos 135",  "TP135", this);
	this->PT[200].specify_trackpos(13.69634, 51.08202, -1, "Trackpos 136",  "TP136", this);
	this->PT[201].specify_regular(13.69485, 51.08192, 40,  this);
	this->PT[202].specify_trackpos(13.69343, 51.08184, -1, "Trackpos 137",  "TP137", this);
	this->PT[203].specify_trackpos(13.69315, 51.08184, -1, "Trackpos 138",  "TP138", this);
	this->PT[204].specify_trackpos(13.69275, 51.08187, -1, "Trackpos 139",  "TP139", this);
	this->PT[205].specify_trackpos(13.69233, 51.08194, -1, "Trackpos 140",  "TP140", this);
	this->PT[206].specify_trackpos(13.69202, 51.08203, -1, "Trackpos 141",  "TP141", this);
	this->PT[207].specify_regular(13.69092, 51.08239, 41,  this);
	this->PT[208].specify_regular(13.68699, 51.08377, 42,  this);


	// kaditz ->prohlis

    this->PT[209].specify_depot(13.68724, 51.08341, 42,  this);
	this->PT[210].specify_trackpos(13.68742, 51.08343, -1, "Trackpos 142",  "TP142", this);
	this->PT[211].specify_trackpos(13.68768, 51.08343, -1, "Trackpos 143",  "TP143", this);
	this->PT[212].specify_switch(13.68779, 51.08341, -1, "Weiche 206",  "W26", this);
	this->PT[213].specify_regular(13.69121, 51.08226, 41,  this);
	this->PT[214].specify_trackpos(13.69201, 51.08199, -1, "Trackpos 144",  "TP144", this);
	this->PT[215].specify_trackpos(13.69235, 51.08191, -1, "Trackpos 145",  "TP145", this);
	this->PT[216].specify_trackpos(13.69276, 51.08183, -1, "Trackpos 146",  "TP146", this);
	this->PT[217].specify_trackpos(13.69316, 51.0818, -1, "Trackpos 147",  "TP147", this);
	this->PT[218].specify_trackpos(13.69344, 51.08181, -1, "Trackpos 148",  "TP148", this);
	this->PT[219].specify_regular(13.69579, 51.08195, 40,  this);
	this->PT[220].specify_trackpos(13.69634, 51.08198, -1, "Trackpos 149",  "TP149", this);
	this->PT[221].specify_trackpos(13.69646, 51.08198, -1, "Trackpos 150",  "TP150", this);
	this->PT[222].specify_trackpos(13.69657, 51.08195, -1, "Trackpos 151",  "TP151", this);
	this->PT[223].specify_trackpos(13.69669, 51.0819, -1, "Trackpos 152",  "TP152", this);
	this->PT[224].specify_trackpos(13.69679, 51.08182, -1, "Trackpos 153",  "TP153", this);
	this->PT[225].specify_trackpos(13.69683, 51.08172, -1, "Trackpos 154",  "TP154", this);
	this->PT[226].specify_regular(13.69718, 51.0795, 39,  this);
	this->PT[227].specify_trackpos(13.69747, 51.07753, -1, "Trackpos 155",  "TP155", this);
	this->PT[228].specify_trackpos(13.69754, 51.07739, -1, "Trackpos 156",  "TP156", this);
	this->PT[229].specify_trackpos(13.69766, 51.07729, -1, "Trackpos 157",  "TP157", this);
	this->PT[230].specify_trackpos(13.69781, 51.07723, -1, "Trackpos 158",  "TP158", this);
	this->PT[231].specify_trackpos(13.69793, 51.07721, -1, "Trackpos 159",  "TP159", this);
	this->PT[232].specify_trackpos(13.69805, 51.0772, -1, "Trackpos 160",  "TP160", this);
	this->PT[233].specify_trackpos(13.69825, 51.0772, -1, "Trackpos 161",  "TP161", this);
	this->PT[234].specify_regular(13.6989, 51.07725, 38,  this);
	this->PT[235].specify_trackpos(13.70001, 51.07738, -1, "Trackpos 162",  "TP162", this);
	this->PT[236].specify_regular(13.7033, 51.07805, 37,  this);
	this->PT[237].specify_trackpos(13.70648, 51.07871, -1, "Trackpos 163",  "TP163", this);
	this->PT[238].specify_trackpos(13.70701, 51.07878, -1, "Trackpos 164",  "TP164", this);
	this->PT[239].specify_regular(13.70807, 51.07888, 36,  this);
	this->PT[240].specify_regular(13.7138, 51.07944, 35,  this);
	this->PT[241].specify_trackpos(13.71431, 51.0795, -1, "Trackpos 165",  "TP165", this);
	this->PT[242].specify_trackpos(13.71446, 51.07949, -1, "Trackpos 166",  "TP166", this);
	this->PT[243].specify_trackpos(13.71454, 51.07947, -1, "Trackpos 167",  "TP167", this);
	this->PT[244].specify_trackpos(13.71462, 51.07944, -1, "Trackpos 168",  "TP168", this);
	this->PT[245].specify_switch(13.71468, 51.07939, -1, "Weiche 27",  "W27", this);
	this->PT[246].specify_trackpos(13.71532, 51.07839, -1, "Trackpos 169",  "TP169", this);
	this->PT[247].specify_trackpos(13.71561, 51.07811, -1, "Trackpos 170",  "TP170", this);
	this->PT[248].specify_trackpos(13.71597, 51.07788, -1, "Trackpos 171",  "TP171", this);
	this->PT[249].specify_trackpos(13.71618, 51.07776, -1, "Trackpos 172",  "TP172", this);
	this->PT[250].specify_switch(13.71667, 51.07754, -1, "Weiche 28",  "W28", this);
	this->PT[251].specify_trackpos(13.71693, 51.07748, -1, "Trackpos 173",  "TP173", this);
	this->PT[252].specify_trackpos(13.71711, 51.07747, -1, "Trackpos 174",  "TP174", this);
	this->PT[253].specify_regular(13.71766, 51.07749, 34,  this);
	this->PT[254].specify_trackpos(13.71806, 51.07752, -1, "Trackpos 175",  "TP175", this);
	this->PT[255].specify_trackpos(13.71885, 51.07741, -1, "Trackpos 176",  "TP176", this);
	this->PT[256].specify_trackpos(13.71976, 51.07736, -1, "Trackpos 177",  "TP177", this);
	this->PT[257].specify_trackpos(13.72119, 51.07739, -1, "Trackpos 178",  "TP178", this);
	this->PT[258].specify_regular(13.72176, 51.07737, 33,  this);
	this->PT[259].specify_trackpos(13.72302, 51.07734, -1, "Trackpos 179",  "TP179", this);
	this->PT[260].specify_trackpos(13.72336, 51.07732, -1, "Trackpos 180",  "TP180", this);
	this->PT[261].specify_trackpos(13.72503, 51.07698, -1, "Trackpos 181",  "TP181", this);
	this->PT[262].specify_regular(13.7258, 51.07685, 32,  this);
	this->PT[263].specify_trackpos(13.72921, 51.07631, -1, "Trackpos 182",  "TP182", this);
	this->PT[264].specify_trackpos(13.72943, 51.07632, -1, "Trackpos 183",  "TP183", this);
	this->PT[265].specify_trackpos(13.72967, 51.07638, -1, "Trackpos 184",  "TP184", this);
	this->PT[266].specify_trackpos(13.73174, 51.07726, -1, "Trackpos 185",  "TP185", this);
	this->PT[267].specify_switch(13.73192, 51.07731, -1, "Weiche 29 / Harkortstr",  "W29", this);
	this->PT[268].specify_trackpos(13.73208, 51.07732, -1, "Trackpos 186",  "TP186", this);
	this->PT[269].specify_trackpos(13.7323, 51.07728, -1, "Trackpos 187",  "TP187", this);
	this->PT[270].specify_trackpos(13.73259, 51.07717, -1, "Trackpos 188",  "TP188", this);
	this->PT[271].specify_trackpos(13.73268, 51.07711, -1, "Trackpos 189",  "TP189", this);
	this->PT[272].specify_switch(13.73281, 51.07698, -1, "Weiche 30 /Harkort/Groenhainer",  "W30", this);
	this->PT[273].specify_regular(13.73304, 51.07647, 31,  this);
	this->PT[274].specify_switch(13.73354, 51.0756, -1, "Weiche 31 / Abzwei in Fritz-Reuter",  "W31", this);
	this->PT[275].specify_trackpos(13.73373, 51.07548, -1, "Trackpos 190",  "TP190", this);
	this->PT[276].specify_trackpos(13.73383, 51.07543, -1, "Trackpos 191",  "TP191", this);
	this->PT[277].specify_trackpos(13.734, 51.07538, -1, "Trackpos 192",  "TP192", this);
	this->PT[278].specify_switch(13.73457, 51.07522, -1, "Weiche 32",  "W32", this);
	this->PT[279].specify_regular(13.74116, 51.07337, 30,  this);
	this->PT[280].specify_trackpos(13.74496, 51.07231, -1, "Trackpos 193",  "TP193", this);
	this->PT[281].specify_trackpos(13.74523, 51.07225, -1, "Trackpos 194",  "TP194", this);
	this->PT[282].specify_regular(13.74685, 51.072, 29,  this);
	this->PT[283].specify_regular(13.75015, 51.07146, 28,  this);
	this->PT[284].specify_switch(13.75021, 51.07145, -1, "Weiche 33 / Bischofsweg",  "W33", this);
	this->PT[285].specify_trackpos(13.7505, 51.0714, -1, "Trackpos 195",  "TP195", this);
	this->PT[286].specify_trackpos(13.75082, 51.07134, -1, "Trackpos 196",  "TP196", this);
	this->PT[287].specify_trackpos(13.75094, 51.07131, -1, "Trackpos 197",  "TP197", this);
	this->PT[288].specify_regular(13.75567, 51.06987, 27,  this);
	this->PT[289].specify_trackpos(13.75582, 51.06982, -1, "Trackpos 198",  "TP198", this);
	this->PT[290].specify_trackpos(13.75595, 51.06976, -1, "Trackpos 199",  "TP199", this);
	this->PT[291].specify_trackpos(13.75599, 51.06969, -1, "Trackpos 200",  "TP200", this);




	this->PT[292].specify_regular(13.75351, 51.06576, 26,  this);
	this->PT[293].specify_trackpos(13.75185, 51.06324, -1, "Trackpos 202",  "TP202", this);
	this->PT[294].specify_trackpos(13.75179, 51.06305, -1, "Trackpos 203",  "TP203", this);
	this->PT[295].specify_trackpos(13.75179, 51.06294, -1, "Trackpos 204",  "TP204", this);
	this->PT[296].specify_switch(13.75185, 51.06276, -1, "Weiche 34 / Einf. Von Osten",  "W34", this);
	this->PT[297].specify_switch(13.75186, 51.06272, -1, "Weiche 35 / Einf. Von Westen",  "W35", this);
	this->PT[298].specify_regular(13.75197, 51.06233, 25,  this);
	this->PT[299].specify_switch(13.75266, 51.05993, -1, "Weiche 36",  "W36", this);
	this->PT[300].specify_switch(13.75271, 51.05977, -1, "Weiche 37",  "W37", this);
	this->PT[301].specify_regular(13.75287, 51.05921, 24,  this);
	this->PT[302].specify_trackpos(13.75303, 51.05869, -1, "Trackpos 205",  "TP205", this);
	this->PT[303].specify_trackpos(13.7531, 51.05853, -1, "Trackpos 206",  "TP206", this);
	this->PT[304].specify_trackpos(13.75322, 51.05838, -1, "Trackpos 207",  "TP207", this);
	this->PT[305].specify_regular(13.75776, 51.05421, 23,  this);
	this->PT[306].specify_switch(13.75795, 51.05403, -1, "Weiche 38",  "W38", this);
	this->PT[307].specify_trackpos(13.75813, 51.05385, -1, "Trackpos 208",  "TP208", this);
	this->PT[308].specify_trackpos(13.75822, 51.05371, -1, "Trackpos 209",  "TP209", this);
	this->PT[309].specify_trackpos(13.75832, 51.05349, -1, "Trackpos 210",  "TP210", this);
	this->PT[310].specify_switch(13.75835, 51.0533, -1, "Weiche 39",  "W39", this);
	this->PT[311].specify_trackpos(13.75832, 51.05305, -1, "Trackpos 211",  "TP211", this);
	this->PT[312].specify_trackpos(13.75829, 51.05293, -1, "Trackpos 212",  "TP212", this);
	this->PT[313].specify_regular(13.75787, 51.05184, 22,  this);
	this->PT[314].specify_regular(13.75727, 51.05009, 21,  this);
	this->PT[315].specify_trackpos(13.75718, 51.04984, -1, "Trackpos 213",  "TP213", this);
	this->PT[316].specify_trackpos(13.7571, 51.04963, -1, "Trackpos 214",  "TP214", this);
	this->PT[317].specify_trackpos(13.75676, 51.049, -1, "Trackpos 215",  "TP215", this);
	this->PT[318].specify_regular(13.75515, 51.04645, 20,  this);
	this->PT[319].specify_trackpos(13.75501, 51.04623, -1, "Trackpos 216",  "TP216", this);
	this->PT[320].specify_switch(13.75471, 51.04592, -1, "Weiche 40 / Einf. Von SdOst",  "W40", this);
	this->PT[321].specify_regular(13.75152, 51.04276, 19,  this);
	this->PT[322].specify_trackpos(13.74788, 51.03915, -1, "Trackpos 217",  "TP217", this);
	this->PT[323].specify_trackpos(13.74773, 51.03903, -1, "Trackpos 218",  "TP218", this);
	this->PT[324].specify_trackpos(13.7475, 51.03888, -1, "Trackpos 219",  "TP219", this);
	this->PT[325].specify_switch(13.74711, 51.03865, -1, "Weiche 41",  "W41", this);
	this->PT[326].specify_trackpos(13.74702, 51.03858, -1, "Trackpos 220",  "TP220", this);
	this->PT[327].specify_trackpos(13.74694, 51.03849, -1, "Trackpos 221",  "TP221", this);
	this->PT[328].specify_trackpos(13.7469, 51.03837, -1, "Trackpos 222",  "TP222", this);
	this->PT[329].specify_trackpos(13.74692, 51.0383, -1, "Trackpos 223",  "TP223", this);
	this->PT[330].specify_trackpos(13.747, 51.03819, -1, "Trackpos 224",  "TP224", this);
	this->PT[331].specify_switch(13.74707, 51.03815, -1, "Weiche 42",  "W42", this);
	this->PT[332].specify_regular(13.74825, 51.03762, 18,  this);
	this->PT[333].specify_switch(13.7485, 51.03752, -1, "Weiche 43",  "W43", this);
	this->PT[334].specify_trackpos(13.74871, 51.03746, -1, "Trackpos 225",  "TP225", this);
	this->PT[335].specify_trackpos(13.74916, 51.03743, -1, "Trackpos 226",  "TP226", this);
	this->PT[336].specify_trackpos(13.75035, 51.03737, -1, "Trackpos 227",  "TP227", this);
	this->PT[337].specify_trackpos(13.75084, 51.03732, -1, "Trackpos 228",  "TP228", this);
	this->PT[338].specify_trackpos(13.75126, 51.03724, -1, "Trackpos 229",  "TP229", this);
	this->PT[339].specify_trackpos(13.7514, 51.03721, -1, "Trackpos 230",  "TP230", this);
	this->PT[340].specify_trackpos(13.75178, 51.03708, -1, "Trackpos 231",  "TP231", this);
	this->PT[341].specify_trackpos(13.75231, 51.03685, -1, "Trackpos 232",  "TP232", this);
	this->PT[342].specify_regular(13.75311, 51.03654, 17,  this);
	this->PT[343].specify_regular(13.75832, 51.03447, 16,  this);
	this->PT[344].specify_trackpos(13.76238, 51.03284, -1, "Trackpos 233",  "TP233", this);
	this->PT[345].specify_trackpos(13.76247, 51.03276, -1, "Trackpos 234",  "TP234", this);
	this->PT[346].specify_trackpos(13.76247, 51.0327, -1, "Trackpos 235",  "TP235", this);
	this->PT[347].specify_trackpos(13.76245, 51.03264, -1, "Trackpos 236",  "TP236", this);
	this->PT[348].specify_trackpos(13.76163, 51.03168, -1, "Trackpos 237",  "TP237", this);
	this->PT[349].specify_regular(13.76106, 51.03098, 15,  this);
	this->PT[350].specify_trackpos(13.75921, 51.0285, -1, "Trackpos 238",  "TP238", this);
	this->PT[351].specify_trackpos(13.75917, 51.02841, -1, "Trackpos 239",  "TP239", this);
	this->PT[352].specify_trackpos(13.75919, 51.0283, -1, "Trackpos 240",  "TP240", this);
	this->PT[353].specify_trackpos(13.75955, 51.02772, -1, "Trackpos 241",  "TP241", this);
	this->PT[354].specify_regular(13.75985, 51.02738, 14,  this);
	this->PT[355].specify_regular(13.76201, 51.02471, 13,  this);
	this->PT[356].specify_trackpos(13.76214, 51.02455, -1, "Trackpos 242",  "TP242", this);
	this->PT[357].specify_trackpos(13.76227, 51.02444, -1, "Trackpos 243",  "TP243", this);
	this->PT[358].specify_regular(13.76525, 51.0225, 12,  this);
	this->PT[359].specify_trackpos(13.76548, 51.02236, -1, "Trackpos 244",  "TP244", this);
	this->PT[360].specify_trackpos(13.76566, 51.0223, -1, "Trackpos 245",  "TP245", this);
	this->PT[361].specify_trackpos(13.76577, 51.02228, -1, "Trackpos 246",  "TP246", this);
	this->PT[362].specify_trackpos(13.76588, 51.02229, -1, "Trackpos 247",  "TP247", this);
	this->PT[363].specify_switch(13.76601, 51.02232, -1, "Weiche 44",  "W44", this);
	this->PT[369].specify_trackpos(13.7712, 51.02427, -1, "Trackpos 248",  "TP248", this);
	this->PT[365].specify_trackpos(13.76965, 51.0248, -1, "Trackpos 249",  "TP249", this);
	this->PT[366].specify_trackpos(13.76983, 51.02486, -1, "Trackpos 250",  "TP250", this);
	this->PT[367].specify_trackpos(13.77003, 51.02486, -1, "Trackpos 251",  "TP251", this);
	this->PT[368].specify_trackpos(13.77012, 51.02484, -1, "Trackpos 252",  "TP252", this);
	this->PT[364].specify_regular(13.76937, 51.02458, 11,  this);
	this->PT[370].specify_regular(13.77436, 51.02264, 10,  this);
	this->PT[371].specify_regular(13.78043, 51.01948, 9,  this);
	this->PT[372].specify_trackpos(13.78197, 51.01869, -1, "Trackpos 253",  "TP253", this);
	this->PT[373].specify_regular(13.78378, 51.01767, 8,  this);
    this->PT[374].specify_trackpos(13.78612, 51.01637, -1, "Trackpos 254",  "TP254", this);
	this->PT[375].specify_trackpos(13.78669, 51.01599, -1, "Trackpos 255",  "TP255", this);
	this->PT[376].specify_regular(13.78679, 51.0159, 7,  this);
	this->PT[377].specify_trackpos(13.78806, 51.0148, -1, "Trackpos 256",  "TP256", this);
	this->PT[378].specify_trackpos(13.78858, 51.01443, -1, "Trackpos 257",  "TP257", this);
	this->PT[379].specify_trackpos(13.78901, 51.01419, -1, "Trackpos 258",  "TP258", this);
	this->PT[380].specify_trackpos(13.7895, 51.01396, -1, "Trackpos 259",  "TP259", this);
	this->PT[381].specify_trackpos(13.78976, 51.01385, -1, "Trackpos 260",  "TP260", this);
	this->PT[382].specify_regular(13.79082, 51.0135, 6,  this);
	this->PT[383].specify_trackpos(13.79139, 51.01332, -1, "Trackpos 261",  "TP261", this);
	this->PT[384].specify_trackpos(13.79272, 51.01309, -1, "Trackpos 262",  "TP262", this);
	this->PT[385].specify_trackpos(13.79343, 51.01304, -1, "Trackpos 263",  "TP263", this);
	this->PT[386].specify_trackpos(13.79425, 51.01308, -1, "Trackpos 264",  "TP264", this);
	this->PT[387].specify_trackpos(13.79469, 51.01307, -1, "Trackpos 265",  "TP265", this);
	this->PT[388].specify_trackpos(13.79515, 51.01302, -1, "Trackpos 266",  "TP266", this);
	this->PT[389].specify_regular(13.79535, 51.01298, 5,  this);
	this->PT[390].specify_trackpos(13.79545, 51.01296, -1, "Trackpos 267",  "TP267", this);
	this->PT[391].specify_trackpos(13.79626, 51.01274, -1, "Trackpos 268",  "TP268", this);
	this->PT[392].specify_trackpos(13.79649, 51.0127, -1, "Trackpos 269",  "TP269", this);
	this->PT[393].specify_trackpos(13.79849, 51.01247, -1, "Trackpos 270",  "TP270", this);
	this->PT[394].specify_trackpos(13.79862, 51.01241, -1, "Trackpos 271",  "TP271", this);
	this->PT[395].specify_trackpos(13.79875, 51.01226, -1, "Trackpos 272",  "TP272", this);
	this->PT[396].specify_trackpos(13.79874, 51.01208, -1, "Trackpos 273",  "TP273", this);
	this->PT[397].specify_trackpos(13.79878, 51.01193, -1, "Trackpos 274",  "TP274", this);
	this->PT[398].specify_trackpos(13.79883, 51.01183, -1, "Trackpos 275",  "TP275", this);
	this->PT[399].specify_trackpos(13.80018, 51.01044, -1, "Trackpos 276",  "TP276", this);
	this->PT[400].specify_regular(13.80043, 51.01026, 4,  this);
	this->PT[401].specify_trackpos(13.80064, 51.01011, -1, "Trackpos 277",  "TP277", this);
	this->PT[402].specify_trackpos(13.80083, 51.00999, -1, "Trackpos 278",  "TP278", this);
	this->PT[403].specify_switch(13.80168, 51.00954, -1, "Weiche 45 / Abzw. BH0",  "W45", this);
	this->PT[404].specify_switch(13.80244, 51.00915, -1, "Weiche 46 / aus BH0",  "W46", this);
	this->PT[405].specify_switch(13.80334, 51.00872, -1, "Weiche 47",  "W47", this);
	this->PT[406].specify_trackpos(13.80344, 51.00865, -1, "Trackpos 279",  "TP279", this);
	this->PT[407].specify_trackpos(13.8035, 51.00854, -1, "Trackpos 280",  "TP280", this);
	this->PT[408].specify_trackpos(13.80349, 51.00848, -1, "Trackpos 281",  "TP281", this);
	this->PT[409].specify_switch(13.80344, 51.00841, -1, "Weiche 48",  "W48", this);
	this->PT[410].specify_regular(13.80266, 51.00784, 3,  this);
	this->PT[411].specify_regular(13.79908, 51.00523, 2,  this);
	this->PT[412].specify_trackpos(13.79829, 51.00465, -1, "Trackpos 282",  "TP282", this);
	this->PT[413].specify_trackpos(13.79805, 51.00438, -1, "Trackpos 283",  "TP283", this);
	this->PT[414].specify_trackpos(13.79795, 51.00416, -1, "Trackpos 284",  "TP284", this);
	this->PT[415].specify_trackpos(13.79793, 51.00404, -1, "Trackpos 285",  "TP285", this);
	this->PT[416].specify_trackpos(13.79806, 51.00253, -1, "Trackpos 286",  "TP286", this);
	this->PT[417].specify_trackpos(13.79804, 51.00237, -1, "Trackpos 287",  "TP287", this);
	this->PT[418].specify_regular(13.798, 51.0023, 1,  this);
	this->PT[419].specify_trackpos(13.79796, 51.00221, -1, "Trackpos 288",  "TP288", this);
	this->PT[420].specify_trackpos(13.79695, 51.00078, -1, "Trackpos 289",  "TP289", this);
	this->PT[421].specify_trackpos(13.79689, 51.00067, -1, "Trackpos 290",  "TP290", this);
	this->PT[422].specify_trackpos(13.79685, 51.00058, -1, "Trackpos 291",  "TP291", this);
	this->PT[423].specify_trackpos(13.79685, 51.00047, -1, "Trackpos 292",  "TP292", this);
	this->PT[424].specify_switch(13.79686, 51.00039, -1, "Weiche 49 / Einf. Prohlis",  "W49", this);
	this->PT[425].specify_trackpos(13.79688, 51.00031, -1, "Trackpos 293",  "TP293", this);
	this->PT[426].specify_trackpos(13.79694, 51.00022, -1, "Trackpos 294",  "TP294", this);
	this->PT[427].specify_trackpos(13.79701, 51.00015, -1, "Trackpos 295",  "TP295", this);
	this->PT[428].specify_trackpos(13.7971, 51.00007, -1, "Trackpos 296",  "TP296", this);
	this->PT[429].specify_trackpos(13.79721, 51, -1, "Trackpos 297",  "TP297", this);
	this->PT[430].specify_regular(13.79749, 50.99986, 0,  this);

	// Komplettierung Gleisschleife Prohlis

	// Verknpfung Ankunft und Abfahrt Auengleis
    this->PT[431].specify_trackpos(13.7981, 50.99955, 0, "Trackpoint 298",  "TP298", this);

	// Innengleis

	this->PT[432].specify_trackpos(13.79704, 51.00018, -1, "Trackpoint 299",  "TP299", this);
	this->PT[433].specify_regular(13.79749, 50.99992, 0,  this);
	this->PT[434].specify_trackpos(13.79813, 50.99962, -1, "Trackpoint 300",  "TP300", this);
	this->PT[435].specify_depot(13.79888, 50.99924, 0,  this);
	this->PT[436].specify_trackpos(13.7991, 50.99913, -1, "Trackpoint 301",  "TP301", this);
	this->PT[437].specify_trackpos(13.79934, 50.99908, -1, "Trackpoint 302",  "TP302", this);
	this->PT[438].specify_trackpos(13.79958, 50.99909, -1, "Trackpoint 303",  "TP303", this);
	this->PT[439].specify_trackpos(13.79974, 50.99915, -1, "Trackpoint 304",  "TP304", this);

	// Kaditz / Riegelplatz Auengleis

    this->PT[440].specify_trackpos(13.6867, 51.08387, -1, "Trackpoint 305",  "TP305", this);
	this->PT[441].specify_trackpos(13.68646, 51.08388, -1, "Trackpoint 306",  "TP306", this);
	this->PT[442].specify_trackpos(13.68634, 51.08384, -1, "Trackpoint 307",  "TP307", this);
	this->PT[443].specify_trackpos(13.68625, 51.08378, -1, "Trackpoint 308",  "TP308", this);
	this->PT[444].specify_switch(13.68619, 51.08369, -1, "W50 / Einfahrt Kaditz",  "W50", this);
	this->PT[445].specify_trackpos(13.68614, 51.0836, -1, "Trackpoint 309",  "TP309", this);
	this->PT[446].specify_trackpos(13.68613, 51.08353, -1, "Trackpoint 310",  "TP310", this);
	this->PT[447].specify_trackpos(13.68615, 51.08346, -1, "Trackpoint 311",  "TP311", this);
	this->PT[448].specify_trackpos(13.68625, 51.08339, -1, "Trackpoint 312",  "TP312", this);
	this->PT[449].specify_trackpos(13.68634, 51.08337, -1, "Trackpoint 313",  "TP313", this);
	this->PT[450].specify_trackpos(13.68647, 51.08335, 42, "Trackpoint 314",  "TP314", this);

    // Kaditz Innengleis
    this->PT[451].specify_trackpos(13.68618, 51.0836, -1, "Trackpoint 315",  "TP315", this);
	this->PT[452].specify_trackpos(13.68619, 51.08353, -1, "Trackpoint 316",  "TP316", this);
	this->PT[453].specify_trackpos(13.68625, 51.08347, -1, "Trackpoint 317",  "TP317", this);
	this->PT[454].specify_trackpos(13.68638, 51.08343, -1, "Trackpoint 318",  "TP318", this);
	this->PT[455].specify_trackpos(13.68654, 51.08343, -1, "Trackpoint 319",  "TP319", this);
	this->PT[456].specify_depot(13.68723, 51.08348, 42,  this);
	this->PT[457].specify_trackpos(13.68736, 51.08349, -1, "Trackpoint 320",  "TP320", this);
	this->PT[458].specify_trackpos(13.68758, 51.08348, -1, "Trackpoint 321",  "TP321", this);

	// korrigierte Fehler
	// berlappung an Knoten 372
    this->PT[459].specify_trackpos(13.78199, 51.01871, -1, "Trackpoint 322",  "TP322", this);
	// berlappung an Knoten 60
	this->PT[460].specify_trackpos(13.76709, 51.02287, -1, "Trackpoint 323",  "TP323", this);

    // points specification ends here
}

void BUSSIM_NETWORK::specify_arcs(void)
{
    // type in the arc specifications here

	this->ARC[0].specify(0, 0, 1);
	this->ARC[1].specify(1, 1, 2);
	this->ARC[2].specify(2, 2, 3);
	this->ARC[3].specify(3, 3, 4);
	this->ARC[4].specify(4, 4, 5);
	this->ARC[5].specify(5, 5, 6);
	this->ARC[6].specify(6, 6, 7);
	this->ARC[7].specify(7, 7, 8);
	this->ARC[8].specify(8, 8, 9);
	this->ARC[9].specify(9, 9, 10);
	this->ARC[10].specify(10, 10, 11);
	this->ARC[11].specify(11, 11, 12);
	this->ARC[12].specify(12, 12, 13);
	this->ARC[13].specify(13, 13, 14);
	this->ARC[14].specify(14, 14, 15);
	this->ARC[15].specify(15, 15, 16);
	this->ARC[16].specify(16, 16, 17);
	this->ARC[17].specify(17, 17, 18);
	this->ARC[18].specify(18, 18, 19);
	this->ARC[19].specify(19, 19, 20);
	this->ARC[20].specify(20, 20, 21);
	this->ARC[21].specify(21, 21, 22);
	this->ARC[22].specify(22, 22, 23);
	this->ARC[23].specify(23, 23, 24);
	this->ARC[24].specify(24, 24, 25);
	this->ARC[25].specify(25, 25, 26);
	this->ARC[26].specify(26, 26, 27);
	this->ARC[27].specify(27, 27, 28);
	this->ARC[28].specify(28, 28, 29);
	this->ARC[29].specify(29, 29, 30);
	this->ARC[30].specify(30, 30, 31);
	this->ARC[31].specify(31, 31, 32);
	this->ARC[32].specify(32, 32, 33);
	this->ARC[33].specify(33, 33, 34);
	this->ARC[34].specify(34, 34, 35);
	this->ARC[35].specify(35, 35, 36);
	this->ARC[36].specify(36, 36, 37);
	this->ARC[37].specify(37, 37, 38);
	this->ARC[38].specify(38, 38, 39);
	this->ARC[39].specify(39, 39, 40);
	this->ARC[40].specify(40, 40, 41);
	this->ARC[41].specify(41, 41, 42);
	this->ARC[42].specify(42, 42, 43);
	this->ARC[43].specify(43, 43, 44);
	this->ARC[44].specify(44, 44, 45);
	this->ARC[45].specify(45, 45, 46);
	this->ARC[46].specify(46, 46, 47);
	this->ARC[47].specify(47, 47, 48);
	this->ARC[48].specify(48, 48, 49);
	this->ARC[49].specify(49, 49, 50);
	this->ARC[50].specify(50, 50, 51);
	this->ARC[51].specify(51, 51, 52);
	this->ARC[52].specify(52, 52, 53);
	this->ARC[53].specify(53, 53, 54);
	this->ARC[54].specify(54, 54, 55);
	this->ARC[55].specify(55, 55, 56);
	this->ARC[56].specify(56, 56, 57);
	this->ARC[57].specify(57, 57, 58);
	this->ARC[58].specify(58, 58, 59);
	this->ARC[59].specify(59, 59, 60);
	this->ARC[60].specify(60, 60, 61);
	this->ARC[61].specify(61, 61, 62);
	this->ARC[62].specify(62, 62, 63);
	this->ARC[63].specify(63, 63, 64);
	this->ARC[64].specify(64, 64, 65);
	this->ARC[65].specify(65, 65, 66);
	this->ARC[66].specify(66, 66, 67);
	this->ARC[67].specify(67, 67, 68);
	this->ARC[68].specify(68, 68, 69);
	this->ARC[69].specify(69, 69, 70);
	this->ARC[70].specify(70, 70, 71);
	this->ARC[71].specify(71, 71, 72);
	this->ARC[72].specify(72, 72, 73);
	this->ARC[73].specify(73, 73, 74);
	this->ARC[74].specify(74, 74, 75);
	this->ARC[75].specify(75, 75, 76);
	this->ARC[76].specify(76, 76, 77);
	this->ARC[77].specify(77, 77, 78);
	this->ARC[78].specify(78, 78, 79);
	this->ARC[79].specify(79, 79, 80);
	this->ARC[80].specify(80, 80, 81);
	this->ARC[81].specify(81, 81, 82);
	this->ARC[82].specify(82, 82, 83);
	this->ARC[83].specify(83, 83, 84);
	this->ARC[84].specify(84, 84, 85);
	this->ARC[85].specify(85, 85, 86);
	this->ARC[86].specify(86, 86, 87);
	this->ARC[87].specify(87, 87, 88);
	this->ARC[88].specify(88, 88, 89);
	this->ARC[89].specify(89, 89, 90);
	this->ARC[90].specify(90, 90, 91);
	this->ARC[91].specify(91, 91, 92);
	this->ARC[92].specify(92, 92, 93);
	this->ARC[93].specify(93, 93, 94);
	this->ARC[94].specify(94, 94, 95);
	this->ARC[95].specify(95, 95, 96);
	this->ARC[96].specify(96, 96, 97);
	this->ARC[97].specify(97, 97, 98);
	this->ARC[98].specify(98, 98, 99);
	this->ARC[99].specify(99, 99, 100);
	this->ARC[100].specify(100, 100, 101);
	this->ARC[101].specify(101, 101, 102);
	this->ARC[102].specify(102, 102, 103);
	this->ARC[103].specify(103, 103, 104);
	this->ARC[104].specify(104, 104, 105);
	this->ARC[105].specify(105, 105, 106);
	this->ARC[106].specify(106, 106, 107);
	this->ARC[107].specify(107, 107, 108);
	this->ARC[108].specify(108, 108, 109);
	this->ARC[109].specify(109, 109, 110);
	this->ARC[110].specify(110, 110, 111);
	this->ARC[111].specify(111, 111, 112);
	this->ARC[112].specify(112, 112, 113);
	this->ARC[113].specify(113, 113, 114);
	this->ARC[114].specify(114, 114, 115);
	this->ARC[115].specify(115, 115, 116);
	this->ARC[116].specify(116, 116, 117);
	this->ARC[117].specify(117, 117, 118);
	this->ARC[118].specify(118, 118, 119);
	this->ARC[119].specify(119, 119, 120);
	this->ARC[120].specify(120, 120, 121);
	this->ARC[121].specify(121, 121, 122);
	this->ARC[122].specify(122, 122, 123);
	this->ARC[123].specify(123, 123, 124);
	this->ARC[124].specify(124, 124, 125);
	this->ARC[125].specify(125, 125, 126);
	this->ARC[126].specify(126, 126, 127);
	this->ARC[127].specify(127, 127, 128);
	this->ARC[128].specify(128, 128, 129);
	this->ARC[129].specify(129, 129, 130);
	this->ARC[130].specify(130, 130, 131);
	this->ARC[131].specify(131, 131, 132);
	this->ARC[132].specify(132, 132, 133);
	this->ARC[133].specify(133, 133, 134);
	this->ARC[134].specify(134, 134, 135);
	this->ARC[135].specify(135, 135, 136);
	this->ARC[136].specify(136, 136, 137);
	this->ARC[137].specify(137, 137, 138);
	this->ARC[138].specify(138, 138, 139);
	this->ARC[139].specify(139, 139, 140);
	this->ARC[140].specify(140, 140, 141);
	this->ARC[141].specify(141, 141, 142);
	this->ARC[142].specify(142, 142, 143);
	this->ARC[143].specify(143, 143, 144);
	this->ARC[144].specify(144, 144, 145);
	this->ARC[145].specify(145, 145, 146);
	this->ARC[146].specify(146, 146, 147);
	this->ARC[147].specify(147, 147, 148);
	this->ARC[148].specify(148, 148, 149);
	this->ARC[149].specify(149, 149, 150);
	this->ARC[150].specify(150, 150, 151);
	this->ARC[151].specify(151, 151, 152);
	this->ARC[152].specify(152, 152, 153);
	this->ARC[153].specify(153, 153, 154);
	this->ARC[154].specify(154, 154, 155);
	this->ARC[155].specify(155, 155, 156);
	this->ARC[156].specify(156, 156, 157);
	this->ARC[157].specify(157, 157, 158);
	this->ARC[158].specify(158, 158, 159);
	this->ARC[159].specify(159, 159, 160);
	this->ARC[160].specify(160, 160, 161);
	this->ARC[161].specify(161, 161, 162);
	this->ARC[162].specify(162, 162, 163);
	this->ARC[163].specify(163, 163, 164);
	this->ARC[164].specify(164, 164, 165);
	this->ARC[165].specify(165, 165, 166);
	this->ARC[166].specify(166, 166, 167);
	this->ARC[167].specify(167, 167, 168);
	this->ARC[168].specify(168, 168, 169);
	this->ARC[169].specify(169, 169, 170);
	this->ARC[170].specify(170, 170, 171);
	this->ARC[171].specify(171, 171, 172);
	this->ARC[172].specify(172, 172, 173);
	this->ARC[173].specify(173, 173, 174);
	this->ARC[174].specify(174, 174, 175);
	this->ARC[175].specify(175, 175, 176);
	this->ARC[176].specify(176, 176, 177);
	this->ARC[177].specify(177, 177, 178);
	this->ARC[178].specify(178, 178, 179);
	this->ARC[179].specify(179, 179, 180);
	this->ARC[180].specify(180, 180, 181);
	this->ARC[181].specify(181, 181, 182);
	this->ARC[182].specify(182, 182, 183);
	this->ARC[183].specify(183, 183, 184);
	this->ARC[184].specify(184, 184, 185);
	this->ARC[185].specify(185, 185, 186);
	this->ARC[186].specify(186, 186, 187);
	this->ARC[187].specify(187, 187, 188);
	this->ARC[188].specify(188, 188, 189);
	this->ARC[189].specify(189, 189, 190);
	this->ARC[190].specify(190, 190, 191);
	this->ARC[191].specify(191, 191, 192);
	this->ARC[192].specify(192, 192, 193);
	this->ARC[193].specify(193, 193, 194);
	this->ARC[194].specify(194, 194, 195);
	this->ARC[195].specify(195, 195, 196);
	this->ARC[196].specify(196, 196, 197);
	this->ARC[197].specify(197, 197, 198);
	this->ARC[198].specify(198, 198, 199);
	this->ARC[199].specify(199, 199, 200);
	this->ARC[200].specify(200, 200, 201);
	this->ARC[201].specify(201, 201, 202);
	this->ARC[202].specify(202, 202, 203);
	this->ARC[203].specify(203, 203, 204);
	this->ARC[204].specify(204, 204, 205);
	this->ARC[205].specify(205, 205, 206);
	this->ARC[206].specify(206, 206, 207);
	this->ARC[207].specify(207, 207, 208);

    this->ARC[208].specify(208, 209, 210);
	this->ARC[209].specify(209, 210, 211);
	this->ARC[210].specify(210, 211, 212);
	this->ARC[211].specify(211, 212, 213);
	this->ARC[212].specify(212, 213, 214);
	this->ARC[213].specify(213, 214, 215);
	this->ARC[214].specify(214, 215, 216);
	this->ARC[215].specify(215, 216, 217);
	this->ARC[216].specify(216, 217, 218);
	this->ARC[217].specify(217, 218, 219);
	this->ARC[218].specify(218, 219, 220);
	this->ARC[219].specify(219, 220, 221);
	this->ARC[220].specify(220, 221, 222);
	this->ARC[221].specify(221, 222, 223);
	this->ARC[222].specify(222, 223, 224);
	this->ARC[223].specify(223, 224, 225);
	this->ARC[224].specify(224, 225, 226);
	this->ARC[225].specify(225, 226, 227);
	this->ARC[226].specify(226, 227, 228);
	this->ARC[227].specify(227, 228, 229);
	this->ARC[228].specify(228, 229, 230);
	this->ARC[229].specify(229, 230, 231);
	this->ARC[230].specify(230, 231, 232);
	this->ARC[231].specify(231, 232, 233);
	this->ARC[232].specify(232, 233, 234);
	this->ARC[233].specify(233, 234, 235);
	this->ARC[234].specify(234, 235, 236);
	this->ARC[235].specify(235, 236, 237);
	this->ARC[236].specify(236, 237, 238);
	this->ARC[237].specify(237, 238, 239);
	this->ARC[238].specify(238, 239, 240);
	this->ARC[239].specify(239, 240, 241);
	this->ARC[240].specify(240, 241, 242);
	this->ARC[241].specify(241, 242, 243);
	this->ARC[242].specify(242, 243, 244);
	this->ARC[243].specify(243, 244, 245);
	this->ARC[244].specify(244, 245, 246);
	this->ARC[245].specify(245, 246, 247);
	this->ARC[246].specify(246, 247, 248);
	this->ARC[247].specify(247, 248, 249);
	this->ARC[248].specify(248, 249, 250);
	this->ARC[249].specify(249, 250, 251);
	this->ARC[250].specify(250, 251, 252);
	this->ARC[251].specify(251, 252, 253);
	this->ARC[252].specify(252, 253, 254);
	this->ARC[253].specify(253, 254, 255);
	this->ARC[254].specify(254, 255, 256);
	this->ARC[255].specify(255, 256, 257);
	this->ARC[256].specify(256, 257, 258);
	this->ARC[257].specify(257, 258, 259);
	this->ARC[258].specify(258, 259, 260);
	this->ARC[259].specify(259, 260, 261);
	this->ARC[260].specify(260, 261, 262);
	this->ARC[261].specify(261, 262, 263);
	this->ARC[262].specify(262, 263, 264);
	this->ARC[263].specify(263, 264, 265);
	this->ARC[264].specify(264, 265, 266);
	this->ARC[265].specify(265, 266, 267);
	this->ARC[266].specify(266, 267, 268);
	this->ARC[267].specify(267, 268, 269);
	this->ARC[268].specify(268, 269, 270);
	this->ARC[269].specify(269, 270, 271);
	this->ARC[270].specify(270, 271, 272);
	this->ARC[271].specify(271, 272, 273);
	this->ARC[272].specify(272, 273, 274);
	this->ARC[273].specify(273, 274, 275);
	this->ARC[274].specify(274, 275, 276);
	this->ARC[275].specify(275, 276, 277);
	this->ARC[276].specify(276, 277, 278);
	this->ARC[277].specify(277, 278, 279);
	this->ARC[278].specify(278, 279, 280);
	this->ARC[279].specify(279, 280, 281);
	this->ARC[280].specify(280, 281, 282);
	this->ARC[281].specify(281, 282, 283);
	this->ARC[282].specify(282, 283, 284);
	this->ARC[283].specify(283, 284, 285);
	this->ARC[284].specify(284, 285, 286);
	this->ARC[285].specify(285, 286, 287);
	this->ARC[286].specify(286, 287, 288);
	this->ARC[287].specify(287, 288, 289);
	this->ARC[288].specify(288, 289, 290);
	this->ARC[289].specify(289, 290, 291);
	this->ARC[290].specify(290, 292, 293);
	this->ARC[291].specify(291, 293, 294);
	this->ARC[292].specify(292, 294, 295);
	this->ARC[293].specify(293, 295, 296);
	this->ARC[294].specify(294, 296, 297);
	this->ARC[295].specify(295, 297, 298);
	this->ARC[296].specify(296, 298, 299);
	this->ARC[297].specify(297, 299, 300);
	this->ARC[298].specify(298, 300, 301);
	this->ARC[299].specify(299, 301, 302);
	this->ARC[300].specify(300, 302, 303);
	this->ARC[301].specify(301, 303, 304);
	this->ARC[302].specify(302, 304, 305);
	this->ARC[303].specify(303, 305, 306);
	this->ARC[304].specify(304, 306, 307);
	this->ARC[305].specify(305, 307, 308);
	this->ARC[306].specify(306, 308, 309);
	this->ARC[307].specify(307, 309, 310);
	this->ARC[308].specify(308, 310, 311);
	this->ARC[309].specify(309, 311, 312);
	this->ARC[310].specify(310, 312, 313);
	this->ARC[311].specify(311, 313, 314);
	this->ARC[312].specify(312, 314, 315);
	this->ARC[313].specify(313, 315, 316);
	this->ARC[314].specify(314, 316, 317);
	this->ARC[315].specify(315, 317, 318);
	this->ARC[316].specify(316, 318, 319);
	this->ARC[317].specify(317, 319, 320);
	this->ARC[318].specify(318, 320, 321);
	this->ARC[319].specify(319, 321, 322);
	this->ARC[320].specify(320, 322, 323);
	this->ARC[321].specify(321, 323, 324);
	this->ARC[322].specify(322, 324, 325);
	this->ARC[323].specify(323, 325, 326);
	this->ARC[324].specify(324, 326, 327);
	this->ARC[325].specify(325, 327, 328);
	this->ARC[326].specify(326, 328, 329);
	this->ARC[327].specify(327, 329, 330);
	this->ARC[328].specify(328, 330, 331);
	this->ARC[329].specify(329, 331, 332);
	this->ARC[330].specify(330, 332, 333);
	this->ARC[331].specify(331, 333, 334);
	this->ARC[332].specify(332, 334, 335);
	this->ARC[333].specify(333, 335, 336);
	this->ARC[334].specify(334, 336, 337);
	this->ARC[335].specify(335, 337, 338);
	this->ARC[336].specify(336, 338, 339);
	this->ARC[337].specify(337, 339, 340);
	this->ARC[338].specify(338, 340, 341);
	this->ARC[339].specify(339, 341, 342);
	this->ARC[340].specify(340, 342, 343);
	this->ARC[341].specify(341, 343, 344);
	this->ARC[342].specify(342, 344, 345);
	this->ARC[343].specify(343, 345, 346);
	this->ARC[344].specify(344, 346, 347);
	this->ARC[345].specify(345, 347, 348);
	this->ARC[346].specify(346, 348, 349);
	this->ARC[347].specify(347, 349, 350);
	this->ARC[348].specify(348, 350, 351);
	this->ARC[349].specify(349, 351, 352);
	this->ARC[350].specify(350, 352, 353);
	this->ARC[351].specify(351, 353, 354);
	this->ARC[352].specify(352, 354, 355);
	this->ARC[353].specify(353, 355, 356);
	this->ARC[354].specify(354, 356, 357);
	this->ARC[355].specify(355, 357, 358);
	this->ARC[356].specify(356, 358, 359);
	this->ARC[357].specify(357, 359, 360);
	this->ARC[358].specify(358, 360, 361);
	this->ARC[359].specify(359, 361, 362);
	this->ARC[360].specify(360, 362, 363);
	this->ARC[361].specify(361, 363, 364);
	this->ARC[362].specify(362, 364, 365);
	this->ARC[363].specify(363, 365, 366);
	this->ARC[364].specify(364, 366, 367);
	this->ARC[365].specify(365, 367, 368);
	this->ARC[366].specify(366, 368, 369);
	this->ARC[367].specify(367, 369, 370);
	this->ARC[368].specify(368, 370, 371);
	this->ARC[369].specify(369, 371, 372);
	this->ARC[370].specify(370, 372, 373);
	this->ARC[371].specify(371, 373, 374);
	this->ARC[372].specify(372, 374, 375);
	this->ARC[373].specify(373, 375, 376);
	this->ARC[374].specify(374, 376, 377);
	this->ARC[375].specify(375, 377, 378);
	this->ARC[376].specify(376, 378, 379);
	this->ARC[377].specify(377, 379, 380);
	this->ARC[378].specify(378, 380, 381);
	this->ARC[379].specify(379, 381, 382);
	this->ARC[380].specify(380, 382, 383);
	this->ARC[381].specify(381, 383, 384);
	this->ARC[382].specify(382, 384, 385);
	this->ARC[383].specify(383, 385, 386);
	this->ARC[384].specify(384, 386, 387);
	this->ARC[385].specify(385, 387, 388);
	this->ARC[386].specify(386, 388, 389);
	this->ARC[387].specify(387, 389, 390);
	this->ARC[388].specify(388, 390, 391);
	this->ARC[389].specify(389, 391, 392);
	this->ARC[390].specify(390, 392, 393);
	this->ARC[391].specify(391, 393, 394);
	this->ARC[392].specify(392, 394, 395);
	this->ARC[393].specify(393, 395, 396);
	this->ARC[394].specify(394, 396, 397);
	this->ARC[395].specify(395, 397, 398);
	this->ARC[396].specify(396, 398, 399);
	this->ARC[397].specify(397, 399, 400);
	this->ARC[398].specify(398, 400, 401);
	this->ARC[399].specify(399, 401, 402);
	this->ARC[400].specify(400, 402, 403);
	this->ARC[401].specify(401, 403, 404);
	this->ARC[402].specify(402, 404, 405);
	this->ARC[403].specify(403, 405, 406);
	this->ARC[404].specify(404, 406, 407);
	this->ARC[405].specify(405, 407, 408);
	this->ARC[406].specify(406, 408, 409);
	this->ARC[407].specify(407, 409, 410);
	this->ARC[408].specify(408, 410, 411);
	this->ARC[409].specify(409, 411, 412);
	this->ARC[410].specify(410, 412, 413);
	this->ARC[411].specify(411, 413, 414);
	this->ARC[412].specify(412, 414, 415);
	this->ARC[413].specify(413, 415, 416);
	this->ARC[414].specify(414, 416, 417);
	this->ARC[415].specify(415, 417, 418);
	this->ARC[416].specify(416, 418, 419);
	this->ARC[417].specify(417, 419, 420);
	this->ARC[418].specify(418, 420, 421);
	this->ARC[419].specify(419, 421, 422);
	this->ARC[420].specify(420, 422, 423);
	this->ARC[421].specify(421, 423, 424);
	this->ARC[422].specify(422, 424, 425);
	this->ARC[423].specify(423, 425, 426);
	this->ARC[424].specify(424, 426, 427);
	this->ARC[425].specify(425, 427, 428);
	this->ARC[426].specify(426, 428, 429);
	this->ARC[427].specify(427, 429, 430);
	this->ARC[428].specify(428, 128, 127);
	this->ARC[429].specify(429, 127, 126);
	this->ARC[430].specify(430, 291, 128);
	this->ARC[431].specify(431, 126, 292);

	// Komplettierung Gleisschleife Prohlis
	this->ARC[432].specify(432, 430, 431);
	this->ARC[433].specify(433, 431, 0);
	this->ARC[434].specify(434, 424, 432);
	this->ARC[435].specify(435, 432, 433);
	this->ARC[436].specify(436, 433, 434);
	this->ARC[437].specify(437, 434, 435);
	this->ARC[438].specify(438, 435, 436);
	this->ARC[439].specify(439, 436, 437);
	this->ARC[440].specify(440, 437, 438);
	this->ARC[441].specify(441, 438, 439);
	this->ARC[442].specify(442, 439, 4);

	// Kaditz Auengleis
    this->ARC[443].specify(443, 208, 440);
	this->ARC[444].specify(444, 440, 441);
	this->ARC[445].specify(445, 441, 442);
	this->ARC[446].specify(446, 442, 443);
	this->ARC[447].specify(447, 443, 444);
	this->ARC[448].specify(448, 444, 445);
	this->ARC[449].specify(449, 445, 446);
	this->ARC[450].specify(450, 446, 447);
	this->ARC[451].specify(451, 447, 448);
	this->ARC[452].specify(452, 448, 449);
	this->ARC[453].specify(453, 449, 450);
	this->ARC[454].specify(454, 450, 209);

	// Kaditz Innengleis
    this->ARC[455].specify(455, 444, 451);
	this->ARC[456].specify(456, 451, 452);
	this->ARC[457].specify(457, 452, 453);
	this->ARC[458].specify(458, 453, 454);
	this->ARC[459].specify(459, 454, 455);
	this->ARC[460].specify(460, 455, 456);
	this->ARC[461].specify(461, 456, 457);
	this->ARC[462].specify(462, 457, 458);
	this->ARC[463].specify(463, 458, 212);

    // Korrektur in der Nhe von Knoten 372
    this->ARC[52].specify(52, 52, 459);
	this->ARC[464].specify(464, 459, 53);

    // Korrektur in der Nhe von Knoten 60
	this->ARC[361].specify(361, 363, 460);
	this->ARC[465].specify(465, 460, 364);

    // arc specification ends here.
}

void BUSSIM_NETWORK::specify_lines(void)
{

    // type in your line specifications here
    this->LINE[0].configure(0,13,0.0,0.0,1.0,"Linie 13: Prohlis -> Kaditz");
	this->LINE[0].append_arc(this->ARC[433]);
	this->LINE[0].append_arc(this->ARC[0]);
	this->LINE[0].append_arc(this->ARC[1]);
	this->LINE[0].append_arc(this->ARC[2]);
	this->LINE[0].append_arc(this->ARC[3]);
	this->LINE[0].append_arc(this->ARC[4]);
	this->LINE[0].append_arc(this->ARC[5]);
	this->LINE[0].append_arc(this->ARC[6]);
	this->LINE[0].append_arc(this->ARC[7]);
	this->LINE[0].append_arc(this->ARC[8]);
	this->LINE[0].append_arc(this->ARC[9]);
	this->LINE[0].append_arc(this->ARC[10]);
	this->LINE[0].append_arc(this->ARC[11]);
	this->LINE[0].append_arc(this->ARC[12]);
	this->LINE[0].append_arc(this->ARC[13]);
	this->LINE[0].append_arc(this->ARC[14]);
	this->LINE[0].append_arc(this->ARC[15]);
	this->LINE[0].append_arc(this->ARC[16]);
	this->LINE[0].append_arc(this->ARC[17]);
	this->LINE[0].append_arc(this->ARC[18]);
	this->LINE[0].append_arc(this->ARC[19]);
	this->LINE[0].append_arc(this->ARC[20]);
	this->LINE[0].append_arc(this->ARC[21]);
	this->LINE[0].append_arc(this->ARC[22]);
	this->LINE[0].append_arc(this->ARC[23]);
	this->LINE[0].append_arc(this->ARC[24]);
	this->LINE[0].append_arc(this->ARC[25]);
	this->LINE[0].append_arc(this->ARC[26]);
	this->LINE[0].append_arc(this->ARC[27]);
	this->LINE[0].append_arc(this->ARC[28]);
	this->LINE[0].append_arc(this->ARC[29]);
	this->LINE[0].append_arc(this->ARC[30]);
	this->LINE[0].append_arc(this->ARC[31]);
	this->LINE[0].append_arc(this->ARC[32]);
	this->LINE[0].append_arc(this->ARC[33]);
	this->LINE[0].append_arc(this->ARC[34]);
	this->LINE[0].append_arc(this->ARC[35]);
	this->LINE[0].append_arc(this->ARC[36]);
	this->LINE[0].append_arc(this->ARC[37]);
	this->LINE[0].append_arc(this->ARC[38]);
	this->LINE[0].append_arc(this->ARC[39]);
	this->LINE[0].append_arc(this->ARC[40]);
	this->LINE[0].append_arc(this->ARC[41]);
	this->LINE[0].append_arc(this->ARC[42]);
	this->LINE[0].append_arc(this->ARC[43]);
	this->LINE[0].append_arc(this->ARC[44]);
	this->LINE[0].append_arc(this->ARC[45]);
	this->LINE[0].append_arc(this->ARC[46]);
	this->LINE[0].append_arc(this->ARC[47]);
	this->LINE[0].append_arc(this->ARC[48]);
	this->LINE[0].append_arc(this->ARC[49]);
	this->LINE[0].append_arc(this->ARC[50]);
	this->LINE[0].append_arc(this->ARC[51]);
	this->LINE[0].append_arc(this->ARC[52]);
	this->LINE[0].append_arc(this->ARC[464]);
	this->LINE[0].append_arc(this->ARC[53]);
	this->LINE[0].append_arc(this->ARC[54]);
	this->LINE[0].append_arc(this->ARC[55]);
	this->LINE[0].append_arc(this->ARC[56]);
	this->LINE[0].append_arc(this->ARC[57]);
	this->LINE[0].append_arc(this->ARC[58]);
	this->LINE[0].append_arc(this->ARC[59]);
	this->LINE[0].append_arc(this->ARC[60]);
	this->LINE[0].append_arc(this->ARC[61]);
	this->LINE[0].append_arc(this->ARC[62]);
	this->LINE[0].append_arc(this->ARC[63]);
	this->LINE[0].append_arc(this->ARC[64]);
	this->LINE[0].append_arc(this->ARC[65]);
	this->LINE[0].append_arc(this->ARC[66]);
	this->LINE[0].append_arc(this->ARC[67]);
	this->LINE[0].append_arc(this->ARC[68]);
	this->LINE[0].append_arc(this->ARC[69]);
	this->LINE[0].append_arc(this->ARC[70]);
	this->LINE[0].append_arc(this->ARC[71]);
	this->LINE[0].append_arc(this->ARC[72]);
	this->LINE[0].append_arc(this->ARC[73]);
	this->LINE[0].append_arc(this->ARC[74]);
	this->LINE[0].append_arc(this->ARC[75]);
	this->LINE[0].append_arc(this->ARC[76]);
	this->LINE[0].append_arc(this->ARC[77]);
	this->LINE[0].append_arc(this->ARC[78]);
	this->LINE[0].append_arc(this->ARC[79]);
	this->LINE[0].append_arc(this->ARC[80]);
	this->LINE[0].append_arc(this->ARC[81]);
	this->LINE[0].append_arc(this->ARC[82]);
	this->LINE[0].append_arc(this->ARC[83]);
	this->LINE[0].append_arc(this->ARC[84]);
	this->LINE[0].append_arc(this->ARC[85]);
	this->LINE[0].append_arc(this->ARC[86]);
	this->LINE[0].append_arc(this->ARC[87]);
	this->LINE[0].append_arc(this->ARC[88]);
	this->LINE[0].append_arc(this->ARC[89]);
	this->LINE[0].append_arc(this->ARC[90]);
	this->LINE[0].append_arc(this->ARC[91]);
	this->LINE[0].append_arc(this->ARC[92]);
	this->LINE[0].append_arc(this->ARC[93]);
	this->LINE[0].append_arc(this->ARC[94]);
	this->LINE[0].append_arc(this->ARC[95]);
	this->LINE[0].append_arc(this->ARC[96]);
	this->LINE[0].append_arc(this->ARC[97]);
	this->LINE[0].append_arc(this->ARC[98]);
	this->LINE[0].append_arc(this->ARC[99]);
	this->LINE[0].append_arc(this->ARC[100]);
	this->LINE[0].append_arc(this->ARC[101]);
	this->LINE[0].append_arc(this->ARC[102]);
	this->LINE[0].append_arc(this->ARC[103]);
	this->LINE[0].append_arc(this->ARC[104]);
	this->LINE[0].append_arc(this->ARC[105]);
	this->LINE[0].append_arc(this->ARC[106]);
	this->LINE[0].append_arc(this->ARC[107]);
	this->LINE[0].append_arc(this->ARC[108]);
	this->LINE[0].append_arc(this->ARC[109]);
	this->LINE[0].append_arc(this->ARC[110]);
	this->LINE[0].append_arc(this->ARC[111]);
	this->LINE[0].append_arc(this->ARC[112]);
	this->LINE[0].append_arc(this->ARC[113]);
	this->LINE[0].append_arc(this->ARC[114]);
	this->LINE[0].append_arc(this->ARC[115]);
	this->LINE[0].append_arc(this->ARC[116]);
	this->LINE[0].append_arc(this->ARC[117]);
	this->LINE[0].append_arc(this->ARC[118]);
	this->LINE[0].append_arc(this->ARC[119]);
	this->LINE[0].append_arc(this->ARC[120]);
	this->LINE[0].append_arc(this->ARC[121]);
	this->LINE[0].append_arc(this->ARC[122]);
	this->LINE[0].append_arc(this->ARC[123]);
	this->LINE[0].append_arc(this->ARC[124]);
	this->LINE[0].append_arc(this->ARC[125]);
	this->LINE[0].append_arc(this->ARC[126]);
	this->LINE[0].append_arc(this->ARC[127]);
	this->LINE[0].append_arc(this->ARC[128]);
	this->LINE[0].append_arc(this->ARC[129]);
	this->LINE[0].append_arc(this->ARC[130]);
	this->LINE[0].append_arc(this->ARC[131]);
	this->LINE[0].append_arc(this->ARC[132]);
	this->LINE[0].append_arc(this->ARC[133]);
	this->LINE[0].append_arc(this->ARC[134]);
	this->LINE[0].append_arc(this->ARC[135]);
	this->LINE[0].append_arc(this->ARC[136]);
	this->LINE[0].append_arc(this->ARC[137]);
	this->LINE[0].append_arc(this->ARC[138]);
	this->LINE[0].append_arc(this->ARC[139]);
	this->LINE[0].append_arc(this->ARC[140]);
	this->LINE[0].append_arc(this->ARC[141]);
	this->LINE[0].append_arc(this->ARC[142]);
	this->LINE[0].append_arc(this->ARC[143]);
	this->LINE[0].append_arc(this->ARC[144]);
	this->LINE[0].append_arc(this->ARC[145]);
	this->LINE[0].append_arc(this->ARC[146]);
	this->LINE[0].append_arc(this->ARC[147]);
	this->LINE[0].append_arc(this->ARC[148]);
	this->LINE[0].append_arc(this->ARC[149]);
	this->LINE[0].append_arc(this->ARC[150]);
	this->LINE[0].append_arc(this->ARC[151]);
	this->LINE[0].append_arc(this->ARC[152]);
	this->LINE[0].append_arc(this->ARC[153]);
	this->LINE[0].append_arc(this->ARC[154]);
	this->LINE[0].append_arc(this->ARC[155]);
	this->LINE[0].append_arc(this->ARC[156]);
	this->LINE[0].append_arc(this->ARC[157]);
	this->LINE[0].append_arc(this->ARC[158]);
	this->LINE[0].append_arc(this->ARC[159]);
	this->LINE[0].append_arc(this->ARC[160]);
	this->LINE[0].append_arc(this->ARC[161]);
	this->LINE[0].append_arc(this->ARC[162]);
	this->LINE[0].append_arc(this->ARC[163]);
	this->LINE[0].append_arc(this->ARC[164]);
	this->LINE[0].append_arc(this->ARC[165]);
	this->LINE[0].append_arc(this->ARC[166]);
	this->LINE[0].append_arc(this->ARC[167]);
	this->LINE[0].append_arc(this->ARC[168]);
	this->LINE[0].append_arc(this->ARC[169]);
	this->LINE[0].append_arc(this->ARC[170]);
	this->LINE[0].append_arc(this->ARC[171]);
	this->LINE[0].append_arc(this->ARC[172]);
	this->LINE[0].append_arc(this->ARC[173]);
	this->LINE[0].append_arc(this->ARC[174]);
	this->LINE[0].append_arc(this->ARC[175]);
	this->LINE[0].append_arc(this->ARC[176]);
	this->LINE[0].append_arc(this->ARC[177]);
	this->LINE[0].append_arc(this->ARC[178]);
	this->LINE[0].append_arc(this->ARC[179]);
	this->LINE[0].append_arc(this->ARC[180]);
	this->LINE[0].append_arc(this->ARC[181]);
	this->LINE[0].append_arc(this->ARC[182]);
	this->LINE[0].append_arc(this->ARC[183]);
	this->LINE[0].append_arc(this->ARC[184]);
	this->LINE[0].append_arc(this->ARC[185]);
	this->LINE[0].append_arc(this->ARC[186]);
	this->LINE[0].append_arc(this->ARC[187]);
	this->LINE[0].append_arc(this->ARC[188]);
	this->LINE[0].append_arc(this->ARC[189]);
	this->LINE[0].append_arc(this->ARC[190]);
	this->LINE[0].append_arc(this->ARC[191]);
	this->LINE[0].append_arc(this->ARC[192]);
	this->LINE[0].append_arc(this->ARC[193]);
	this->LINE[0].append_arc(this->ARC[194]);
	this->LINE[0].append_arc(this->ARC[195]);
	this->LINE[0].append_arc(this->ARC[196]);
	this->LINE[0].append_arc(this->ARC[197]);
	this->LINE[0].append_arc(this->ARC[198]);
	this->LINE[0].append_arc(this->ARC[199]);
	this->LINE[0].append_arc(this->ARC[200]);
	this->LINE[0].append_arc(this->ARC[201]);
	this->LINE[0].append_arc(this->ARC[202]);
	this->LINE[0].append_arc(this->ARC[203]);
	this->LINE[0].append_arc(this->ARC[204]);
	this->LINE[0].append_arc(this->ARC[205]);
	this->LINE[0].append_arc(this->ARC[206]);
	this->LINE[0].append_arc(this->ARC[207]);
	this->LINE[0].append_arc(this->ARC[443]);
	this->LINE[0].append_arc(this->ARC[444]);
	this->LINE[0].append_arc(this->ARC[445]);
	this->LINE[0].append_arc(this->ARC[446]);
	this->LINE[0].append_arc(this->ARC[447]);
	this->LINE[0].append_arc(this->ARC[448]);
	this->LINE[0].append_arc(this->ARC[449]);
	this->LINE[0].append_arc(this->ARC[450]);
	this->LINE[0].append_arc(this->ARC[451]);
	this->LINE[0].append_arc(this->ARC[452]);
	this->LINE[0].append_arc(this->ARC[453]);

    this->LINE[1].configure(1,13,0.0,1.0,1.0,"Linie 13: Kaditz -> Prohlis");
	this->LINE[1].append_arc(this->ARC[454]);
	this->LINE[1].append_arc(this->ARC[208]);
	this->LINE[1].append_arc(this->ARC[209]);
	this->LINE[1].append_arc(this->ARC[210]);
	this->LINE[1].append_arc(this->ARC[211]);
	this->LINE[1].append_arc(this->ARC[212]);
	this->LINE[1].append_arc(this->ARC[213]);
	this->LINE[1].append_arc(this->ARC[214]);
	this->LINE[1].append_arc(this->ARC[215]);
	this->LINE[1].append_arc(this->ARC[216]);
	this->LINE[1].append_arc(this->ARC[217]);
	this->LINE[1].append_arc(this->ARC[218]);
	this->LINE[1].append_arc(this->ARC[219]);
	this->LINE[1].append_arc(this->ARC[220]);
	this->LINE[1].append_arc(this->ARC[221]);
	this->LINE[1].append_arc(this->ARC[222]);
	this->LINE[1].append_arc(this->ARC[223]);
	this->LINE[1].append_arc(this->ARC[224]);
	this->LINE[1].append_arc(this->ARC[225]);
	this->LINE[1].append_arc(this->ARC[226]);
	this->LINE[1].append_arc(this->ARC[227]);
	this->LINE[1].append_arc(this->ARC[228]);
	this->LINE[1].append_arc(this->ARC[229]);
	this->LINE[1].append_arc(this->ARC[230]);
	this->LINE[1].append_arc(this->ARC[231]);
	this->LINE[1].append_arc(this->ARC[232]);
	this->LINE[1].append_arc(this->ARC[233]);
	this->LINE[1].append_arc(this->ARC[234]);
	this->LINE[1].append_arc(this->ARC[235]);
	this->LINE[1].append_arc(this->ARC[236]);
	this->LINE[1].append_arc(this->ARC[237]);
	this->LINE[1].append_arc(this->ARC[238]);
	this->LINE[1].append_arc(this->ARC[239]);
	this->LINE[1].append_arc(this->ARC[240]);
	this->LINE[1].append_arc(this->ARC[241]);
	this->LINE[1].append_arc(this->ARC[242]);
	this->LINE[1].append_arc(this->ARC[243]);
	this->LINE[1].append_arc(this->ARC[244]);
	this->LINE[1].append_arc(this->ARC[245]);
	this->LINE[1].append_arc(this->ARC[246]);
	this->LINE[1].append_arc(this->ARC[247]);
	this->LINE[1].append_arc(this->ARC[248]);
	this->LINE[1].append_arc(this->ARC[249]);
	this->LINE[1].append_arc(this->ARC[250]);
	this->LINE[1].append_arc(this->ARC[251]);
	this->LINE[1].append_arc(this->ARC[252]);
	this->LINE[1].append_arc(this->ARC[253]);
	this->LINE[1].append_arc(this->ARC[254]);
	this->LINE[1].append_arc(this->ARC[255]);
	this->LINE[1].append_arc(this->ARC[256]);
	this->LINE[1].append_arc(this->ARC[257]);
	this->LINE[1].append_arc(this->ARC[258]);
	this->LINE[1].append_arc(this->ARC[259]);
	this->LINE[1].append_arc(this->ARC[260]);
	this->LINE[1].append_arc(this->ARC[261]);
	this->LINE[1].append_arc(this->ARC[262]);
	this->LINE[1].append_arc(this->ARC[263]);
	this->LINE[1].append_arc(this->ARC[264]);
	this->LINE[1].append_arc(this->ARC[265]);
	this->LINE[1].append_arc(this->ARC[266]);
	this->LINE[1].append_arc(this->ARC[267]);
	this->LINE[1].append_arc(this->ARC[268]);
	this->LINE[1].append_arc(this->ARC[269]);
	this->LINE[1].append_arc(this->ARC[270]);
	this->LINE[1].append_arc(this->ARC[271]);
	this->LINE[1].append_arc(this->ARC[272]);
	this->LINE[1].append_arc(this->ARC[273]);
	this->LINE[1].append_arc(this->ARC[274]);
	this->LINE[1].append_arc(this->ARC[275]);
	this->LINE[1].append_arc(this->ARC[276]);
	this->LINE[1].append_arc(this->ARC[277]);
	this->LINE[1].append_arc(this->ARC[278]);
	this->LINE[1].append_arc(this->ARC[279]);
	this->LINE[1].append_arc(this->ARC[280]);
	this->LINE[1].append_arc(this->ARC[281]);
	this->LINE[1].append_arc(this->ARC[282]);
	this->LINE[1].append_arc(this->ARC[283]);
	this->LINE[1].append_arc(this->ARC[284]);
	this->LINE[1].append_arc(this->ARC[285]);
	this->LINE[1].append_arc(this->ARC[286]);
	this->LINE[1].append_arc(this->ARC[287]);
	this->LINE[1].append_arc(this->ARC[288]);
	this->LINE[1].append_arc(this->ARC[289]);
	this->LINE[1].append_arc(this->ARC[430]);
	this->LINE[1].append_arc(this->ARC[428]);
	this->LINE[1].append_arc(this->ARC[429]);
	this->LINE[1].append_arc(this->ARC[431]);
	this->LINE[1].append_arc(this->ARC[290]);
	this->LINE[1].append_arc(this->ARC[291]);
	this->LINE[1].append_arc(this->ARC[292]);
	this->LINE[1].append_arc(this->ARC[293]);
	this->LINE[1].append_arc(this->ARC[294]);
	this->LINE[1].append_arc(this->ARC[295]);
	this->LINE[1].append_arc(this->ARC[296]);
	this->LINE[1].append_arc(this->ARC[297]);
	this->LINE[1].append_arc(this->ARC[298]);
	this->LINE[1].append_arc(this->ARC[299]);
	this->LINE[1].append_arc(this->ARC[300]);
	this->LINE[1].append_arc(this->ARC[301]);
	this->LINE[1].append_arc(this->ARC[302]);
	this->LINE[1].append_arc(this->ARC[303]);
	this->LINE[1].append_arc(this->ARC[304]);
	this->LINE[1].append_arc(this->ARC[305]);
	this->LINE[1].append_arc(this->ARC[306]);
	this->LINE[1].append_arc(this->ARC[307]);
	this->LINE[1].append_arc(this->ARC[308]);
	this->LINE[1].append_arc(this->ARC[309]);
	this->LINE[1].append_arc(this->ARC[310]);
	this->LINE[1].append_arc(this->ARC[311]);
	this->LINE[1].append_arc(this->ARC[312]);
	this->LINE[1].append_arc(this->ARC[313]);
	this->LINE[1].append_arc(this->ARC[314]);
	this->LINE[1].append_arc(this->ARC[315]);
	this->LINE[1].append_arc(this->ARC[316]);
	this->LINE[1].append_arc(this->ARC[317]);
	this->LINE[1].append_arc(this->ARC[318]);
	this->LINE[1].append_arc(this->ARC[319]);
	this->LINE[1].append_arc(this->ARC[320]);
	this->LINE[1].append_arc(this->ARC[321]);
	this->LINE[1].append_arc(this->ARC[322]);
	this->LINE[1].append_arc(this->ARC[323]);
	this->LINE[1].append_arc(this->ARC[324]);
	this->LINE[1].append_arc(this->ARC[325]);
	this->LINE[1].append_arc(this->ARC[326]);
	this->LINE[1].append_arc(this->ARC[327]);
	this->LINE[1].append_arc(this->ARC[328]);
	this->LINE[1].append_arc(this->ARC[329]);
	this->LINE[1].append_arc(this->ARC[330]);
	this->LINE[1].append_arc(this->ARC[331]);
	this->LINE[1].append_arc(this->ARC[332]);
	this->LINE[1].append_arc(this->ARC[333]);
	this->LINE[1].append_arc(this->ARC[334]);
	this->LINE[1].append_arc(this->ARC[335]);
	this->LINE[1].append_arc(this->ARC[336]);
	this->LINE[1].append_arc(this->ARC[337]);
	this->LINE[1].append_arc(this->ARC[338]);
	this->LINE[1].append_arc(this->ARC[339]);
	this->LINE[1].append_arc(this->ARC[340]);
	this->LINE[1].append_arc(this->ARC[341]);
	this->LINE[1].append_arc(this->ARC[342]);
	this->LINE[1].append_arc(this->ARC[343]);
	this->LINE[1].append_arc(this->ARC[344]);
	this->LINE[1].append_arc(this->ARC[345]);
	this->LINE[1].append_arc(this->ARC[346]);
	this->LINE[1].append_arc(this->ARC[347]);
	this->LINE[1].append_arc(this->ARC[348]);
	this->LINE[1].append_arc(this->ARC[349]);
	this->LINE[1].append_arc(this->ARC[350]);
	this->LINE[1].append_arc(this->ARC[351]);
	this->LINE[1].append_arc(this->ARC[352]);
	this->LINE[1].append_arc(this->ARC[353]);
	this->LINE[1].append_arc(this->ARC[354]);
	this->LINE[1].append_arc(this->ARC[355]);
	this->LINE[1].append_arc(this->ARC[356]);
	this->LINE[1].append_arc(this->ARC[357]);
	this->LINE[1].append_arc(this->ARC[358]);
	this->LINE[1].append_arc(this->ARC[359]);
	this->LINE[1].append_arc(this->ARC[360]);
	this->LINE[1].append_arc(this->ARC[361]);
	this->LINE[1].append_arc(this->ARC[465]);
	this->LINE[1].append_arc(this->ARC[362]);
	this->LINE[1].append_arc(this->ARC[363]);
	this->LINE[1].append_arc(this->ARC[364]);
	this->LINE[1].append_arc(this->ARC[365]);
	this->LINE[1].append_arc(this->ARC[366]);
	this->LINE[1].append_arc(this->ARC[367]);
	this->LINE[1].append_arc(this->ARC[368]);
	this->LINE[1].append_arc(this->ARC[369]);
	this->LINE[1].append_arc(this->ARC[370]);
	this->LINE[1].append_arc(this->ARC[371]);
	this->LINE[1].append_arc(this->ARC[372]);
	this->LINE[1].append_arc(this->ARC[373]);
	this->LINE[1].append_arc(this->ARC[374]);
	this->LINE[1].append_arc(this->ARC[375]);
	this->LINE[1].append_arc(this->ARC[376]);
	this->LINE[1].append_arc(this->ARC[377]);
	this->LINE[1].append_arc(this->ARC[378]);
	this->LINE[1].append_arc(this->ARC[379]);
	this->LINE[1].append_arc(this->ARC[380]);
	this->LINE[1].append_arc(this->ARC[381]);
	this->LINE[1].append_arc(this->ARC[382]);
	this->LINE[1].append_arc(this->ARC[383]);
	this->LINE[1].append_arc(this->ARC[384]);
	this->LINE[1].append_arc(this->ARC[385]);
	this->LINE[1].append_arc(this->ARC[386]);
	this->LINE[1].append_arc(this->ARC[387]);
	this->LINE[1].append_arc(this->ARC[388]);
	this->LINE[1].append_arc(this->ARC[389]);
	this->LINE[1].append_arc(this->ARC[390]);
	this->LINE[1].append_arc(this->ARC[391]);
	this->LINE[1].append_arc(this->ARC[392]);
	this->LINE[1].append_arc(this->ARC[393]);
	this->LINE[1].append_arc(this->ARC[394]);
	this->LINE[1].append_arc(this->ARC[395]);
	this->LINE[1].append_arc(this->ARC[396]);
	this->LINE[1].append_arc(this->ARC[397]);
	this->LINE[1].append_arc(this->ARC[398]);
	this->LINE[1].append_arc(this->ARC[399]);
	this->LINE[1].append_arc(this->ARC[400]);
	this->LINE[1].append_arc(this->ARC[401]);
	this->LINE[1].append_arc(this->ARC[402]);
	this->LINE[1].append_arc(this->ARC[403]);
	this->LINE[1].append_arc(this->ARC[404]);
	this->LINE[1].append_arc(this->ARC[405]);
	this->LINE[1].append_arc(this->ARC[406]);
	this->LINE[1].append_arc(this->ARC[407]);
	this->LINE[1].append_arc(this->ARC[408]);
	this->LINE[1].append_arc(this->ARC[409]);
	this->LINE[1].append_arc(this->ARC[410]);
	this->LINE[1].append_arc(this->ARC[411]);
	this->LINE[1].append_arc(this->ARC[412]);
	this->LINE[1].append_arc(this->ARC[413]);
	this->LINE[1].append_arc(this->ARC[414]);
	this->LINE[1].append_arc(this->ARC[415]);
	this->LINE[1].append_arc(this->ARC[416]);
	this->LINE[1].append_arc(this->ARC[417]);
	this->LINE[1].append_arc(this->ARC[418]);
	this->LINE[1].append_arc(this->ARC[419]);
	this->LINE[1].append_arc(this->ARC[420]);
	this->LINE[1].append_arc(this->ARC[421]);
	this->LINE[1].append_arc(this->ARC[422]);
	this->LINE[1].append_arc(this->ARC[423]);
	this->LINE[1].append_arc(this->ARC[424]);
	this->LINE[1].append_arc(this->ARC[425]);
	this->LINE[1].append_arc(this->ARC[426]);
	this->LINE[1].append_arc(this->ARC[427]);
	this->LINE[1].append_arc(this->ARC[432]);

 }

 void BUSSIM_NETWORK::specify_lines_intern(void)
{
    char LN[256];

    // specify the deployment lines
    for( int v=0 ; v < this->VEHICLES ; v++ )
    {
        sprintf(LN,"deployment line of vehicle %d",v);
        this->LINE[this->EXT_LINES + v].configure(this->EXT_LINES + v,-1,1.0,1.0,1.0,LN);
        this->LINE[this->EXT_LINES + v].type = BUSSIM_DEPLOYMENTLINE;
        //exit(0);
    }
    // specify the parking lines
    for( int v=0 ; v < this->VEHICLES ; v++ )
    {
        sprintf(LN,"parking line of vehicle %d",v);
        this->LINE[this->EXT_LINES + this->VEHICLES + v].configure(this->EXT_LINES + this->VEHICLES + v,-1,1.0,1.0,1.0,LN);
        this->LINE[this->EXT_LINES + this->VEHICLES + v].type = BUSSIM_PARKINGLINE;
    }

 }

void BUSSIM_NETWORK::specify_vehicles(void)
{
    // this procedure is the place where the vehicle specifications (ID as well as velocity) are specified

    // specify the available vehicles here
    this->VEHICLE[0].configure(0,BUSSIM_VEHCAT_TRAM,20,100,this->ARC[433], this->ARC[432], this);
    this->VEHICLE[1].configure(1,BUSSIM_VEHCAT_TRAM,20,100,this->ARC[454], this->ARC[453], this);

}

void BUSSIM_NETWORK::specify(void)
{
    // this procedure is used to define the network

    // define the network stops
    this->specify_stops();
    this->check_stop_consistency();

    // define the network nodes
    this->specify_nodes_longlat();
    this->check_node_consistency();

    // specify the arcs
    this->specify_arcs();

    // determine the arc length values
    for(int a=0 ; a < this->ARCS ; a++)
        this->ARC[a].update_length(this);

    // specify the tois
    this->specify_tois();

    // determine the world_xy-coordinates
    this->longlat_to_worldxy();

    // determine extreme coordinates
   for(int j=0 ; j < this->POINTS ; j++)
        {
            if( this->PT[j].world_x < this->xmin )
                this->xmin = this->PT[j].world_x;
            if( this->PT[j].world_x > this->xmax )
                this->xmax = this->PT[j].world_x;
            if( this->PT[j].world_y < this->ymin )
                this->ymin = this->PT[j].world_y;
            if( this->PT[j].world_y > this->ymax )
                this->ymax = this->PT[j].world_y;
        }

    for(int j=0 ; j < this->TOIS ; j++)
        {
            struct BUSSIM_TOI_POINT *Ptr;
            Ptr = this->TOI[j].PT;
            while( Ptr != nullptr )
            {
                if( Ptr->world_x < this->xmin )
                    this->xmin = Ptr->world_x;
                if( Ptr->world_x > this->xmax )
                    this->xmax = Ptr->world_x;
                if( Ptr->world_y < this->ymin )
                    this->ymin = Ptr->world_y;
                if( Ptr->world_y > this->ymax )
                    this->ymax = Ptr->world_y;

                Ptr = Ptr->next;
            }
        }



    // ensure that even the outer points are printed completely
    double x_os;
    x_os = (this->xmax - this->xmin) / 20.0;
    double y_os;
    y_os = (this->ymax - this->ymin) / 20.0;

    x_os = 0.0;
    y_os = 0.0;
    this->xmax = this->xmax+x_os;
    this->ymax = this->ymax+y_os;
    this->xmin = this->xmin-x_os;
    this->ymin = this->ymin-y_os;



    // set the screen coordinates
    for(int j=0 ; j < this->POINTS ; j++)
        this->PT[j].calculate_screen_coordinates(this,this->xmin, this->xmax, this->ymin, this->ymax);
    for(int j=0 ; j < this->TOIS ; j++)
        this->TOI[j].calculate_screen_coordinates(this,this->xmin, this->xmax, this->ymin, this->ymax);

    this->center_network();

    // print map information to command line
    std::cout << "bounding box of map area as background " << std::endl;
    std::cout << "WEST = " << this->center_longitude  -  (this->max_long - this->center_longitude)  * 1.0 / this->x_factor << std::endl;
    std::cout << "EAST = " << this->center_longitude  +  (this->max_long - this->center_longitude)  * 1.0 / this->x_factor << std::endl;
    std::cout << "SOUTH = " << this->center_latitude  -  (this->max_lat - this->center_latitude)  * 1.0 / this->y_factor << std::endl;
    std::cout << "NORTH = " << this->center_latitude  +  (this->max_lat - this->center_latitude)  * 1.0 / this->y_factor << std::endl;

    // specify the transfer arcs to realize PAX flows
    this->specify_transfer_arcs();

    if( this->ARCS > 0 )
        this->check_arc_consistency();
    else
        std::cout << "no arcs defined!" << std::endl;
    std::cout << "\n" << std::endl;

    // all nodes and arcs are specified => continue with the evaluations

    // transfer the screen coordinates from points into arcs
    for( int i=0 ; i < this->ARCS ; i++ )
    {
            this->ARC[i].orig_screen_x = this->PT[this->ARC[i].orig_node].screen_x;
            this->ARC[i].orig_screen_y = this->PT[this->ARC[i].orig_node].screen_y;
            this->ARC[i].dest_screen_x = this->PT[this->ARC[i].dest_node].screen_x;
            this->ARC[i].dest_screen_y = this->PT[this->ARC[i].dest_node].screen_y;
            this->ARC[i].update_length(this);
    }

    // specify the arc types
    for(int a=0 ; a < this->ARCS ; a++ )
            this->ARC[a].set_direction_type();

    // set the node labels
    for( int n=0 ; n < this->POINTS ; n++ )
        //if( this->PT[n].INFRASTRUCTURE < 1 )
            this->PT[n].set_label_pos_auto(this);


    // specify the blocks
    this->specify_blocks();

    // specify the lines
    this->specify_lines();
    this->specify_automatic_terminus_labels_of_lines();
    this->specify_lines_intern();
    for( int l=0 ; l < this->LINES ; l++ )
        this->LINE[l].check_consistency(this);

    // specify the vehicles
    this->specify_vehicles();

    // specify the vehicle activation times
    this->specify_vehicle_activation_times();

    // specify the vehicle rotations: assign the (so far) empty deployment line as first
    // line to a vehicle rotation
    this->specify_rotations_deployment_lines();

    // specify the vehicle rotations
    this->specify_rotations();

    // sepcify the parking lines
    this->specify_rotations_parking_lines();


    // specify the vehicle schedules
    this->specify_schedules();

    //for( int i=0 ; i < this->VEHICLES ; i++ )
    //    this->VEHICLE[i].SCHEDULE->print_on_shell(this);
    //exit(0);

    // analyze the lines
    this->merge_lines_arcs();

    // assign colors to stops
    this->set_stop_colors();

    // print out everything
    // this->print_stop_list();

    // connect arcs with stops
    std::cout << "find internal arcs ..." << std::endl;
    this->attach_internal_arcs_to_stops();
    std::cout << "... ok" << std::endl;

    // check if the vehicles are positioned on the first arc of their schedules
    this->inspect_initial_vehicle_positions();

    // check if the vehicles park on the last arc in their schedules
    this->inspect_final_vehicle_positions();

        //std::cout << "-----------------------" << std::endl;
    for( int l=0 ; l < this->LINES ; l++ )
    {
        this->LINE[l].check_consistency(this);
        //this->LINE[l].print_on_console(this);
    }

    //std::cout << "-----------------------" << std::endl;

    // this->VEHICLE[10].SCHEDULE->print_on_shell(this);
    // exit(0);

    // initialize the service matrix
    this->compile_service_matrix();

    // initialize the passenger lists
    this->specify_paxlists();

    //this->PAX_UNKNOWN->get_first_pax()->display_on_screen(this);
    //exit(0);

    // specify the platform positions in the ext. lines
    this->specify_platform_positions_in_lines();
}

void BUSSIM_NETWORK::display_active_vehicles(void)
{
    /*int window_width, window_height;

    window_width = glutGet(GLUT_WINDOW_WIDTH);
    window_height = glutGet(GLUT_WINDOW_HEIGHT);
    */

    double item_width, item_height;

    item_width = 0.35;
    item_height = 0.1;

    double horizontal_dist, vertical_dist;

    horizontal_dist = 0.05;
    vertical_dist = 0.025;

    double xpos, ypos;

    xpos = -0.95;
    ypos = 1 - 0.05;

    double CurRed=0.0, CurGreen=0.0, CurBlue=0.0;

    char S[256];

    double factor = 0.0;

    for( int v=0 ; v < this->VEHICLES ; v++ )
    {
        // set the display color
        if( this->VEHICLE[v].ACTIVE > 0)
        {
                    if( this->VEHICLE[v].on_line >= 0 )
                    {
                        CurRed = this->LINE[this->VEHICLE[v].on_line].RED;
                        CurGreen = this->LINE[this->VEHICLE[v].on_line].GREEN;
                        CurBlue = this->LINE[this->VEHICLE[v].on_line].BLUE;
                    }
        }
        else
        {
                    CurRed = 0.41;
                    CurGreen = 0.41;
                    CurBlue = 0.41;
        }
        glColor4f((float)(CurRed),(float)(CurGreen),(float)(CurBlue), 0.0f);

        // print the polygon
        if( 1 < 2 )
        {
            glBegin(GL_POLYGON);
                glVertex2f((float)(xpos),(float)(ypos));
                glVertex2f((float)(xpos+item_width),(float)(ypos));
                glVertex2f((float)(xpos+item_width),(float)(ypos-item_height));
                glVertex2f((float)(xpos),(float)(ypos-item_height));
            glEnd();
        }
        if( 2 > 1 )
        {
            glLineWidth(2.0f);
            glBegin(GL_LINE_LOOP);
                glVertex2f((float)(xpos),(float)(ypos));
                glVertex2f((float)(xpos+item_width),(float)(ypos));
                glVertex2f((float)(xpos+item_width),(float)(ypos-item_height));
                glVertex2f((float)(xpos),(float)(ypos-item_height));
            glEnd();
        }
        // print the vehicle number
        if( this->VEHICLE[v].ACTIVE < 1 )
            glColor4f((float)(CurRed*0.8),(float)(CurGreen*0.8),(float)(CurBlue*0.8), 0.0f);
        else
            glColor4f((float)(CurRed*0.2),(float)(CurGreen*0.2),(float)(CurBlue*0.2), 0.0f);
        glRasterPos2d(xpos + 0.05, ypos - 0.025);

        if( this->VEHICLE[v].vehicle_category == BUSSIM_VEHCAT_BUS )
            sprintf(S,"B%d",this->VEHICLE[v].ID);
        if( this->VEHICLE[v].vehicle_category == BUSSIM_VEHCAT_TRAM )
            sprintf(S,"T%d",this->VEHICLE[v].ID);

        unsigned char* ubuf = reinterpret_cast<unsigned char*>(S);
        glutBitmapString(GLUT_BITMAP_HELVETICA_12, ubuf);

        // print the capacity utilization

        glRasterPos2d(xpos + 0.1, ypos - 0.025);

        if( this->VEHICLE[v].capacity > 0 )
        {
            sprintf(S,"%8.0f%%",100.0*(double)(this->VEHICLE[v].PAX_ON_BOARD) / (double)(this->VEHICLE[v].capacity));
            unsigned char* ubuf = reinterpret_cast<unsigned char*>(S);
            glutBitmapString(GLUT_BITMAP_HELVETICA_12, ubuf);
        }

        // print capacity indicator bar

        if( this->VEHICLE[v].capacity > 0 )
        {
            // set the background
            glColor4f((float)(0),(float)(0),(float)(0), 0.0f);

            glBegin(GL_POLYGON);
                glVertex2f((float)(xpos+0.9*item_width),(float)(ypos));
                glVertex2f((float)(xpos+item_width),(float)(ypos));
                glVertex2f((float)(xpos+item_width),(float)(ypos-item_height));
                glVertex2f((float)(xpos+0.9*item_width),(float)(ypos-item_height));
            glEnd();

            if( (double)(this->VEHICLE[v].PAX_ON_BOARD) / (double)(this->VEHICLE[v].capacity) <= 0.6 )
            {
                glColor4f((float)(0.0),(float)(1.0),(float)(0), 0.0f);
            }
            else
            {
                if( (double)(this->VEHICLE[v].PAX_ON_BOARD) / (double)(this->VEHICLE[v].capacity) <= 0.8 )
                {
                    glColor4f((float)(1.0),(float)(1.0),(float)(0), 0.0f);
                }
                else
                {
                    glColor4f((float)(1.0),(float)(0.0),(float)(0), 0.0f);
                }
            }

            factor = 1.0 - (double)(this->VEHICLE[v].PAX_ON_BOARD) / (double)(this->VEHICLE[v].capacity);
            if( factor < 0 )
                factor = 0.0;

            // print the colored indicator bar
            glBegin(GL_POLYGON);
                glVertex2f((float)(xpos+0.9*item_width),(float)(ypos - factor*item_height));
                glVertex2f((float)(xpos+item_width),(float)(ypos - factor*item_height));
                glVertex2f((float)(xpos+item_width),(float)(ypos-item_height));
                glVertex2f((float)(xpos+0.9*item_width),(float)(ypos-item_height));
            glEnd();
        }

        // print the schedule progress bar on the left side
        if( this->VEHICLE[v].ACTIVE > 0 )
        {
            if( this->VEHICLE[v].SCHEDULE != nullptr )
            {
                // set the background
                glColor4f((float)(0),(float)(0),(float)(0), 0.0f);

                glBegin(GL_POLYGON);
                    glVertex2f((float)(xpos),(float)(ypos));
                    glVertex2f((float)(xpos+0.1*item_width),(float)(ypos));
                    glVertex2f((float)(xpos+0.1*item_width),(float)(ypos-item_height));
                    glVertex2f((float)(xpos),(float)(ypos-item_height));
                glEnd();



            if( this->VEHICLE[v].SCHEDULE->segments > 0 )
            {
                factor = (double)(this->VEHICLE[v].SCHEDULE->current_segment->ID - this->VEHICLE[v].SCHEDULE->first_segment->ID) /
                        (double)(this->VEHICLE[v].SCHEDULE->last_segment->ID - this->VEHICLE[v].SCHEDULE->first_segment->ID);

                factor = 1.0 - factor;

                // print the colored indicator bar
                glColor4f((float)(1.0),(float)(0.0),(float)(1.0), 0.0f);

                glBegin(GL_POLYGON);
                    glVertex2f((float)(xpos),(float)(ypos - factor*item_height));
                    glVertex2f((float)(xpos+0.1*item_width),(float)(ypos - factor*item_height));
                    glVertex2f((float)(xpos+0.1*item_width),(float)(ypos-item_height));
                    glVertex2f((float)(xpos),(float)(ypos-item_height));
                glEnd();
            }
            }
        }

        // print the terminus indicator
        if( this->VEHICLE[v].on_line >= 0 )
        {
            if( this->VEHICLE[v].on_line <= this->EXT_LINES )
            {
                // print the colored indicator bar
                if( this->LINE[this->VEHICLE[v].on_line].RED + this->LINE[this->VEHICLE[v].on_line].GREEN + this->LINE[this->VEHICLE[v].on_line].BLUE > 1.5 )
                    glColor4f((float)(0.0),(float)(0.0),(float)(0.0), 0.0f);
                else
                    glColor4f((float)(1.0),(float)(1.0),(float)(1.0), 0.0f);
                glRasterPos2d(xpos + 0.075, ypos - 0.075);
                sprintf(S,"%d-> %s",this->VEHICLE[v].on_line,this->LINE[this->VEHICLE[v].on_line].AUTO_LINE_NAME_DEST.c_str());
                unsigned char* ubuf_B = reinterpret_cast<unsigned char*>(S);

                glutBitmapString(GLUT_BITMAP_HELVETICA_12, ubuf_B);
            }
        }
        // update positions

        if( xpos + item_width + horizontal_dist + item_width <= 1 )
        {
            xpos += item_width + horizontal_dist;
        }
        else
        {
            xpos = -0.95;
            ypos = ypos - item_height - vertical_dist;
        }
    }
}

void BUSSIM_NETWORK::display_active_vehicles_orig(void)
{

    //glRasterPos2d(0.0, 0.95);

    char S[256];
    //sprintf(S,"active vehicles: ");

    //glColor4f(0.0f, 1.0f, 0.0f, 0.0f);
    //unsigned char* ubuf = reinterpret_cast<unsigned char*>(S);
    //glutBitmapString(GLUT_BITMAP_HELVETICA_12, ubuf);
    glColor4f(1.0f, 1.0f, 1.0f, 0.0f);
    double y_offset = 0.05;

    double CurRed, CurGreen, CurBlue;
    CurRed = 0.41;
    CurGreen = 0.41;
    CurBlue = 0.41;

    for(int i=0 ; i < this->VEHICLES ; i++ )
    {
        //if( this->VEHICLE[i].ACTIVE >= 0 )
          //  if( this->VEHICLE[i].VISIBLE > 0)
            {
                if( this->VEHICLE[i].ACTIVE > 0)
                {
                    if( this->VEHICLE[i].on_line >= 0 )
                    {
                        CurRed = this->LINE[this->VEHICLE[i].on_line].RED;
                        CurGreen = this->LINE[this->VEHICLE[i].on_line].GREEN;
                        CurBlue = this->LINE[this->VEHICLE[i].on_line].BLUE;
                    }
                }
                else
                {
                    CurRed = 0.41;
                    CurGreen = 0.41;
                    CurBlue = 0.41;
                }
                glColor4f((float)(CurRed),
                            (float)(CurGreen),
                            (float)(CurBlue), 0.0f);
                glRasterPos2d(-0.8,0.95 - y_offset);

                //char S[256];
                if( this->VEHICLE[i].WAIT_FOR_BLOCK < 1)
                {
                    if( this->VEHICLE[i].Thwarted < -10 )
                        sprintf(S,"V%d on arc %d (blocked %d)",this->VEHICLE[i].ID, this->VEHICLE[i].ON_ARC.ID, this->VEHICLE[i].Thwarted);
                    else
                    {
                        if( this->VEHICLE[i].WAITING > 0 )
                        {
                            if( this->VEHICLE[i].SCHEDULE->current_segment != nullptr )
                                sprintf(S,"V%d (on arc %d / POS %.2f W / %d PL / %d PAX) segment %d [%d/%d] -> %s",this->VEHICLE[i].ID, this->VEHICLE[i].ON_ARC.ID, this->VEHICLE[i].POS, this->VEHICLE[i].PAX_LIST_NUM, this->VEHICLE[i].PAX_ON_BOARD, this->VEHICLE[i].SCHEDULE->current_segment->ID, this->VEHICLE[i].SCHEDULE->current_segment->pos_in_schedule+1, this->VEHICLE[i].SCHEDULE->segments, this->VEHICLE[i].SCHEDULE->current_segment->dest_desc.c_str());
                            else
                                sprintf(S,"V%d (on arc %d / POS %.2f W / %d PL / %d PAX) no segment",this->VEHICLE[i].ID, this->VEHICLE[i].ON_ARC.ID, this->VEHICLE[i].POS, this->VEHICLE[i].PAX_LIST_NUM, this->VEHICLE[i].PAX_ON_BOARD);

                        }
                        else
                        {
                            if( this->VEHICLE[i].SCHEDULE->current_segment != nullptr )
                                sprintf(S,"V%d (on arc %d / POS %.2f / %d PL / %d PAX) segment %d [%d/%d] -> %s",this->VEHICLE[i].ID, this->VEHICLE[i].ON_ARC.ID, this->VEHICLE[i].POS, this->VEHICLE[i].PAX_LIST_NUM, this->VEHICLE[i].PAX_ON_BOARD, this->VEHICLE[i].SCHEDULE->current_segment->ID, this->VEHICLE[i].SCHEDULE->current_segment->pos_in_schedule+1, this->VEHICLE[i].SCHEDULE->segments, this->VEHICLE[i].SCHEDULE->current_segment->dest_desc.c_str());
                            else
                                sprintf(S,"V%d (on arc %d / POS %.2f / %d PL / %d PAX) no segment",this->VEHICLE[i].ID, this->VEHICLE[i].ON_ARC.ID, this->VEHICLE[i].POS, this->VEHICLE[i].PAX_LIST_NUM, this->VEHICLE[i].PAX_ON_BOARD);
                        }
                    }
                }
                else
                {
                    sprintf(S,"V%d (on arc %d / POS %.2f B  / %d PL / %d PAX) duty %d/%d",this->VEHICLE[i].ID, this->VEHICLE[i].ON_ARC.ID, this->VEHICLE[i].POS, this->VEHICLE[i].PAX_LIST_NUM, this->VEHICLE[i].PAX_ON_BOARD, this->VEHICLE[i].rotation->current_duty_num, this->VEHICLE[i].rotation->duties);
                }
                unsigned char* ubuf = reinterpret_cast<unsigned char*>(S);
                glutBitmapString(GLUT_BITMAP_HELVETICA_12, ubuf);
                y_offset = y_offset + 0.03;
            }
        }
    }

void BUSSIM_NETWORK::start_rotations(void)
{
    /*
    if( (this->CURRENT_TIME >= 0) && (this->CURRENT_TIME < 0.5) )
    {
        this->VEHICLE[0].activate(this);
    }
    */

}

void BUSSIM_NETWORK::start_schedules(void)
{
    // this procedure checks for all vehicles if its
    // activation time has been reached. If this is true then
    // vehicle is activated
    for( int v=0 ; v < this->VEHICLES ; v++ )
        if( (this->CURRENT_TIME >= this->VEHICLE[v].activation_time)
           && (this->CURRENT_TIME < (this->VEHICLE[v].activation_time + 0.5) ) )
        {
            this->VEHICLE[v].activate_new(this);
            //this->VEHICLE[v].print_paxlists_on_console(this);
            //exit(0);
        }
}

void BUSSIM_NETWORK::append_bunch_point(class BUSSIM_BUNCHPOINT *_POINT)
{
    int check = 1;

    if( this->BUNCHPOINTS > 0 )
    {
        // first check if this point is already saved
        class BUSSIM_BUNCHPOINT *TmpPtr;
        TmpPtr = this->first_bp;
        while( TmpPtr != nullptr )
        {
            if( ( fabs(TmpPtr->screen_x - _POINT->screen_x) < 0.0001)
               && ( fabs(TmpPtr->screen_y - _POINT->screen_y) < 0.0001) )
            {
                // recent point is already saved
                TmpPtr->num_of_appear++;
                check = 0;
                TmpPtr = nullptr;
                this->BUNCHPOINTS++;
            }
            else
            {
                // not yet found, go to next
                TmpPtr = TmpPtr->next;
            }
        }
        //std::cout << "hinzufuegen" << std::endl;
        if( check == 1)
        {
            // we have to create a new bunchpoint-object
            class BUSSIM_BUNCHPOINT *AddBP;
            AddBP = new class BUSSIM_BUNCHPOINT(_POINT->screen_x,_POINT->screen_y);
            AddBP->previous = this->last_bp;
            this->last_bp->next = AddBP;
            this->last_bp = AddBP;
            AddBP = nullptr;
            this->BUNCHPOINTS++;
        }
        //std::cout << "hinzufuegen beendet" << std::endl;
    }
    else
    {
        //std::cout << "1. hinzufuegen" << std::endl;
        class BUSSIM_BUNCHPOINT *AddBP;
        AddBP = new class BUSSIM_BUNCHPOINT(_POINT->screen_x,_POINT->screen_y);
        this->first_bp = AddBP;
        this->last_bp = AddBP;
        AddBP = nullptr;
        this->BUNCHPOINTS++;
        //std::cout << "1. hinzufuegen beendet" << std::endl;
    }


}

void BUSSIM_NETWORK::print_bunch_points(void)
{
    float Color;

    class BUSSIM_BUNCHPOINT *TmpPtr;

    TmpPtr = this->first_bp;

    while( TmpPtr != nullptr)
    {
        double r = 0.01+0.00; //*this->BP[i].label_pos;

        int num_segments = 1000;

        glBegin(GL_LINE_LOOP);
        Color = 0.3f + (float)((TmpPtr->num_of_appear)/((double)(this->BUNCHPOINTS))*0.7);
        glColor4f(Color , Color, Color, 0.0f);
        for(int ii = 0; ii < num_segments; ii++)
        {
            float theta = 2.0f * 3.1415926f * float(ii) / float(num_segments);//get the current angle

            float x = r * cosf(theta);//calculate the x component
            float y = r * sinf(theta);//calculate the y component

            glVertex2f(TmpPtr->screen_x, TmpPtr->screen_y);
            glVertex2f(x + TmpPtr->screen_x, y + TmpPtr->screen_y);//output vertex

        }
        glEnd();
        TmpPtr = TmpPtr->next;
    }

}

void BUSSIM_NETWORK::append_event(class BUSSIM_EVENT *_EVENT)
{
    // this procedure appends an event object to the list of previously added event objects
    if( this->first_event == nullptr )
    {
        // event list is empty
        this->first_event = _EVENT;
        _EVENT->previous = nullptr;
        this->last_event = _EVENT;
        _EVENT->next = nullptr;
        this->EVENTS++;
    }
    else
    {
        // event list already contains some events objects
        _EVENT->previous = this->last_event;
        _EVENT->previous->next = _EVENT;
        this->last_event = _EVENT;
        this->EVENTS++;
    }
}

void BUSSIM_NETWORK::print_event_list(void)
{
    class BUSSIM_EVENT *TmpPtr;

    TmpPtr = this->first_event;

    while( TmpPtr != nullptr )
    {
        TmpPtr->PRINT_ON_SCREEN();
        TmpPtr = TmpPtr->next;
    }
}

int BUSSIM_NETWORK::number_of_active_vehicles(void)
{
    int ret_value = 0;
    for(int v=0 ; v < this->VEHICLES ; v++)
    {
        ret_value += this->VEHICLE[v].ACTIVE;
    }

    return ret_value;
}

void BUSSIM_NETWORK::center_network(void)
{
    // this procedure corrects the point positions of the screen to center the network presentation

    double x_min_nodes = 1000.0;
    double x_max_nodes = -1000.0;
    double x_middle_nodes;
    double y_min_nodes = 1000.0;
    double y_max_nodes = -1000.0;
    double y_middle_nodes;
    // the stops
    for(int n=0 ; n < this->POINTS ; n++)
    {
        if( this->PT[n].screen_x < x_min_nodes )
            x_min_nodes = this->PT[n].screen_x;
        if( this->PT[n].screen_x > x_max_nodes )
            x_max_nodes = this->PT[n].screen_x;
        if( this->PT[n].screen_y < y_min_nodes )
            y_min_nodes = this->PT[n].screen_y;
        if( this->PT[n].screen_y > y_max_nodes )
            y_max_nodes = this->PT[n].screen_y;
    }
    // the tois
    struct BUSSIM_TOI_POINT *Ptr;
    for(int n=0 ; n < this->TOIS ; n++)
    {
        Ptr = this->TOI[n].PT;
        while( Ptr != nullptr )
        {
            if( Ptr->screen_x < x_min_nodes )
                x_min_nodes = Ptr->screen_x;
            if( Ptr->screen_x > x_max_nodes )
                x_max_nodes = Ptr->screen_x;
            if( Ptr->screen_y < y_min_nodes )
                y_min_nodes = Ptr->screen_y;
            if( Ptr->screen_y > y_max_nodes )
                y_max_nodes = Ptr->screen_y;
            Ptr = Ptr->next;
        }
    }

    x_middle_nodes = x_min_nodes + (x_max_nodes - x_min_nodes) / 2.0;
    y_middle_nodes = y_min_nodes + (y_max_nodes - y_min_nodes) / 2.0;

    if( x_middle_nodes < 0 )
    {
        // move everything a little bit up
        for(int n=0 ; n < this->POINTS ; n++)
        {
            this->PT[n].screen_x -= x_middle_nodes;
        }
        for(int n=0 ; n < this->TOIS ; n++)
        {
            Ptr = this->TOI[n].PT;
            while( Ptr != nullptr )
            {
                Ptr->screen_x -= x_middle_nodes;
                Ptr = Ptr->next;
            }
        }
    }
    if( x_middle_nodes > 0 )
    {
        // move everything a little bit down
        for(int n=0 ; n < this->POINTS ; n++)
        {
            this->PT[n].screen_x += x_middle_nodes;
        }
        for(int n=0 ; n < this->TOIS ; n++)
        {
            Ptr = this->TOI[n].PT;
            while( Ptr != nullptr )
            {
                Ptr->screen_x += x_middle_nodes;
                Ptr = Ptr->next;
            }
        }

    }
    if( y_middle_nodes < 0 )
    {
        // move everything a little bit up
        for(int n=0 ; n < this->POINTS ; n++)
        {
            this->PT[n].screen_y -= y_middle_nodes;
        }
        for(int n=0 ; n < this->TOIS ; n++)
        {
            Ptr = this->TOI[n].PT;
            while( Ptr != nullptr )
            {
                Ptr->screen_y -= y_middle_nodes;
                Ptr = Ptr->next;
            }
        }

    }
    if( y_middle_nodes > 0 )
    {
        // move everything a little bit down
        for(int n=0 ; n < this->POINTS ; n++)
        {
            this->PT[n].screen_y += y_middle_nodes;
        }
        for(int n=0 ; n < this->TOIS ; n++)
        {
            Ptr = this->TOI[n].PT;
            while( Ptr != nullptr )
            {
                Ptr->screen_y += y_middle_nodes;
                Ptr = Ptr->next;
            }
        }
    }
}

void BUSSIM_NETWORK::merge_lines_arcs(void)
{
    // go through all lines
    for( int l=0 ; l < this->LINES ; l++ )
    {
        // now run through the arcs of the current line
        for( int a=0 ; a < this->LINE[l].ARCS ; a++ )
        {
            class BUSSIM_ARC_IN_LINE *Tmp;
            Tmp = new class BUSSIM_ARC_IN_LINE;
            Tmp->LINE_ID = l;
            Tmp->next = nullptr;
            if( this->PT[this->ARC[this->LINE[l].USED_ARC[a].ID].orig_node].screen_x
               < this->PT[this->ARC[this->LINE[l].USED_ARC[a].ID].dest_node].screen_x )
            {
                // left to right
                if( this->PT[this->ARC[this->LINE[l].USED_ARC[a].ID].orig_node].screen_y
                        < this->PT[this->ARC[this->LINE[l].USED_ARC[a].ID].dest_node].screen_y )
                        {
                            // to the north east
                            Tmp->x_offset = 0.05;
                            Tmp->y_offset = -0.05;
                        }
                        else
                            if( this->PT[this->ARC[this->LINE[l].USED_ARC[a].ID].orig_node].screen_y
                                > this->PT[this->ARC[this->LINE[l].USED_ARC[a].ID].dest_node].screen_y )
                                {
                                    // to the south east
                                    Tmp->x_offset = -0.05;
                                    Tmp->y_offset = -0.05;
                                }
                                else
                                {
                                    // to the east
                                    Tmp->x_offset = 0.0;
                                    Tmp->y_offset = -0.05;
                                }

            }
            else
                if( this->PT[this->ARC[this->LINE[l].USED_ARC[a].ID].orig_node].screen_x
               > this->PT[this->ARC[this->LINE[l].USED_ARC[a].ID].dest_node].screen_x )
                {
                    // right to left
                     if( this->PT[this->ARC[this->LINE[l].USED_ARC[a].ID].orig_node].screen_y
                        < this->PT[this->ARC[this->LINE[l].USED_ARC[a].ID].dest_node].screen_y )
                        {
                            // to the northwest
                            Tmp->x_offset = 0.05;
                            Tmp->y_offset = 0.05;
                        }
                        else
                            if( this->PT[this->ARC[this->LINE[l].USED_ARC[a].ID].orig_node].screen_y
                            > this->PT[this->ARC[this->LINE[l].USED_ARC[a].ID].dest_node].screen_y )
                            {
                                // to the northeast
                                Tmp->x_offset = -0.05;
                                Tmp->y_offset = +0.05;
                            }
                            else
                            {
                                // to the west
                                Tmp->x_offset = 0.0;
                                Tmp->y_offset = +0.05;
                            }
                }
                else
                {
                    // vertical
                    if( this->PT[this->ARC[this->LINE[l].USED_ARC[a].ID].orig_node].screen_y
                        < this->PT[this->ARC[this->LINE[l].USED_ARC[a].ID].dest_node].screen_y )
                    {
                        // to the north
                        Tmp->x_offset = 0.05;
                        Tmp->y_offset = 0;
                    }
                    else
                    {
                        // to the south
                        Tmp->x_offset = 0;
                        Tmp->y_offset = -0.05;
                    }
                }

            // consider only regularlines
            if( this->LINE[l].type == BUSSIM_REGULARLINE )
                this->ARC[this->LINE[l].USED_ARC[a].ID].append_line(Tmp,this);
            Tmp = nullptr;
        }
    }

    //exit(0);
}

void BUSSIM_NETWORK::set_visibility(void)
{
    // here the visibility of the available vehicles can be specified

    // make all vehicles invisible

    for( int v=0 ; v < this->VEHICLES ; v++ )
    {
        this->VEHICLE[v].VISIBLE = 1;
    }


    // make selected vehicle visisble again
   for( int l=0 ; l < this->LINES ; l++ )
    {
        this->LINE[l].displayed = 1;
    }

}

void BUSSIM_NETWORK::draw_depots(class BUSSIM_NETWORK *_NET)
{
    double r;
    r = 1.0 / (double)(pow(1.05,_NET->Current_Zoom_Level));
    for( int p=0 ; p < this->POINTS ; p++ )
    {
        if( this->PT[p].type == BUSSIM_NODE_SERVICE )
        {
            glBegin(GL_POLYGON);
            glVertex2f(this->PT[p].screen_x-r*0.01, this->PT[p].screen_y+r*0.01);
            glVertex2f(this->PT[p].screen_x+r*0.01, this->PT[p].screen_y+r*0.01);
            glVertex2f(this->PT[p].screen_x+r*0.01, this->PT[p].screen_y-r*0.01);
            glVertex2f(this->PT[p].screen_x-r*0.01, this->PT[p].screen_y-r*0.01);
            glEnd();
        }
    }
}

void BUSSIM_NETWORK::draw_service_arcs(void)
{
    for( int a=0 ; a < this->ARCS ; a++ )
    {
        if( this->ARC[a].lines_on_arc < 1 )
        {
            glBegin(GL_LINES);
            glLineWidth((GLfloat)30);
            glVertex2f(this->ARC[a].orig_screen_x, this->ARC[a].orig_screen_y);
            glVertex2f(this->ARC[a].dest_screen_x, this->ARC[a].dest_screen_y);
            glEnd();
        }
    }
}

void BUSSIM_NETWORK::append_duty_to_vehicle_rotation(int _VEH_ID, int _TRIP_ID)
{
    // this procedure creates a new duty containing line _TRIP_ID and appends this
    // duty to the rotation of _VEH_ID

    // create the pointer for a new duty
    BUSSIM_DUTY *NEW_DUTY;

    // allocate the memory for the new duty
    NEW_DUTY = new struct BUSSIM_DUTY();

    // assign the unique id to the duty
    NEW_DUTY->ID = this->DUTIES;

    // increase the network-wide duty counter
    this->DUTIES++;

    // assign the trip id
    NEW_DUTY->TRIP_ID = _TRIP_ID;

    // append the newly created duty to the specified vehicle's rotation
    this->VEHICLE[_VEH_ID].rotation->append_duty(NEW_DUTY);

    // disconnect the pointer from the created duty
    NEW_DUTY = nullptr;
}

void BUSSIM_NETWORK::specify_rotations(void)
{
    this->append_duty_to_vehicle_rotation(0,0);
    this->append_duty_to_vehicle_rotation(0,1);
    this->append_duty_to_vehicle_rotation(1,1);
    this->append_duty_to_vehicle_rotation(1,0);
}

void BUSSIM_NETWORK::specify_rotations_deployment_lines(void)
{
    // this procedure sets the initial line in a vehicle rotation
    // to its associated deployment line
    for( int i=0 ; i < this->VEHICLES ; i++ )
    {
        this->append_duty_to_vehicle_rotation(i,this->EXT_LINES+i);
    }
}

void BUSSIM_NETWORK::specify_rotations_parking_lines(void)
{
    // this procedure sets the final line in a vehicle rotation
    // to its associated parking line
    for( int i=0 ; i < this->VEHICLES ; i++ )
    {
        this->append_duty_to_vehicle_rotation(i,this->EXT_LINES+this->VEHICLES + i);
    }
}

void BUSSIM_NETWORK::specify_tois(void)
{
    // this procedure specifies the tois shown in the simulation

    // this is the Elbe river crossing the city of Dresden
    // this is the Elbe river crossing the city of Dresden
    // strcpy(this->TOI[0].title,"River Elbe");
    // this->TOI[0].type = BUSSIM_TOI_WATER;


}

void BUSSIM_NETWORK::draw_tois(void)
{
    for( int i=0 ; i < this->TOIS ; i++ )
    {
            this->TOI[i].draw();
    }
}

void BUSSIM_NETWORK::specify_blocks(void)
{

    this->BLOCK[0].specify(0,1,"Block Grlitzer Strae",1.0,0.0,0.0);
    this->BLOCK[0].add_arc(&(this->ARC[125]));
    this->BLOCK[0].add_arc(&(this->ARC[126]));
    this->BLOCK[0].add_arc(&(this->ARC[127]));
    this->BLOCK[0].add_arc(&(this->ARC[128]));
    this->BLOCK[0].add_arc(&(this->ARC[428]));
    this->BLOCK[0].add_arc(&(this->ARC[429]));
    this->BLOCK[0].add_arc(&(this->ARC[430]));
    this->BLOCK[0].add_arc(&(this->ARC[431]));

}

void BUSSIM_NETWORK::specify_schedules(void)
{
    // this procedure is used to generate the detailed schedules for the vehicles

    for( int v=0 ; v < this->VEHICLES ; v++ )
    {
        this->VEHICLE[v].rotation2schedule(this);
        //this->VEHICLE[v].SCHEDULE->print_on_shell(this);
        //exit(0);
    }
}

void BUSSIM_NETWORK::save_protocol(void)
{
    // this procedure prints the collected protocol information into
    // the file protocol_HH_MM_SS

    for( int i=0 ; i < this->POINTS ; i++ )
    {
        if( this->PT[i].WAITINGLIST != nullptr )
        {
            for( int l=0 ; l < this->EXT_LINES + 1 ; l++ )
            {
                if( this->PT[i].WAITINGLIST[l]->get_pax_num() > 0 )
                {
                    this->PT[i].WAITINGLIST[l]->print();
                }
            }
        }
    }

    /*
    for( int i=0 ; i < this->VEHICLES ; i++ )
    {
        std::cout << "PAXLISTS of vehicle " << this->VEHICLE[i].ID << ": " << this->VEHICLE[i].PAX_ON_BOARD
                        << std::endl;
        if( this->VEHICLE[i].DeBoardPaxList != nullptr )
        {
            for( int s=0 ; s < this->VEHICLE[i].PAX_LIST_NUM ; s++ )
            {
                this->VEHICLE[i].print_paxlists_on_console(this);
            }
        }
    }

    this->PAX_ARRIVED->print();
    this->PAX_WALKING->print();
    */
    // get the current time
    time_t rawtime;
    struct tm * timeinfo;
    char buffer [80];

    time (&rawtime);
    timeinfo = localtime (&rawtime);

    strftime(buffer,80,"%d%m%Y_%H%M%S",timeinfo);
    std::fstream PROTOCOL;
    char ABC[256];
    sprintf(ABC,"protocol_%s.txt",buffer);
    PROTOCOL.open(ABC,std::fstream::out);

    if( PROTOCOL.is_open() )
    {
        class BUSSIM_EVENT *TmpPtr;

        TmpPtr = this->first_event;

        while( TmpPtr != nullptr )
        {
            TmpPtr->PRINT_ON_SCREEN(&PROTOCOL);
            TmpPtr = TmpPtr->next;
        }
        std::cout << "simulation terminated - protocol file " << ABC << " created." << std::endl;
    }
    else
    {
        std::cout << "simulation terminated - unable to create protocol file " << ABC << std::endl;
    }

    PROTOCOL.close();
}

void BUSSIM_NETWORK::specify_stops(void)
{
    // this procedure is used to specify the stops

	this->STOP[0].specify(0,"Prohlis Gleisschleife ","PRO");
	this->STOP[1].specify(1,"Georg-Palitzsch-Strae","GPA");
	this->STOP[2].specify(2,"Jacob-Winter-Platz","JWP");
	this->STOP[3].specify(3,"Albert-Wolf-Platz","AWP");
	this->STOP[4].specify(4,"Trattendorfer Strae","TRS");
	this->STOP[5].specify(5,"Altreick","ARE");
	this->STOP[6].specify(6,"Hlestrae","HUL");
	this->STOP[7].specify(7,"Lohrmannstrae","LOR");
	this->STOP[8].specify(8,"Wieckestrae","WIE");
	this->STOP[9].specify(9,"Otto-Dix-Ring","ODX");
	this->STOP[10].specify(10,"Eugen-Bracht-Strae","EBR");
	this->STOP[11].specify(11,"Ccilienstrae","CAC");
	this->STOP[12].specify(12,"Hugo-Brkner-Strae","HBU");
	this->STOP[13].specify(13,"Mockritzer Strae","MOK");
	this->STOP[14].specify(14,"Wasaplatz","WAS");
	this->STOP[15].specify(15,"S-Bf. Strehlen","SSR");
	this->STOP[16].specify(16,"Querallee","QUE");
	this->STOP[17].specify(17,"Zoo","ZOO");
	this->STOP[18].specify(18,"Lennplatz","LEN");
	this->STOP[19].specify(19,"Georg-Arnhold-Bad","GEA");
	this->STOP[20].specify(20,"Straburger Platz","SBP");
	this->STOP[21].specify(21,"St.-Benno-Gymnasium","GYM");
	this->STOP[22].specify(22,"Drerstrae","DUR");
	this->STOP[23].specify(23,"Sachsenallee","SAA");
	this->STOP[24].specify(24,"Rosa-Luxemburg-Platz","RLP");
	this->STOP[25].specify(25,"Bautzner/Rothenburger Str.","BTZ");
	this->STOP[26].specify(26,"Grlitzer Strae","GLS");
	this->STOP[27].specify(27,"Alaunplatz","ALA");
	this->STOP[28].specify(28,"Bischofsweg","BIW");
	this->STOP[29].specify(29,"S-Bf. Bischofsplatz","SBI");
	this->STOP[30].specify(30,"Friedensstrae","FRI");
	this->STOP[31].specify(31,"Liststrae","LIS");
	this->STOP[32].specify(32,"Brgerstrae","BGR");
	this->STOP[33].specify(33,"Rathaus Pieschen","RPN");
	this->STOP[34].specify(34,"Altpieschen","API");
	this->STOP[35].specify(35,"Mickten","MIC");
	this->STOP[36].specify(36,"Trachauer Strae","TAS");
	this->STOP[37].specify(37,"Brockwitzer Strae","BRW");
	this->STOP[38].specify(38,"An der Flutrinne","ADF");
	this->STOP[39].specify(39,"Srnewitzer Strae","SRN");
	this->STOP[40].specify(40,"ElbePark","EPK");
	this->STOP[41].specify(41,"Washingtonstrae","WSH");
	this->STOP[42].specify(42,"Kaditz Riegelplatz","RIE");
	this->STOP[43].specify(43,"Betriebshof Reick ","BH0 ");

}

void BUSSIM_NETWORK::print_stop_list(void)
{
    for( int s=0 ; s < this->STOPS ; s++ )
    {
        std::cout << this->STOP[s].ID << "\t" << this->STOP[s].STOP_NAME << " assigned points";
        std::cout << "RED=" << this->STOP[s].RED;
        std::cout << "|GREEN=" << this->STOP[s].GREEN;
        std::cout << "|BLUE=" << this->STOP[s].BLUE << std::endl;
        struct BUSSIM_ASSIGNED_POINT *TMP;
        TMP = this->STOP[s].first_point;

        while( TMP != nullptr )
        {
            std::cout << "\t" << this->PT[TMP->PT_ID].ID << "\t" << this->PT[TMP->PT_ID].STOPNAME << "(";
            std::cout << this->PT[TMP->PT_ID].SHORTNAME << ") ";
            std::cout << " long=" << this->PT[TMP->PT_ID].world_longitude << "|";
            std::cout << " latt=" << this->PT[TMP->PT_ID].world_latitude << std::endl;
            TMP = TMP->next;
        }

        std::cout << this->STOP[s].ID << "\t" << this->STOP[s].STOP_NAME << " assigned arcs";
        struct BUSSIM_ASSIGNED_ARC *TMP_A;
        TMP_A = this->STOP[s].first_arc;

        while( TMP_A != nullptr )
        {
            std::cout << "\t" << this->ARC[TMP_A->ARC_ID].ID << "\t" << this->ARC[TMP_A->ARC_ID].orig_node << "->";
            std::cout << this->ARC[TMP_A->ARC_ID].dest_node << ") " << std::endl;
            TMP_A = TMP_A->next;
        }

    }
}

void BUSSIM_NETWORK::set_stop_colors(void)
{
    if( this->STOPS > 0)
    {
        for( int s=0 ; s < this->STOPS ; s++ )
        {
            this->STOP[s].RED = 0.1 + 0.85* (double)(rand() % 1001) / 1000.0;
            this->STOP[s].GREEN = 0.1 + 0.85* (double)(rand() % 1001) / 1000.0;
            this->STOP[s].BLUE = 0.1 + 0.85* (double)(rand() % 1001) / 1000.0;
        }
    }

}

void BUSSIM_NETWORK::check_node_consistency(void)
{
    if( this->POINTS > 0)
    {
        if( this->PT[0].ID == 0 )
        {
            //std::cout << "check node numbering consistency: ";
            //std::cout << "PT["<<0<<"]:ok,";
            for( int n=0 ; n < this->POINTS-1 ; n++ )
            {
                if( this->PT[n+1].ID == this->PT[n].ID+1 )
                {
                    //std::cout << "PT["<<n+1<<"]:ok,";
                }
                else
                {
                    std::cout << "check node numbering consistency: PT["<<n+1<<"]: not ok";
                    exit(0);
                }
            }
            std::cout << std::endl;
        }
        else
        {
            std::cout << "first node is not numbered by 0!" << std::endl;
            exit(0);
        }
    }
}

void BUSSIM_NETWORK::check_arc_consistency(void)
{
    // check 1: are all arcs numbered by 0,1,...,ARCS-1?
    // check 2: are all referred node numbers in the set {0...POINTS-1}?
    // check 3: are all pairs of nodes pairwise different?

    std::cout << "check arc definition consistency: ";

    if( this->ARC[0].ID == 0 )
    {
        // std::cout << "ID of ARC["<<0<<"]:ok,";
        for( int a=0 ; a < this->ARCS-1 ; a++ )
        {
            if( this->ARC[a+1].ID == this->ARC[a].ID+1 )
            {
                //std::cout << "ID of ARC["<<a+1<<"]:ok,";
            }
            else
            {
                std::cout << "ID of ARC["<<a+1<<"]: not ok";
                exit(0);
            }
        }
        std::cout << std::endl;
    }
    else
    {
        std::cout << "first arc is not numbered by 0!" << std::endl;
        exit(0);
    }

    for( int a=0 ; a < this->ARCS ; a++ )
    {
        if( (this->ARC[a].orig_node >= 0) && (this->ARC[a].orig_node < this->POINTS)
           &&  (this->ARC[a].dest_node >= 0) && (this->ARC[a].dest_node < this->POINTS) && (this->ARC[a].dest_node != this->ARC[a].orig_node) )
            {
                //std::cout << "node numbers of ARC["<<a<<"]:ok,";
            }
            else
            {
                std::cout << "node numbers of ARC["<<a<<"]: not ok";
                exit(0);
            }
    }
}

void BUSSIM_NETWORK::append_vp_move(int _TYPE)
{
    if( this->FIRST_VP_LIST != nullptr )
    {
        // already saved vp moves available
        struct BUSSIM_VP_MODIFICATION *Tmp;
        Tmp = new struct BUSSIM_VP_MODIFICATION;
        Tmp->type = _TYPE;
        Tmp->next = nullptr;
        this->LAST_VP_LIST->next = Tmp;
        this->LAST_VP_LIST = Tmp;
        Tmp = nullptr;
    }
    else
    {
        // no items save so far
        struct BUSSIM_VP_MODIFICATION *Tmp;
        Tmp = new struct BUSSIM_VP_MODIFICATION;
        Tmp->type = _TYPE;
        Tmp->next = nullptr;
        this->FIRST_VP_LIST = Tmp;
        this->LAST_VP_LIST = Tmp;
        Tmp = nullptr;
    }
    Stored_VP_Move_Items++;
}

void BUSSIM_NETWORK::clear_vp_move_list(void)
{
    if( this->FIRST_VP_LIST != nullptr )
    {
        // there are save vp_move_items
        struct BUSSIM_VP_MODIFICATION *ToBeDeleted;

        ToBeDeleted = this->FIRST_VP_LIST;

        while( ToBeDeleted != nullptr )
        {
            this->FIRST_VP_LIST = this->FIRST_VP_LIST->next;
            delete ToBeDeleted;
            ToBeDeleted = this->FIRST_VP_LIST;
        }

        this->LAST_VP_LIST = nullptr;
        Stored_VP_Move_Items = 0;
    }
}

void BUSSIM_NETWORK::save_vp_move_list(void)
{
    std::ofstream BVFILE;
    BVFILE.open("./bussim_view");
    struct BUSSIM_VP_MODIFICATION *Tmp;
    Tmp = this->FIRST_VP_LIST;
    while( Tmp != nullptr )
    {
        BVFILE << Tmp->type << "\n";
        Tmp = Tmp->next;
    }
    BVFILE.close();
}

void BUSSIM_NETWORK::load_vp_move_list(void)
{
    std::ifstream BVFILE;

    BVFILE.open("./bussim_view");

    if( BVFILE.is_open() )
    {
        std::string fullline;
        int VALUE;
        std::cout << "restore view from previous session..." << std::endl;
        while( std::getline(BVFILE, fullline) )
        {
            VALUE = stoi(fullline);

            if( VALUE == BUSSIM_VIEWPORT_LEFT )
            {
                glMatrixMode(GL_MODELVIEW);
                glTranslatef(-0.01,0,0);
                this->Current_MENUE_OFFSET_X--;
                glPushMatrix();
                glDrawBuffer(GL_MODELVIEW);
                glPopMatrix();
                this->append_vp_move(BUSSIM_VIEWPORT_LEFT);
            }
            if( VALUE == BUSSIM_VIEWPORT_RIGHT )
            {
                glMatrixMode(GL_MODELVIEW);
                glTranslatef(0.01,0,0);
                this->Current_MENUE_OFFSET_X++;
                glPushMatrix();
                glDrawBuffer(GL_MODELVIEW);
                glPopMatrix();
                this->append_vp_move(BUSSIM_VIEWPORT_RIGHT);
            }
            if( VALUE == BUSSIM_VIEWPORT_DOWN )
            {
                glMatrixMode(GL_MODELVIEW);
                glTranslatef(0,-0.01,0);
                this->Current_MENUE_OFFSET_Y--;
                glPushMatrix();
                glDrawBuffer(GL_MODELVIEW);
                glPopMatrix();
                this->append_vp_move(BUSSIM_VIEWPORT_DOWN);

            }
            if( VALUE == BUSSIM_VIEWPORT_UP )
            {
                glMatrixMode(GL_MODELVIEW);
                glTranslatef(0,0.01,0);
                this->Current_MENUE_OFFSET_Y++;
                glPushMatrix();
                glDrawBuffer(GL_MODELVIEW);
                glPopMatrix();
                this->append_vp_move(BUSSIM_VIEWPORT_UP);

            }
            if( VALUE == BUSSIM_VIEWPORT_IN )
            {
                glMatrixMode(GL_MODELVIEW);
                glScalef(1.05,1.05,1);
                glPushMatrix();
                glDrawBuffer(GL_MODELVIEW);
                glPopMatrix();
                this->Current_Zoom_Level++;
                this->append_vp_move(BUSSIM_VIEWPORT_IN);
            }
            if( VALUE == BUSSIM_VIEWPORT_OUT )
            {
                glMatrixMode(GL_MODELVIEW);
                glScalef(1.0 / 1.05,1.0 / 1.05,1);
                glPushMatrix();
                glDrawBuffer(GL_MODELVIEW);
                glPopMatrix();
                this->Current_Zoom_Level--;
                this->append_vp_move(BUSSIM_VIEWPORT_OUT);
            }

            std::cout << VALUE << " ";

        }
        BVFILE.close();
        std::cout << std::endl;
        std::cout << "done!" << std::endl;
    }
}

void BUSSIM_NETWORK::check_stop_consistency(void)
{
    // this procedure checks if the stops are consecutively numbered
    for( int s=0 ; s < this->STOPS ; s++ )
    {
        if( this->STOP[s].ID == s )
        {
            //std::cout << "STOP["<<s<<"] is ok!" << std::endl;
        }
        else
        {
            std::cout << "STOP["<<s<<"]: ID is wrong!" << std::endl;
            exit(0);
        }
    }
}

void BUSSIM_NETWORK::check_stop_and_node_consistency(void)
{

}

void BUSSIM_NETWORK::attach_internal_arcs_to_stops(void)
{
    // this procedures tries to assign arcs to stops.
    // if both nodes associated with an arc belong to the same stop
    // then the arc is assigned to this stop as INTERNAL ARC

    for( int a = 0 ; a < this->ARCS ; a++ )
    {
        //std::cout << "arc [" << this->ARC[a].ID << "]:";
        if( (this->PT[this->ARC[a].orig_node].ref2stop != nullptr)
           && (this->PT[this->ARC[a].dest_node].ref2stop != nullptr)
           && (this->PT[this->ARC[a].orig_node].ref2stop->ID == this->PT[this->ARC[a].dest_node].ref2stop->ID) )
        {
            //std::cout << "..." << std::endl;
            this->STOP[this->PT[this->ARC[a].orig_node].ref2stop->ID].append_arc(a);
            //std::cout << "... is internal for stop " << this->PT[this->ARC[a].orig_node].ref2stop->ID << std::endl;
        }
        else
        {
            //std::cout << "... is non-internal" << std::endl;
        }
    }
}

void BUSSIM_NETWORK::inspect_initial_vehicle_positions(void)
{
    // this procedure checks for every vehicle if it is already on the first arc according to
    // its specified schedule.

    std::cout << "Check the initial positions of the vehicles with its current schedule." << std::endl;

    for(int v=0 ; v < this->VEHICLES ; v++ )
    {
        if( this->VEHICLE[v].compare_init_pos_with_schedule(this) > 0 )
        {
            // the initial position of the vehicle fits with its schedule
            // std::cout << "vehicle " << v << " is placed on its first scheduled arc! ->ok" << std::endl;
        }
        else
        {
            // the initial position of the vehicle does not fit with its schedule
            std::cout << "vehicle " << v << " is placed outside its first scheduled arc! ->try to solve" << std::endl;
            //std::cout << "\t current position: on arc " << this->VEHICLE[v].ON_ARC.ID << " (POS="<<this->VEHICLE[v].POS<<")" << std::endl;
            if( this->VEHICLE[v].SCHEDULE != nullptr )
            {
                if( this->VEHICLE[v].SCHEDULE->first_segment != nullptr )
                {
                    //std::cout << "\t first arc according to schedule "
                    //<< this->VEHICLE[v].SCHEDULE->first_segment->ref2arc->ID
                    //<< std::endl;
                    if( (this->PT[this->VEHICLE[v].SCHEDULE->first_segment->ref2arc->orig_node].ref2stop != nullptr)
                       && (this->PT[this->ARC[this->VEHICLE[v].ON_ARC.ID].orig_node].ref2stop != nullptr )  )
                    {
                        std::cout << "\t look for option to solve this problem" << std::endl;
                        if( this->PT[this->VEHICLE[v].SCHEDULE->first_segment->ref2arc->orig_node].ref2stop->ID
                       == this->PT[this->ARC[this->VEHICLE[v].ON_ARC.ID].orig_node].ref2stop->ID )
                        {
                            std::cout << "both involved arcs are part of the same stop "<<this->PT[this->ARC[this->VEHICLE[v].ON_ARC.ID].orig_node].ref2stop->ID<<" try to find connecting path" << std::endl;
                            std::cout << "looking for a path from ";
                            std::cout << "node " << this->PT[this->ARC[this->VEHICLE[v].ON_ARC.ID].orig_node].ID;
                            std::cout << " to node " << this->PT[this->VEHICLE[v].SCHEDULE->first_segment->ref2arc->orig_node].ID;
                            std::cout << std::endl;

                            // call the dijkstra procedure to determine a deployment path completion if such one exists
                            this->simple_dijkstra_at_stop(this->PT[this->ARC[this->VEHICLE[v].ON_ARC.ID].orig_node].ID,
                                     this->PT[this->VEHICLE[v].SCHEDULE->first_segment->ref2arc->orig_node].ID,&(this->LINE[this->EXT_LINES + v]));

                            // print the found additional service line
                            double delay=0.0;
                            std::cout << "deployment line found: ";
                            for( int t=0 ; t < (this->LINE[this->EXT_LINES + v].ARCS) ; t++ )
                            {
                                std::cout << this->LINE[this->EXT_LINES + v].USED_ARC[t].ID;
                                std::cout << " (" << this->LINE[this->EXT_LINES + v].USED_ARC[t].orig_node << "->" << this->LINE[this->EXT_LINES + v].USED_ARC[t].dest_node;
                                std::cout << ") ";
                                delay += this->LINE[this->EXT_LINES + v].USED_ARC[t].length;
                            }
                            std::cout << std::endl;
                            delay = round(delay / this->VEHICLE[v].velocity*60);
                            std::cout << "total right shift of vehicle " << v << "'s schedule: " << delay << " minute(s)" << std::endl;
                            //std::cout << "current schedule" << std::endl;
                            //this->VEHICLE[v].SCHEDULE->print_on_shell(this);

                            // upate the existing schedule

                            // i) clean the existing schedule for this vehicle
                            this->VEHICLE[v].SCHEDULE->clean(this);

                            // ii) re-generate the schedule using the additional deployment line
                            this->VEHICLE[v].rotation2schedule(this);

                            //this->VEHICLE[v].SCHEDULE->print_on_shell(this);

                            //exit(0);
                            // i) insert the new line at the beginning of the rotation

                            // ii) recall the conversion from rotation to schedule

                            // release the memory of the auxiliary line-object
                        }
                        else
                        {
                            std::cout << "no sequence completion possible! the two nodes belong to different stops"<< std::endl;
                            exit(0);
                        }
                    }
                    else
                    {
                        std::cout << "no chance to solve this problem because at least one point is not assigned to a stop" << std::endl;
                        exit(0);
                    }
                }
                else
                {
                    std::cout << "no segment in schedule specified! -> no line correction needed" << std::endl;
                }
            }
            else
            {
                std::cout << "no schedule specified! -> no line correction needed" << std::endl;
            }

        }
    }
}

void BUSSIM_NETWORK::inspect_final_vehicle_positions(void)
{
    // this procedure checks for every vehicle if its rotation stops on the specified parking app.

    std::cout << "Compare the parking positions of the vehicles with their current schedule." << std::endl;

    for(int v=0 ; v < this->VEHICLES ; v++ )
    {
        if( this->VEHICLE[v].compare_park_pos_with_schedule(this) > 0 )
        {
            // the park position of the vehicle fits with its schedule
            // std::cout << "vehicle " << v << " is going to park on its last scheduled arc! ->ok" << std::endl;
        }
        else
        {
            // the park position of the vehicle does not fit with its schedule
            std::cout << "vehicle " << v << " is going to park outside its last scheduled arc! ->try to solve" << std::endl;
            std::cout << "\t planned park position: on arc " << this->VEHICLE[v].park_arc.ID << " (POS="<<this->VEHICLE[v].park_pos<<")" << std::endl;
            if( this->VEHICLE[v].SCHEDULE != nullptr )
            {
                if( this->VEHICLE[v].SCHEDULE->last_segment != nullptr )
                {
                    std::cout << "\t last arc according to schedule "
                    << this->VEHICLE[v].SCHEDULE->last_segment->ref2arc->ID
                    << std::endl;
                    if( (this->PT[this->VEHICLE[v].SCHEDULE->last_segment->ref2arc->dest_node].ref2stop != nullptr)
                       && (this->PT[this->ARC[this->VEHICLE[v].park_arc.ID].dest_node].ref2stop != nullptr )  )
                    {
                        std::cout << "\t look for option to solve this problem" << std::endl;
                        if( this->PT[this->VEHICLE[v].SCHEDULE->last_segment->ref2arc->dest_node].ref2stop->ID
                       == this->PT[this->ARC[this->VEHICLE[v].park_arc.ID].dest_node].ref2stop->ID )
                        {
                            std::cout << "both involved arcs are part of the same stop "<<this->PT[this->ARC[this->VEHICLE[v].park_arc.ID].dest_node].ref2stop->ID<<" try to find connecting path" << std::endl;
                            std::cout << "looking for a path from ";
                            std::cout << "node " << this->PT[this->VEHICLE[v].SCHEDULE->last_segment->ref2arc->dest_node].ID;
                            std::cout << " to node " << this->PT[this->ARC[this->VEHICLE[v].park_arc.ID].orig_node].ID;
                            std::cout << std::endl;

                            // call the dijkstra procedure to determine a deployment path completion if such one exists
                            this->simple_dijkstra_at_stop(this->PT[this->VEHICLE[v].SCHEDULE->last_segment->ref2arc->dest_node].ID,this->PT[this->ARC[this->VEHICLE[v].park_arc.ID].orig_node].ID,&(this->LINE[this->EXT_LINES + this->VEHICLES + v]));

                            // add the park arc to the parking line
                            this->LINE[this->EXT_LINES + this->VEHICLES + v].append_arc(this->VEHICLE[v].park_arc);

                            // print the found additional service line
                            double delay=0.0;
                            std::cout << "deployment line found: ";
                            for( int t=0 ; t < (this->LINE[this->EXT_LINES + this->VEHICLES + v].ARCS) ; t++ )
                            {
                                std::cout << this->LINE[this->EXT_LINES + this->VEHICLES + v].USED_ARC[t].ID;
                                std::cout << " (" << this->LINE[this->EXT_LINES + this->VEHICLES + v].USED_ARC[t].orig_node << "->" << this->LINE[this->EXT_LINES + this->VEHICLES + v].USED_ARC[t].dest_node;
                                std::cout << ") ";
                                delay += this->LINE[this->EXT_LINES + this->VEHICLES + v].USED_ARC[t].length;
                            }
                            std::cout << std::endl;
                            delay = round(delay / this->VEHICLE[v].velocity*60);
                            std::cout << "total right shift of vehicle " << v << "'s schedule: " << delay << " minute(s)" << std::endl;
                            //std::cout << "current schedule" << std::endl;
                            //this->VEHICLE[v].SCHEDULE->print_on_shell(this);

                            // upate the existing schedule

                            // i) clean the existing schedule for this vehicle
                            this->VEHICLE[v].SCHEDULE->clean(this);

                            // ii) re-generate the schedule using the additional deployment line
                            this->VEHICLE[v].rotation2schedule(this);

                            // specify the parking position on the final arc
                            this->VEHICLE[v].SCHEDULE->last_segment->park_pos_on_arc = this->VEHICLE[v].park_pos;
                            //std::cout << "final park position " << this->VEHICLE[v].park_pos << std::endl;

                            //this->VEHICLE[v].SCHEDULE->print_on_shell(this);

                            //exit(0);
                            // i) insert the new line at the beginning of the rotation

                            // ii) recall the conversion from rotation to schedule

                            // release the memory of the auxiliary line-object
                        }
                        else
                        {
                            std::cout << "no sequence completion possible! the two nodes belong to different stops"<< std::endl;
                            exit(0);
                        }
                    }
                    else
                    {
                        std::cout << "no chance to solve this problem because at least one point is not assigned to a stop" << std::endl;
                        exit(0);
                    }
                }
                else
                {
                    std::cout << "no segment in schedule specified! -> no line correction needed" << std::endl;
                }
            }
            else
            {
                std::cout << "no schedule specified! -> no line correction needed" << std::endl;
            }

        }
    }
}

void BUSSIM_NETWORK::simple_dijkstra_at_stop(int _START_NODE, int _TARGET_NODE, class BUSSIM_LINE *_PTR_LINE)
{
    // this procedure tries to find a shortest path from _START_NODE to _TARGET_NODE
    // it is necessary that _START_NODE and _TARGET_NODE belong to the same BUSSIM_STOP-instance
    // if such a shortest path exists it is stored in the referred _PTR_LINE pointer
    // only arcs with ROUTING_TYPE = BUSSIM_RT_VEHICLE are considered

    if( this->PT[_START_NODE].ref2stop != nullptr )
    {
        if( this->PT[_TARGET_NODE].ref2stop != nullptr )
        {
            //std::cout << "GAGA" << std::endl;
            //std::cout << "START_NODE=" << _START_NODE << std::endl;
            //std::cout << "TARGET_NODE=" << _TARGET_NODE << std::endl;
            // both nodes are attached to the same stop
            if( this->PT[_START_NODE].ref2stop->ID == this->PT[_TARGET_NODE].ref2stop->ID )
            {
                // both nodes are attached to the same stop
                int CurrentStop;
                CurrentStop = this->PT[_START_NODE].ref2stop->ID;

                int NODENUM = this->STOP[CurrentStop].ASSIGNED_POINTS;

                if( NODENUM > 0 )
                {
                    std::cout << "prepare to start Dijkstra at Stop " << CurrentStop << std::endl;
                    //std::cout << "the network has stops " << this->STOP[CurrentStop].ASSIGNED_POINTS << std::endl;
                    //std::cout << "the network has arcs " << this->STOP[CurrentStop].ASSIGNED_ARCS << std::endl;

                    // define the point structure
                    struct SPP_POINT {
                        int EXT_ID;
                        int ID;
                        double LABELVALUE;
                        int PRED;
                        int permanent;
                    };

                    // declare the array of nodes of this subnetwork
                    struct SPP_POINT *NODE;
                    NODE = new struct SPP_POINT[NODENUM];

                    // create an auxiliary array to identify the associated ID <-> extern ID values
                    int *LOCAL_ID;
                    LOCAL_ID = new int [this->POINTS];
                    for( int i=0 ; i < this->POINTS ; i++ )
                        LOCAL_ID[i] = -1;

                    // set the initial values for the dijkstra algorithm
                    BUSSIM_ASSIGNED_POINT *PTR;
                    PTR = this->STOP[CurrentStop].first_point;
                    int CNTR = 0;
                    while ( PTR != nullptr )
                    {
                        NODE[CNTR].EXT_ID = PTR->PT_ID;
                        NODE[CNTR].ID = CNTR;
                        NODE[CNTR].LABELVALUE = BUSSIM_BIG_M;
                        NODE[CNTR].PRED = -1;
                        NODE[CNTR].permanent = 0;
                        LOCAL_ID[NODE[CNTR].EXT_ID] = NODE[CNTR].ID;
                        //std::cout << "next node "<< CNTR <<" in subnetwork " << PTR->PT_ID << std::endl;
                        CNTR++;
                        PTR = PTR->next;
                    }

                    // allocate memory for the local distance matrix
                    double **DISTMAT;
                    DISTMAT = new double* [NODENUM];
                    for( int i=0 ; i < NODENUM ; i++ )
                    {
                        DISTMAT[i] = new double [NODENUM];
                        for( int j=0 ; j < NODENUM ; j++ )
                            DISTMAT[i][j] = BUSSIM_BIG_M;
                    }

                    // fill the distance matrix
                    int START_NODE=-1, STOP_NODE=-1;

                    struct BUSSIM_ASSIGNED_ARC *ARCPTR;
                    ARCPTR = this->STOP[CurrentStop].first_arc;
                    while( ARCPTR != nullptr )
                    {
                        START_NODE = LOCAL_ID[this->PT[this->ARC[ARCPTR->ARC_ID].orig_node].ID];
                        STOP_NODE = LOCAL_ID[this->PT[this->ARC[ARCPTR->ARC_ID].dest_node].ID];
                        if( (this->ARC[ARCPTR->ARC_ID].ROUTING_TYPE) == 0 )
                            DISTMAT[START_NODE][STOP_NODE] = this->ARC[ARCPTR->ARC_ID].length;
                        ARCPTR = ARCPTR->next;
                    }

                    int ARCCNTR=0;
                    for( int i=0 ; i < this->POINTS ; i++ )
                        for( int j=0 ; j < this->POINTS ; j++ )
                        {
                            if( (LOCAL_ID[i]>=0) && (LOCAL_ID[j] >=0) )
                            if( DISTMAT[LOCAL_ID[i]][LOCAL_ID[j]] < BUSSIM_BIG_M )
                            {
                                //std::cout << ARCCNTR << "DIST added for " << LOCAL_ID[i] << "->" << LOCAL_ID[j] << std::endl;
                                ARCCNTR++;
                            }
                        }

                    // perform dijkstra algorithm steps
                    START_NODE = LOCAL_ID[this->PT[_START_NODE].ID];
                    STOP_NODE = LOCAL_ID[this->PT[_TARGET_NODE].ID];
                    //std::cout << "STARTING DIJKSTRA FROM " << START_NODE << "-->" << STOP_NODE << std::endl;
                    //std::cout << "STARTING DIJKSTRA FROM " << this->PT[_START_NODE].ID << "-->" << this->PT[_TARGET_NODE].ID << std::endl;

                    // to store the local code of the next permanently labelled nod
                    double MinLabelValue;
                    int NextNodePerm;
                    NextNodePerm = START_NODE;
                    NODE[NextNodePerm].LABELVALUE = 0.0;
                    NODE[NextNodePerm].PRED = -1;

                    for(int CNTR=0 ; CNTR < NODENUM ; CNTR++)
                    {
                        //std::cout << "DIJKSTRA STEP " << CNTR << "/"<<NODENUM<<" (NextNodePerm=" << NextNodePerm << ")" << std::endl;
                        // mark NextNodePerm as permanent
                        NODE[NextNodePerm].permanent = 1;

                        // update temporarily labelled nodes
                        for(int CNTR_A=0 ; CNTR_A < NODENUM ; CNTR_A++)
                        {
                            if( NODE[CNTR_A].permanent < 1 )
                            {
                                // we check only temporarily labelled nodes
                                if( DISTMAT[NextNodePerm][CNTR_A] < BUSSIM_BIG_M )
                                {
                                    // we have another path extension
                                    if( (NODE[NextNodePerm].LABELVALUE + DISTMAT[NextNodePerm][CNTR_A])
                                            < NODE[CNTR_A].LABELVALUE )
                                    {
                                        // we have a path length reduction
                                        NODE[CNTR_A].LABELVALUE = NODE[NextNodePerm].LABELVALUE + DISTMAT[NextNodePerm][CNTR_A];
                                        NODE[CNTR_A].PRED = NextNodePerm;
                                    }
                                }
                            }
                        }

                        // identify the next NextNodePerm
                        NextNodePerm = -1;
                        MinLabelValue = BUSSIM_BIG_M;
                        for(int CNTR_A=0 ; CNTR_A < NODENUM ; CNTR_A++)
                        {
                            if( NODE[CNTR_A].permanent < 1 )
                            {
                                if( NODE[CNTR_A].LABELVALUE <= MinLabelValue )
                                {
                                    MinLabelValue = NODE[CNTR_A].LABELVALUE;
                                    NextNodePerm = CNTR_A;
                                }
                            }
                        }

                        //if( NextNodePerm < 0 )
                        //    CNTR = NODENUM;
                    }

                    /*
                    for(int CNTR_A=0 ; CNTR_A < NODENUM ; CNTR_A++)
                    {
                        std::cout << "NODE-ID:" << NODE[CNTR_A].ID;
                        std::cout << std::setw(6) << NODE[CNTR_A].LABELVALUE;
                        std::cout << std::setw(6) << NODE[CNTR_A].PRED;
                        std::cout << std::setw(6) << NODE[CNTR_A].permanent;
                        std::cout << std::endl;
                    }
                    */

                    //std::cout << "GAGA" << std::endl;
                    // reconstruct the determined path (if such a path exists)
                    if( NODE[STOP_NODE].LABELVALUE < BUSSIM_BIG_M )
                    {
                        // there exists a shortest path from current post to start node of specified line

                        /*
                        std::cout << "Total Duration " << NODE[STOP_NODE].LABELVALUE << std::endl;
                        std::cout << "-> final node=" << NODE[STOP_NODE].ID << std::endl;
                        int PredNode = NODE[STOP_NODE].PRED;

                        while(PredNode >= 0)
                        {
                            std::cout << "Total Duration " << NODE[PredNode].LABELVALUE << std::endl;
                            std::cout << "-> final node=" << NODE[PredNode].ID << std::endl;

                            PredNode = NODE[PredNode].PRED;
                        }
                        */

                        // store the path
                        //std::cout << "involved arcs=" << _PTR_LINE->ARCS << std::endl;
                        //exit(0);
                        //HIER WEITER
                        int X;
                        X = STOP_NODE;
                        while( (NODE[X].EXT_ID != _START_NODE) )
                        {
                            class BUSSIM_ARC TmpArc;
                            TmpArc = this->get_arc_by_od(NODE[NODE[X].PRED].EXT_ID,NODE[X].EXT_ID); //HIER globale IDs
                            if( TmpArc.ID >= 0 )
                                _PTR_LINE->append_arc(TmpArc);
                            X = NODE[X].PRED;
                            // std::cout << X << " zurueck: TMP " << TmpArc.ID << ": " << NODE[NODE[X].PRED].EXT_ID << "->" <<  NODE[X].EXT_ID<< std::endl;
                        }

                        // exit(0);
                        // print the existing schedule in the shell
                    }
                    else
                    {
                        // we cannot complete the vehicle rotation and schedule
                        std::cout << "completion of path on stop " << CurrentStop << " impossible!" << std::endl;
                        exit(0);
                    }

                    // print the determined path
                    /*
                    std::cout << "ABC " << _PTR_LINE->ARCS <<std::endl;
                    for( int a=0 ; a < _PTR_LINE->ARCS ; a++ )
                    {
                        std::cout << "-->" << _PTR_LINE->USED_ARC[a].ID << std::endl;
                    }
                    */

                    // invert stored path (if possible)
                    if( _PTR_LINE->ARCS > 0 )
                    {
                        class BUSSIM_ARC *TmpUsedArc;
                        TmpUsedArc = new class BUSSIM_ARC[_PTR_LINE->ARCS];
                        for( int a=0 ; a < _PTR_LINE->ARCS ; a++ )
                        {
                            TmpUsedArc[a] = _PTR_LINE->USED_ARC[a];
                        }

                        for( int a=0 ; a < _PTR_LINE->ARCS ; a++ )
                        {
                            _PTR_LINE->USED_ARC[a] = TmpUsedArc[(_PTR_LINE->ARCS-1) - a];
                        }
                        delete[] TmpUsedArc;
                    }

                    /*
                    std::cout << "invertiert " << _PTR_LINE->ARCS <<std::endl;
                    for( int a=0 ; a < _PTR_LINE->ARCS ; a++ )
                    {
                        std::cout << "-->" << _PTR_LINE->USED_ARC[a].ID <<": ";
                        std::cout << _PTR_LINE->USED_ARC[a].orig_node << "-->"<< _PTR_LINE->USED_ARC[a].dest_node<< std::endl;
                    }
                    exit(0);
                    */

                    // release allocated memory
                    delete[] LOCAL_ID;
                    for( int i=0 ; i < NODENUM ; i++ )
                        delete[] DISTMAT[i];
                    delete[] DISTMAT;

                    delete[] NODE;
                }
            }
        }
    }
}

class BUSSIM_ARC BUSSIM_NETWORK::get_arc_by_od(int _ORIGIN_NODE_ID, int _DESTINATION_NODE_ID)
{
    // this procedure searches the first arc in the arc list with matching orig. as well as dest node ids

    class BUSSIM_ARC TMPARC;

    for( int i=0 ; i < this->ARCS ; i++ )
    {
        if( (this->ARC[i].orig_node == _ORIGIN_NODE_ID) && (this->ARC[i].dest_node == _DESTINATION_NODE_ID) )
        {
            TMPARC = this->ARC[i];
            i = this->ARCS;
        }
    }

    return(TMPARC);
}

void BUSSIM_NETWORK::specify_paxlists(void)
{
    // this procedure fills the initial paxlists

    class BUSSIM_PAX *Pax;
    long PF_A=0, PF_B=0;
    for( long i=0 ; i < this->PAXS ; i++ )
    {
        if( i== 0 )
        {
            PF_A = this->PLATFORM_LIST->get_random_node_id();
            PF_B = this->PLATFORM_LIST->get_random_node_id();
            while ( this->PT[PF_A].ref2stop->ID == this->PT[PF_B].ref2stop->ID )
            {
                PF_B = this->PLATFORM_LIST->get_random_node_id();
            }
            std::cout << "create travel path for pax " << i << "/" << this->PAXS << "\r" << std::flush;
            Pax = new class BUSSIM_PAX(i,"Passenger " + std::to_string(i),10,PF_A,PF_B);
            Pax->get_shortest_od_path_dijkstra(this);
        }
        else
        {
            PF_A = this->PLATFORM_LIST->get_random_node_id();
            PF_B = this->PLATFORM_LIST->get_random_node_id();
            while ( this->PT[PF_A].ref2stop->ID == this->PT[PF_B].ref2stop->ID )
            {
                PF_B = this->PLATFORM_LIST->get_random_node_id();
            }
            std::cout << "create travel path for pax " << i << "/" << this->PAXS << "\r" << std::flush;
            Pax = new class BUSSIM_PAX(i,"Passenger " + std::to_string(i),this->PAX_UNKNOWN->get_last_pax()->get_show_up_time()+0.1*(rand()%2),PF_A,PF_B);
            Pax->get_shortest_od_path_dijkstra(this);
        }
        this->PAX_UNKNOWN->append_pax(Pax,0);

        /*
        if( i==37 )
        {
            Pax->TravelPath->print_on_console(this);
            //exit(0);
        }
        */

    }
    std::cout << std::endl;
}

void BUSSIM_NETWORK::update_paxlists(void)
{
    // this procedure updates the three pax lists by shifting pax from one list to another if needed

    // from PAX_UNKNOWN to PAX_IN
    //std::cout << "START" << std::endl;
    if( this->PAX_UNKNOWN->get_pax_num() > 0 )
    {
        class BUSSIM_PAX_LISTITEM *ItemPtr;
        ItemPtr = this->PAX_UNKNOWN->get_first_pax_item();
        double CurrentShowUpTime = this->PAX_UNKNOWN->get_first_show_up_time();

        //std::cout << "START-A" << std::endl;
        while( (ItemPtr != nullptr) && (CurrentShowUpTime <= this->CURRENT_TIME) )
        {
            // extract first item from PAX_UNKOWN and insert it into PAX_IN
            this->PAX_UNKNOWN->extract_item(ItemPtr);

            // indicate the first travelpath segment as currently used segment
            ItemPtr->get_ref2pax()->proceed_on_travelpath(this);

            ItemPtr->get_ref2pax()->Ref2PaxInList = ItemPtr;

            // append the new pax-list-item to the existint list of passengers in the system
            this->PAX_IN->insert_item_after(ItemPtr,this->PAX_IN->get_last_pax_item());

            // std::cout << "welcome pax " << ItemPtr->get_ref2pax()->ID << std::endl;
            // ItemPtr->get_ref2pax()->TravelPath->print_on_console(this);
            // insert this pax in the waiting list at the starting point of the pax journey
            this->PT[ItemPtr->get_ref2pax()->Origin_Node].init_add_to_waiting_list(ItemPtr->get_ref2pax(),this);

            this->NUM_PAX_IN_WAITING++;
            // std::cout << "pax " << ItemPtr->get_ref2pax()->ID << " added to initial waiting list" << std::endl;
            // draw pointer away from paxlistitem
            ItemPtr = nullptr;

            // goto next candidate
            if( this->PAX_UNKNOWN->get_pax_num() > 0 )
            {
                CurrentShowUpTime = this->PAX_UNKNOWN->get_first_show_up_time();
                if( CurrentShowUpTime > this->CURRENT_TIME )
                    ItemPtr = nullptr;
                else
                    ItemPtr = this->PAX_UNKNOWN->get_first_pax_item();

            }
        }
        //std::cout << "ENDE-A" << std::endl;
    }
    //std::cout << "ENDE" << std::endl;
    // from PAX_IN to PAX_ARRIVED
}

void BUSSIM_NETWORK::specify_transfer_arcs(void)
{
    // this procedure identifies the additional arc required to realize walking transfers

    int ADD_ARCS = 0;
    struct BUSSIM_ASSIGNED_POINT *Tmp_Orig, *Tmp_Dest;

    for( int s=0 ; s < this->STOPS ; s++ )
    {
        // std::cout << "stop " << s << std::endl;
        Tmp_Orig = this->STOP[s].first_point;
        while( Tmp_Orig != nullptr )
        {
            //std::cout << "=>POINT " << Tmp_Orig->PT_ID << " " << this->PT[Tmp_Orig->PT_ID].INFRATYPE <<std::endl;
            if( this->PT[Tmp_Orig->PT_ID].INFRASTRUCTURE == 0 && (this->PT[Tmp_Orig->PT_ID].type >= 1)  )
            //if( 1 < 2  )
            {
                Tmp_Dest = this->STOP[s].first_point;
                while( Tmp_Dest != nullptr )
                {
                    if( (this->PT[Tmp_Dest->PT_ID].INFRASTRUCTURE == 0) && (this->PT[Tmp_Dest->PT_ID].type >= 1) )
                    {
                        if( Tmp_Dest != Tmp_Orig )
                        {
                            // std::cout << "we have to add arc " << Tmp_Orig->PT_ID << "->" << Tmp_Dest->PT_ID << std::endl;
                            ADD_ARCS++;
                        }
                    }
                    Tmp_Dest = Tmp_Dest->next;
                }
            }
            Tmp_Orig = Tmp_Orig->next;
        }
    }

    // std::cout << "We have to add " << ADD_ARCS << " arcs as walking connections" << std::endl;

    if (ADD_ARCS > 0)
    {
        // create the additional array to store the additional arcs
        class BUSSIM_ARC *_ADD_ARC;
        _ADD_ARC = new class BUSSIM_ARC [ADD_ARCS];
        ADD_ARCS = 0;

        for( int s=0 ; s < this->STOPS ; s++ )
        {
            Tmp_Orig = this->STOP[s].first_point;
            while( Tmp_Orig != nullptr )
            {
                if( this->PT[Tmp_Orig->PT_ID].INFRASTRUCTURE == 0 && (this->PT[Tmp_Orig->PT_ID].type >= 1)  )
                {
                    Tmp_Dest = this->STOP[s].first_point;
                    while( Tmp_Dest != nullptr )
                    {
                        if( (this->PT[Tmp_Dest->PT_ID].INFRASTRUCTURE == 0) && (this->PT[Tmp_Dest->PT_ID].type >= 1) )
                        {
                            if( Tmp_Dest != Tmp_Orig )
                            {
                                // std::cout << "we have to add arc " << Tmp_Orig->PT_ID << "->" << Tmp_Dest->PT_ID << std::endl;
                                _ADD_ARC[ADD_ARCS].specify(this->ARCS+ADD_ARCS,Tmp_Orig->PT_ID,Tmp_Dest->PT_ID);
                                ADD_ARCS++;
                            }
                        }
                        Tmp_Dest = Tmp_Dest->next;
                    }
                }
                Tmp_Orig = Tmp_Orig->next;
            }
        }

        // save the original array of points
        class BUSSIM_ARC *AUX_ARC;
        AUX_ARC = new class BUSSIM_ARC [this->ARCS];
        for( int i=0 ; i < this->ARCS ; i++ )
        {
            AUX_ARC[i] = this->ARC[i];
            this->ARC[i].first_block = nullptr;
            this->ARC[i].first_line = nullptr;
            this->ARC[i].first_waiting_vehicle = nullptr;
            this->ARC[i].last_block = nullptr;
            this->ARC[i].last_line = nullptr;
            this->ARC[i].last_waiting_vehicle = nullptr;
        }

        // delete the original arc array
        delete[] this->ARC;
        this->ARC = new class BUSSIM_ARC[this->ARCS+ADD_ARCS];
        for( int i=0 ; i < this->ARCS ; i++ )
        {
            this->ARC[i] = AUX_ARC[i];
        }
        for( int i=this->ARCS ; i < this->ARCS+ADD_ARCS ; i++ )
        {
            this->ARC[i] = _ADD_ARC[i - this->ARCS];
            //this->ARC[i].ID = this->ARC[i].ID+this->ARCS;
            this->ARC[i].ROUTING_TYPE = BUSSIM_RT_WALK;
        }
        this->ARCS = this->ARCS+ADD_ARCS;
        delete[] AUX_ARC;
    }
    //exit(0);
}

void BUSSIM_NETWORK::compile_service_matrix(void)
{
    for( int l=0 ; l < this->EXT_LINES ; l++ )
    {
        if( this->LINE[l].type == BUSSIM_REGULARLINE )
        {


        for( int a=0 ; a < this->LINE[l].ARCS ; a++)
        {
            if( a < this->LINE[l].ARCS-1 )
            {

                for( int b=a+1 ; b < this->LINE[l].ARCS ; b++ )
                {
                    if( b < this->LINE[l].ARCS-1 )
                    {
                        this->SERVICE[l][this->LINE[l].USED_ARC[a].orig_node][this->LINE[l].USED_ARC[b].orig_node] = 1;
                        //std::cout << l << "A: "<< this->LINE[l].USED_ARC[a].ID << ": " << this->LINE[l].USED_ARC[a].orig_node << "->" << this->LINE[l].USED_ARC[b].orig_node << std::endl;
                    }
                    else
                    {
                        this->SERVICE[l][this->LINE[l].USED_ARC[a].orig_node][this->LINE[l].USED_ARC[b].orig_node] = 1;
                        this->SERVICE[l][this->LINE[l].USED_ARC[a].orig_node][this->LINE[l].USED_ARC[b].dest_node] = 1;
                        //std::cout << l << "B: "<< this->LINE[l].USED_ARC[a].ID << ": " << this->LINE[l].USED_ARC[a].orig_node << "->" << this->LINE[l].USED_ARC[b].orig_node << std::endl;
                        //std::cout << l << "B: "<< this->LINE[l].USED_ARC[a].ID << ": " << this->LINE[l].USED_ARC[a].orig_node << "->" << this->LINE[l].USED_ARC[b].dest_node << std::endl;
                    }
                }
            }
            else
            {
                // checked
                this->SERVICE[l][this->LINE[l].USED_ARC[a].orig_node][this->LINE[l].USED_ARC[a].dest_node] = 1;
                //std::cout << l << "C: "<< this->LINE[l].USED_ARC[a].ID << ": " << this->LINE[l].USED_ARC[a].orig_node << "->" << this->LINE[l].USED_ARC[a].dest_node << std::endl;
            }
        }

        }

    }

}

// specifying the nodelist objects

BUSSIM_NODELIST_ENTRY::BUSSIM_NODELIST_ENTRY()
{
    this->REF2POINT = nullptr;
    this->previous = nullptr;
    this->next = nullptr;
    this->SortValue = (double)( rand() % 10000 ) / 10000.0;
}

BUSSIM_NODELIST_ENTRY::~BUSSIM_NODELIST_ENTRY()
{
    this->REF2POINT = nullptr;
    this->previous = nullptr;
    this->next = nullptr;
}

BUSSIM_NODELIST_ENTRY::BUSSIM_NODELIST_ENTRY(class BUSSIM_POINT *_REF2POINT)
{
    this->REF2POINT = _REF2POINT;
    this->previous = nullptr;
    this->next = nullptr;
    this->SortValue = (double)( rand() % 10000 ) / 10000.0;
}

BUSSIM_NODELIST_ENTRY::BUSSIM_NODELIST_ENTRY(class BUSSIM_POINT *_REF2POINT, double _SortValue)
{
    this->REF2POINT = _REF2POINT;
    this->previous = nullptr;
    this->next = nullptr;
    this->SortValue = _SortValue;
}

// define the nodelist

BUSSIM_NODELIST::BUSSIM_NODELIST()
{
    this->DESC = "undefined";
    this->NODES = 0;
    this->REF2FIRST = nullptr;
    this->REF2LAST = nullptr;
}

BUSSIM_NODELIST::BUSSIM_NODELIST(std::string _DESC)
{
    this->DESC = _DESC;
    this->NODES = 0;
    this->REF2FIRST = nullptr;
    this->REF2LAST = nullptr;
}

BUSSIM_NODELIST::~BUSSIM_NODELIST()
{
}

void BUSSIM_NODELIST::append_node(class BUSSIM_NODELIST_ENTRY *_NEW_NODELIST_ENTRY)
{
    if( this->REF2FIRST != nullptr )
    {
        // already stored noderefs available

        _NEW_NODELIST_ENTRY->next = nullptr;
        _NEW_NODELIST_ENTRY->previous = this->REF2LAST;
        _NEW_NODELIST_ENTRY->previous->next = _NEW_NODELIST_ENTRY;

        this->REF2LAST = _NEW_NODELIST_ENTRY;

        this->NODES++;
    }
    else
    {
        // nothing stored so far
        _NEW_NODELIST_ENTRY->next = nullptr;
        _NEW_NODELIST_ENTRY->previous = nullptr;

        this->REF2FIRST = _NEW_NODELIST_ENTRY;
        this->REF2LAST = _NEW_NODELIST_ENTRY;

        this->NODES++;
    }
}

void BUSSIM_NODELIST::print_to_shell(void)
{
    class BUSSIM_NODELIST_ENTRY *Ptr;
    Ptr = this->REF2FIRST;
    std::cout << "this is nodelist " << this->DESC << "(" << this->NODES<< " stored nodes)" << std::endl;
    while( Ptr != nullptr )
    {
        std::cout << "\t" << Ptr->REF2POINT->ID << "("<< Ptr->SortValue<<")";
        Ptr = Ptr->next;
    }
    std::cout << std::endl;
}

long BUSSIM_NODELIST::get_random_node_id(void)
{
    long ret_val = -1;
    if( this->NODES > 0 )
    {
        long Pos;
        Pos = rand() % this->NODES;
        class BUSSIM_NODELIST_ENTRY *Ptr;
        Ptr = this->REF2FIRST;
        for( long i=0 ; i < Pos ; i++ )
            Ptr = Ptr->next;
        ret_val = Ptr->REF2POINT->ID;
    }

    return( ret_val );
}

void BUSSIM_NODELIST::get_random_nodes_id(long _NUM, long *_NUMARRAY)
{

}

void BUSSIM_NETWORK::update_walking_paxes(void)
{
    // update position of all walking pax

    //std::cout << "THIS IS THE LIST OF ALL CURRENTLY WALKING PAXES" << std::endl;
    //this->PAX_WALKING->print();
    class BUSSIM_PAX_LISTITEM *Ptr;
    Ptr = this->PAX_WALKING->get_first_pax_item();
    while( Ptr != nullptr )
    {

        /*
        std::cout << "we have pax " << Ptr->get_ref2pax()->ID << std::endl;
        if( Ptr->get_ref2pax()->Ref2CurrentTPSeg->get_ref2point() != nullptr )
        {
            std::cout << "at platform node " << Ptr->get_ref2pax()->Ref2CurrentTPSeg->get_ref2point()->ID;
            std::cout << std::endl;
        }
        if( Ptr->get_ref2pax()->Ref2CurrentTPSeg->get_ref2arc() != nullptr )
        {
            std::cout << "on arc " << Ptr->get_ref2pax()->Ref2CurrentTPSeg->get_ref2arc()->ID;
            std::cout << ": from " << Ptr->get_ref2pax()->Ref2CurrentTPSeg->get_ref2arc()->orig_node
                        << " to " << Ptr->get_ref2pax()->Ref2CurrentTPSeg->get_ref2arc()->dest_node;
            std::cout << std::endl;
        }
        */
        //Ptr->get_ref2pax()->Ref2CurrentTPSeg->get_ref2point()->print_waiting_lists(0,0,this);
        //Ptr->get_ref2pax()->TravelPath->print_on_console(this);
        Ptr->get_ref2pax()->update_pax_pos_walking(this);
        //std::cout << "update done!" << std::endl;
        Ptr = Ptr->get_next();
    }
}

void BUSSIM_NETWORK::fill_passenger_window(void)
{

    double y_offset = 0.05;

    // display the scenario headline

    glColor4f(1.00f , 1.00f, 1.00f, 0.0f);
    glRasterPos2d(-0.95, 0.95- y_offset);

    char S[256];
    sprintf(S,"progress of the scenario with %d paxes",this->PAXS);
    unsigned char* ubufabcde = reinterpret_cast<unsigned char*>(S);
    ubufabcde = reinterpret_cast<unsigned char*>(S);
    glutBitmapString(GLUT_BITMAP_HELVETICA_18, ubufabcde);


    // draw the progress bar of the already known passengers

    y_offset += 0.05;

    glRasterPos2d(-0.95, 0.95- y_offset);

    glColor4f(0.56f , 0.50f, 0.44f, 0.0f);

    glBegin(GL_POLYGON);
        glVertex2f(-0.95, 0.975- y_offset);
        glVertex2f(+0.95, 0.975- y_offset);
        glVertex2f(+0.95, 0.925- y_offset);
        glVertex2f(-0.95, 0.925- y_offset);
        glVertex2f(-0.95, 0.975- y_offset);
    glEnd();

    double PERCENT;
    if( this->PAXS > 0 )
    {
        PERCENT = (double)(this->PAX_IN->get_pax_num()+this->PAX_ARRIVED->get_pax_num()) / (double)(this->PAXS);
    }
    else
    {
        PERCENT = 1.00;
    }

    glColor4f(0.22f , 0.11f, 0.00f, 0.0f);
    glLineWidth(10.0f);
    glBegin(GL_LINES);
        glVertex2f(-0.95 + 2*0.95*PERCENT, 0.975- y_offset);
        glVertex2f(-0.95 + 2*0.95*PERCENT, 0.925- y_offset);
    glEnd();
    glLineWidth(1.0f);

    glColor4f(0.33f , 0.55f, 0.33f, 0.0f);
    glBegin(GL_POLYGON);
        glVertex2f(-0.95, 0.975- y_offset);
        glVertex2f(-0.95 + 2*0.95*PERCENT, 0.975- y_offset);
        glVertex2f(-0.95 + 2*0.95*PERCENT, 0.925- y_offset);
        glVertex2f(-0.95, 0.925- y_offset);
        glVertex2f(-0.95, 0.975- y_offset);
    glEnd();

    glColor4f(1.0f, 1.0f, 1.0f, 0.0f);
    glRasterPos2d(-0.90, 0.94 - y_offset);

    sprintf(S,"%ld / %d pax shown up",this->PAX_IN->get_pax_num()+this->PAX_ARRIVED->get_pax_num(),this->PAXS);

    unsigned char* ubufg = reinterpret_cast<unsigned char*>(S);
    ubufg = reinterpret_cast<unsigned char*>(S);
    glutBitmapString(GLUT_BITMAP_HELVETICA_18, ubufg);

    // draw the progress bar of the already arrived passengers

    y_offset += 0.075;

    glRasterPos2d(-0.95, 0.95- y_offset);

    glColor4f(0.56f , 0.50f, 0.44f, 0.0f);

    glBegin(GL_POLYGON);
        glVertex2f(-0.95, 0.975- y_offset);
        glVertex2f(+0.95, 0.975- y_offset);
        glVertex2f(+0.95, 0.925- y_offset);
        glVertex2f(-0.95, 0.925- y_offset);
        glVertex2f(-0.95, 0.975- y_offset);
    glEnd();

    if( (this->PAX_IN->get_pax_num()+this->PAX_ARRIVED->get_pax_num()) > 0 )
    {
        PERCENT = (double)(this->PAX_ARRIVED->get_pax_num()) / (double)(this->PAX_IN->get_pax_num()+this->PAX_ARRIVED->get_pax_num());
    }
    else
    {
        PERCENT = 0.00;
    }

    glColor4f(0.22f , 0.11f, 0.00f, 0.0f);
    glLineWidth(10.0f);
    glBegin(GL_LINES);
        glVertex2f(-0.95 + 2*0.95*PERCENT, 0.975- y_offset);
        glVertex2f(-0.95 + 2*0.95*PERCENT, 0.925- y_offset);
    glEnd();
    glLineWidth(1.0f);

    glColor4f(0.33f , 0.55f, 0.33f, 0.0f);
    glBegin(GL_POLYGON);
        glVertex2f(-0.95, 0.975- y_offset);
        glVertex2f(-0.95 + 2*0.95*PERCENT, 0.975- y_offset);
        glVertex2f(-0.95 + 2*0.95*PERCENT, 0.925- y_offset);
        glVertex2f(-0.95, 0.925- y_offset);
        glVertex2f(-0.95, 0.975- y_offset);
    glEnd();

    glColor4f(1.0f, 1.0f, 1.0f, 0.0f);
    glRasterPos2d(-0.90, 0.94 - y_offset);

    sprintf(S,"%ld / %ld pax have reached their destination",this->PAX_ARRIVED->get_pax_num(), this->PAX_IN->get_pax_num()+this->PAX_ARRIVED->get_pax_num());

    unsigned char* ubufgh = reinterpret_cast<unsigned char*>(S);
    ubufgh = reinterpret_cast<unsigned char*>(S);
    glutBitmapString(GLUT_BITMAP_HELVETICA_18, ubufgh);

    //y_offset += 0.1;

    //glRasterPos2d(-0.95, 0.95- y_offset);

    // print the headline of the current in system pax section

    y_offset += 0.1;

    glRasterPos2d(-0.95, 0.95- y_offset);

    glColor4f(0.33f , 0.55f, 0.33f, 0.0f);

    sprintf(S,"Status of the %ld pax currently in the network",this->PAX_IN->get_pax_num());

    unsigned char* ubufabcd = reinterpret_cast<unsigned char*>(S);
    ubufabcd = reinterpret_cast<unsigned char*>(S);
    glutBitmapString(GLUT_BITMAP_HELVETICA_18, ubufabcd);

    // print the percentage bar for the walking pax
    y_offset += 0.05;

    glRasterPos2d(-0.95, 0.95- y_offset);

    glColor4f(0.56f , 0.50f, 0.44f, 0.0f);

    glBegin(GL_POLYGON);
        glVertex2f(-0.95, 0.975- y_offset);
        glVertex2f(+0.95, 0.975- y_offset);
        glVertex2f(+0.95, 0.925- y_offset);
        glVertex2f(-0.95, 0.925- y_offset);
        glVertex2f(-0.95, 0.975- y_offset);
    glEnd();

    if( this->PAX_IN->get_pax_num() > 0 )
    {
        PERCENT = (double)(this->PAX_WALKING->get_pax_num()) / (double)(this->PAX_IN->get_pax_num());
    }
    else
    {
        PERCENT = 0.00;
    }

    glColor4f(0.22f , 0.11f, 0.00f, 0.0f);
    glLineWidth(10.0f);
    glBegin(GL_LINES);
        glVertex2f(-0.95 + 2*0.95*PERCENT, 0.975- y_offset);
        glVertex2f(-0.95 + 2*0.95*PERCENT, 0.925- y_offset);
    glEnd();
    glLineWidth(1.0f);

    glColor4f(0.33f , 0.55f, 0.33f, 0.0f);

    glBegin(GL_POLYGON);
        glVertex2f(-0.95, 0.975- y_offset);
        glVertex2f(-0.95 + 2*0.95*PERCENT, 0.975- y_offset);
        glVertex2f(-0.95 + 2*0.95*PERCENT, 0.925- y_offset);
        glVertex2f(-0.95, 0.925- y_offset);
        glVertex2f(-0.95, 0.975- y_offset);
    glEnd();

    glColor4f(1.0f, 1.0f, 1.0f, 0.0f);
    glRasterPos2d(-0.90, 0.94 - y_offset);

    sprintf(S,"%ld pax in walking transfer",this->PAX_WALKING->get_pax_num());

    unsigned char* ubuf = reinterpret_cast<unsigned char*>(S);
    ubuf = reinterpret_cast<unsigned char*>(S);
    glutBitmapString(GLUT_BITMAP_HELVETICA_18, ubuf);

    // print the percentage bar for the waiting pax
    y_offset += 0.075;

    glRasterPos2d(-0.95, 0.95- y_offset);

    //glColor4f(0.33f , 0.55f, 0.33f, 0.0f);
    glColor4f(0.56f , 0.50f, 0.44f, 0.0f);

    glBegin(GL_POLYGON);
        glVertex2f(-0.95, 0.975- y_offset);
        glVertex2f(+0.95, 0.975- y_offset);
        glVertex2f(+0.95, 0.925- y_offset);
        glVertex2f(-0.95, 0.925- y_offset);
        glVertex2f(-0.95, 0.975- y_offset);
    glEnd();

    double PERCENT_WAIT;
    if( this->PAX_IN->get_pax_num() > 0 )
    {
        PERCENT_WAIT = (double)(this->NUM_PAX_IN_WAITING) / (double)(this->PAX_IN->get_pax_num());
    }
    else
    {
        PERCENT_WAIT = 0.00;
    }

    glColor4f(0.22f , 0.11f, 0.00f, 0.0f);
    glLineWidth(10.0f);
    glBegin(GL_LINES);
        glVertex2f(-0.95 + 2*0.95*PERCENT_WAIT, 0.975- y_offset);
        glVertex2f(-0.95 + 2*0.95*PERCENT_WAIT, 0.925- y_offset);
    glEnd();
    glLineWidth(1.0f);

    glColor4f(0.33f , 0.55f, 0.33f, 0.0f);

    glBegin(GL_POLYGON);
        glVertex2f(-0.95, 0.975- y_offset);
        glVertex2f(-0.95 + 2*0.95*PERCENT_WAIT, 0.975- y_offset);
        glVertex2f(-0.95 + 2*0.95*PERCENT_WAIT, 0.925- y_offset);
        glVertex2f(-0.95, 0.925- y_offset);
        glVertex2f(-0.95, 0.975- y_offset);
    glEnd();

    glColor4f(1.0f, 1.0f, 1.0f, 0.0f);
    glRasterPos2d(-0.90, 0.94 - y_offset);

    sprintf(S,"%ld pax waiting at platforms",this->NUM_PAX_IN_WAITING);

    unsigned char* ubufAB = reinterpret_cast<unsigned char*>(S);
    ubufAB = reinterpret_cast<unsigned char*>(S);
    glutBitmapString(GLUT_BITMAP_HELVETICA_18, ubufAB);

    // print the percentage bar for the riding pax
    y_offset += 0.075;

    glRasterPos2d(-0.95, 0.95- y_offset);

    glColor4f(0.56f , 0.50f, 0.44f, 0.0f);

    glBegin(GL_POLYGON);
        glVertex2f(-0.95, 0.975- y_offset);
        glVertex2f(+0.95, 0.975- y_offset);
        glVertex2f(+0.95, 0.925- y_offset);
        glVertex2f(-0.95, 0.925- y_offset);
        glVertex2f(-0.95, 0.975- y_offset);
    glEnd();

    double PERCENT_RIDING;
    if( this->PAX_IN->get_pax_num() > 0 )
    {
        PERCENT_RIDING = 1.0 - PERCENT - PERCENT_WAIT;
    }
    else
    {
        PERCENT_RIDING = 0.00;
    }

    glColor4f(0.22f , 0.11f, 0.00f, 0.0f);
    glLineWidth(10.0f);
    glBegin(GL_LINES);
        glVertex2f(-0.95 + 2*0.95*PERCENT_RIDING, 0.975- y_offset);
        glVertex2f(-0.95 + 2*0.95*PERCENT_RIDING, 0.925- y_offset);
    glEnd();
    glLineWidth(1.0f);

    glColor4f(0.33f , 0.55f, 0.33f, 0.0f);

    glBegin(GL_POLYGON);
        glVertex2f(-0.95, 0.975- y_offset);
        glVertex2f(-0.95 + 2*0.95*PERCENT_RIDING, 0.975- y_offset);
        glVertex2f(-0.95 + 2*0.95*PERCENT_RIDING, 0.925- y_offset);
        glVertex2f(-0.95, 0.925- y_offset);
        glVertex2f(-0.95, 0.975- y_offset);
    glEnd();

    glColor4f(1.0f, 1.0f, 1.0f, 0.0f);
    glRasterPos2d(-0.90, 0.94 - y_offset);

    sprintf(S,"%ld pax riding in a vehicle",this->PAX_IN->get_pax_num() - this->NUM_PAX_IN_WAITING - this->PAX_WALKING->get_pax_num());

    unsigned char* ubufABC = reinterpret_cast<unsigned char*>(S);
    ubufABC = reinterpret_cast<unsigned char*>(S);
    glutBitmapString(GLUT_BITMAP_HELVETICA_18, ubufABC);

    glColor4f(1.0f, 0.0f, 0.0f, 0.0f);
    glBegin(GL_LINES);
        glVertex2f(-0.95 + 2*0.95*PERCENT, 0.95- y_offset + 0.15);
        glVertex2f(-0.95 + 2*0.95*PERCENT_WAIT, 0.95- y_offset + 0.075);
        glVertex2f(-0.95 + 2*0.95*PERCENT_WAIT, 0.95- y_offset + 0.075);
        glVertex2f(-0.95 + 2*0.95*PERCENT_RIDING, 0.95- y_offset);
    glEnd();
}

void BUSSIM_NETWORK::specify_platform_positions_in_lines(void)
{
    // this procedure sets the array BUSSIM_POINT::PlatformPosInLine
    if( this->EXT_LINES > 0)
    {
        // first, reserve the memory and initialize the array fields
        for( long i=0 ; i < this->POINTS ; i++ )
        {
            if( this->PT[i].INFRASTRUCTURE < 1 )
            {
                this->PT[i].PlatformPosInLine = new long [this->EXT_LINES];
                for( int l=0 ; l < this->EXT_LINES ; l++ )
                    this->PT[i].PlatformPosInLine[l] = -1;
            }
        }

        // second, run through the lines and identify the positions
        int PlatFormCounter = 0;
        for( int l=0 ; l < this->EXT_LINES ; l++ )
        {
            PlatFormCounter = 0;
            for( int a=0 ; a < this->LINE[l].ARCS ; a++ )
            {
                if( a < (this->LINE[l].ARCS -1 ) )
                {
                    // not the last arc, inspect origin node
                    if( this->PT[this->LINE[l].USED_ARC[a].orig_node].INFRASTRUCTURE < 1 )
                    {
                        this->PT[this->LINE[l].USED_ARC[a].orig_node].PlatformPosInLine[l] = PlatFormCounter;
                        PlatFormCounter++;
                    }
                }
                else
                {
                    // last arc, inspect both nodes
                    if( this->PT[this->LINE[l].USED_ARC[a].orig_node].INFRASTRUCTURE < 1 )
                    {
                        this->PT[this->LINE[l].USED_ARC[a].orig_node].PlatformPosInLine[l] = PlatFormCounter;
                        PlatFormCounter++;
                    }
                    if( this->PT[this->LINE[l].USED_ARC[a].dest_node].INFRASTRUCTURE < 1 )
                    {
                        this->PT[this->LINE[l].USED_ARC[a].dest_node].PlatformPosInLine[l] = PlatFormCounter;
                        PlatFormCounter++;
                    }
                }
            }

            /*
            if( l==12 )
            {
                for( long i=0 ; i < this->POINTS ; i++ )
                {
                    if( this->PT[i].INFRASTRUCTURE < 1 )
                    {
                        if( this->PT[i].PlatformPosInLine[l] >= 0 )
                        std::cout << "PT=" << this->PT[i].ID << " POS=" << this->PT[i].PlatformPosInLine[l] << std::endl;
                    }
                }
                exit(0);
            }
            */
        }
    }

}

void BUSSIM_NETWORK::specify_vehicle_activation_times(void)
{
    // set the vehicle activation times
    this->VEHICLE[0].set_activation_time(0);
    this->VEHICLE[1].set_activation_time(32);
}

void BUSSIM_NETWORK::specify_automatic_terminus_labels_of_lines(void)
{
    for( int l=0 ; l < this->EXT_LINES ; l++ )
        this->LINE[l].set_automatic_desc(this);
}
